#!/usr/bin/perl

#=== File Prolog =============================================================
#    Distributed.net Personal Proxy Statistics Log Analyzer
#    Copyright (C) 1998-2002 GNU General Public License
#    http://ppstats.sourceforge.net/
#
#--- Contents ----------------------------------------------------------------
#    Perl interface routines for reading Distributed.Net personal proxy log 
#    files, connecting through Pg module to a PostgreSQL database, creating
#    HTML based statistics, and creating image based charts using GNUplot.
#
#--- Description -------------------------------------------------------------
#    A complete description of Personal Proxy Statistics can be found at
#    http://ppstats.sourceforge.net/
#
#    See Readme.txt for a description and system requirements for this script.
#    See Install.txt for a step by step procedure regarding installation.
#    See License.txt for licensing terms of the GNU General Public License.
#    See ChangeLog.txt for revision history information.
#    See Todo.txt for a listing of outstanding issues.
#
#    Edit the ppstats.ini file to customize this script for your team.
#    There should not be a need to modify this file directly (see Notes).
#
#--- Notes -------------------------------------------------------------------
#    If this script is run under Windows 9x/NT using the ActiveState Perl
#    distribution, please comment out the "setpriority(...)" command located
#    near the beginning of this script. "setpriority" is not yet supported
#    under the ActiveState Perl distribution and will cause the script to
#    fail when executed.
#
#--- Warning -----------------------------------------------------------------
#    This program is free software; you can redistribute it and/or
#    modify it under the terms of the GNU General Public License
#    as published by the Free Software Foundation; either version 2
#    of the License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#=== End File Prolog =========================================================


#--- Priority ----------------------------------------------------------------
# Set script to nice priority. Comment out the following line for use on NT.
# For NT users, try Thomas Minor's program for setting script priority at
# http://ppstats.sourceforge.net/pub/ppstats/contrib/runit-idle.zip
setpriority(0,0,20);


#--- Use Declarations --------------------------------------------------------
use strict;        #--Declare all variables.
use Time::Local;   #--Use the Time::Local module, timegm() function;
use DBI;           #--Standard Perl database interface.
use DBD::mysql;    #--Database driver for MySQL
#use DBD::Pg;       #--Database driver for PostgreSQL.

#--- Global Variables --------------------------------------------------------
my %SystemPrefs;         #--Internal system variables.
my %UserPrefs;           #--External user defined preferences.
my @OSID;                #--Operating system list.
my @OSIconID;            #--Operating system icon lookup.
my @CPUID;               #--CPU type list.
my @CPUIconID;           #--CPU type icon lookup.
my %CountryID;           #--Country code lookup.
my @Months;              #--Months of the year.
my @Days;                #--Days of the week.
my %TimeZones;           #--Available time zones.
my @AvailableUserPrefs;  #--Recognized user preferences.
my %opt;                 #--Command line switches.
my @optArgs;             #--Command line switch arguments.
my %Emails;              #--Email list.
my %Hosts;               #--IP/Host list.
my @UserTS;              #--Timestamp in user defined timezone;
my @GMTTS;               #--Timestamp in GMT timezone.
my @Average;		 #--Array to calculate average blocks
my @pageblocks;          #--Array for blocks per page

# I'm sorry for this....
#my $true="t";	# PostgreSQL
my $true="1";	# MySQL
my $false="0";	# MySQL


#--- Method Prolog -----------------------------------------------------------
#  Name: Main Procedure
#
#  Parameters: None.
#
#  Return Value: None.
#
#  Description:
#    Main procedure starts here.
#
#--- End Method Prolog -------------------------------------------------------
umask("002");
&SystemInit();

#--Process command line arguements and the ppstats.ini file.
&ProcessCommandLine();
&ProcessUserPrefs();
&SanityCheck();

#--Establish a connect to the database.
&ConnectDB($SystemPrefs{"dbname"});

#--All personal proxy related dates are in GMT format.
#&SetTimeZoneDB("GMT");

#--All dates should be displayed in YYYY-MM-DD HH:MM:SS+Timezone format.
#&SetDateStyleDB("ISO");

#--Load into memory all known hosts and email addresses from the database.
&LoadHosts();
&LoadEmails();

#--Import data from proxy log files into the database.
&ImportLogfiles();

#--Set timestamp details.
&SetTimestamp();

#--Compress and reindex the database. This greatly speeds queries.
&VacuumDB();

#--Reset time zone to the user defined setting.
#&SetTimeZoneDB($UserPrefs{"timezone"});

#--Create HTML files.
&create_html();

#--Disconnect from the database.
&DisconnectDB();

exit;

#--- Method Prolog -----------------------------------------------------------
#  Name: create_html
#
#  Parameters:   None.
#
#  Return Value: None.
#
#  Description:
#    Calling the procedures that write the html-pages for each active contest
#
#--- End Method Prolog -------------------------------------------------------
sub create_html
{
    my $Contest;    
    
    if ($UserPrefs{"do_html"} == 1)
    {

	#--Lookup hostnames from IP addresses.
	&ResolveHosts();

	#--Start writing html for each contest
        for ($Contest = 1; $Contest <= $SystemPrefs{"projects"}; $Contest++)
	{
	    if ($UserPrefs{"do_" . $SystemPrefs{"contestsuffix"}[$Contest] } == $true)
	    {
		&write_css($Contest);
    	        &write_index($Contest); 
    		&write_exec($Contest);
		&write_byhost($Contest);
		&write_byemail($Contest);
		&write_byos($Contest);
		&write_bycpu($Contest);
		&write_byversion($Contest);
		&write_byfull($Contest);
		&write_bydomain($Contest);
		&write_byhour($Contest);
		if ($UserPrefs{"disable_bydate"} != $true)
		{
		    &write_bydate($Contest);
		}
		if ($UserPrefs{"disable_byweek"} != $true)
		{
		    &write_byweek($Contest);
		}
		if ($UserPrefs{"disable_bymonth"} != $true)
		{
		    &write_bymonth($Contest);
		}
		if ($UserPrefs{"disable_byyear"} != $true)
		{
		    &write_byyear($Contest);
		}
		&write_bydayofweek($Contest);
	    }
	}
    }
} # create_html


#--- Method Prolog -----------------------------------------------------------
#  Name: SystemInit
#
#  Parameters: None.
#
#  Return Value: None.
#
#  Description:
#    Initialize internal script variables. Do not edit these settings.
#
#--- End Method Prolog -------------------------------------------------------
sub SystemInit
{
    my ($year, $month, $mday, $hour, $min, $sec, $wday);
    my $y;
    
    $SystemPrefs{"projects"} = 5; # How many Projects are supported

    #--Contest specific information.
    $SystemPrefs{"contestname"}[1] = "Project RC5-64";
    $SystemPrefs{"contestsuffix"}[1] = "rc5";
    $SystemPrefs{"contesthome"}[1] = "rc5";
    $SystemPrefs{"contestsize"}[1] = 64;
    $SystemPrefs{"contestblockbase"}[1] = 28; # 2**28
    $SystemPrefs{"contestblockweight"}[1] = 1; # 1 * 2**28
    $SystemPrefs{"contestheaderblocktype"}[1] = "Block";
    $SystemPrefs{"contestblocktype"}[1] = "block"; # "s" will be added if needed in functions
    $SystemPrefs{"contestblockmeasure"}[1] = "key"; # "s" will be added if needed in functions
    $SystemPrefs{"contestheaderblockmeasure"}[1] = "Key"; # "s" will be added if needed in functions
    $SystemPrefs{"contestblocktypeabbreviation"}[1] = "k";
    $SystemPrefs{"contestdivide"}[1] = 322; # (Seconds per day / (keys in one workunit / 1M)) = ((24*60*60) / (2^28 / 1000000)) =~ 322
    $SystemPrefs{"contestfactor"}[1] = 1000; # Used to make the GNUplot output more readable (values and x/y-description)
    $SystemPrefs{"contestwutable"}[1] = "TBL_WU_RC564"; 
    $SystemPrefs{"conteststats"}[1] = "?project_id=5";
   
    $SystemPrefs{"contestname"}[2] = "Project DES";
    $SystemPrefs{"contestsuffix"}[2] = "des";
    $SystemPrefs{"contesthome"}[2] = "des";
    $SystemPrefs{"contestsize"}[2] = 64;
    $SystemPrefs{"contestblockbase"}[2] = 28; # 2**28
    $SystemPrefs{"contestblockweight"}[2] = 2; # 2 * 2**28
    $SystemPrefs{"contestheaderblocktype"}[2] = "Block";
    $SystemPrefs{"contestblocktype"}[2] = "block";
    $SystemPrefs{"contestblockmeasure"}[2] = "keys";
    $SystemPrefs{"contestheaderblockmeasure"}[4] = "Key";
    $SystemPrefs{"contestblocktypeabbreviation"}[3] = "k";
    $SystemPrefs{"contestdivide"}[2] = 322; # (Seconds per day / (keys in one workunit / 1M)) = ((24*60*60) / (2^28 / 1000000)) =~ 322
    $SystemPrefs{"contestfactor"}[2] = 1000; 
    $SystemPrefs{"contestwutable"}[2] = "TBL_WU_DES";
    $SystemPrefs{"conteststats"}[2] = "?project_id=na";

    $SystemPrefs{"contestname"}[3] = "Project CSC";
    $SystemPrefs{"contestsuffix"}[3] = "csc";
    $SystemPrefs{"contesthome"}[3] = "csc";
    $SystemPrefs{"contestsize"}[3] = 56;
    $SystemPrefs{"contestblockbase"}[3] = 28; # 2**28
    $SystemPrefs{"contestblockweight"}[3] = 1; # 1 * 2**28
    $SystemPrefs{"contestheaderblocktype"}[3] = "Block";
    $SystemPrefs{"contestblocktype"}[3] = "block";
    $SystemPrefs{"contestblockmeasure"}[3] = "keys";
    $SystemPrefs{"contestheaderblockmeasure"}[3] = "Key";
    $SystemPrefs{"contestblocktypeabbreviation"}[3] = "k";
    $SystemPrefs{"contestdivide"}[3] = 322; # (Seconds per day / (keys in one workunit / 1M)) = ((24*60*60) / (2^28 / 1000000)) =~ 322
    $SystemPrefs{"contestfactor"}[3] = 1000; 
    $SystemPrefs{"contestwutable"}[3] = "TBL_WU_CSC";
    $SystemPrefs{"conteststats"}[3] = "?project_id=na";

    $SystemPrefs{"contestname"}[4] = "Project OGR-25";
    $SystemPrefs{"contestsuffix"}[4] = "ogr";
    $SystemPrefs{"contesthome"}[4] = "ogr";
    $SystemPrefs{"contestsize"}[4] = 1; # (0)
    $SystemPrefs{"contestblockbase"}[4] = 0; # 2**0
    $SystemPrefs{"contestblockweight"}[4] = 1; # 1 * 1
    $SystemPrefs{"contestheaderblocktype"}[4] = "Node";
    $SystemPrefs{"contestblocktype"}[4] = "node";
    $SystemPrefs{"contestheaderblockmeasure"}[4] = "Node";
    $SystemPrefs{"contestblockmeasure"}[4] = "node";
    $SystemPrefs{"contestblocktypeabbreviation"}[4] = "n";
    $SystemPrefs{"contestdivide"}[4] = 86400; # (Seconds per day / (keys in one workunit) = ((24*60*60) / (1)) = 86400
    $SystemPrefs{"contestfactor"}[4] = 0.001; # This is to compensate that all other do everything in kkeys and ogr not... :-( 
    $SystemPrefs{"contestwutable"}[4] = "TBL_WU_OGR";
    $SystemPrefs{"conteststats"}[4] = "?project_id=25";

    $SystemPrefs{"contestname"}[5] = "Project RC5-72";
    $SystemPrefs{"contestsuffix"}[5] = "rc572";
    $SystemPrefs{"contesthome"}[5] = "rc5";
    $SystemPrefs{"contestsize"}[5] = 72;
    $SystemPrefs{"contestblockbase"}[5] = 32; # 2**32
    $SystemPrefs{"contestblockweight"}[5] = 1; # 1 * 2**32
    $SystemPrefs{"contestheaderblocktype"}[5] = "Block";
    $SystemPrefs{"contestblocktype"}[5] = "block"; # "s" will be added if needed in functions
    $SystemPrefs{"contestblockmeasure"}[5] = "key"; # "s" will be added if needed in functions
    $SystemPrefs{"contestheaderblockmeasure"}[5] = "Key"; # "s" will be added if needed in functions
    $SystemPrefs{"contestblocktypeabbreviation"}[5] = "k";
    $SystemPrefs{"contestdivide"}[5] = 20; # (Seconds per day / (keys in one workunit / 1M)) = ((24*60*60) / (2^28 / 1000000)) =~ 20
    $SystemPrefs{"contestfactor"}[5] = 1000; # Used to make the GNUplot output more readable (values and x/y-description)
    $SystemPrefs{"contestwutable"}[5] = "TBL_WU_RC572"; 
    $SystemPrefs{"conteststats"}[5] = "?project_id=8";
   
    $SystemPrefs{"stats-server"} = "http://stats.distributed.net";

    #--Set name, path, version, and debug variables.
    $SystemPrefs{"appname"} = "Personal Proxy Statistics";
    $SystemPrefs{"appver"} = "8.1-alpha-16";
    ($SystemPrefs{"appdir"}, $SystemPrefs{"app"}) = split(m|([^/]+)$|, $0, 2);
    $SystemPrefs{"appini"} = "ppstats.ini";
    $SystemPrefs{"debugenable"} = 0;
    $SystemPrefs{"sqldebugenable"} = 0;

    #--Database related variables.
    $SystemPrefs{"dbname"} = "ppstats";  #--The name of our ppstats database.
    $SystemPrefs{"dbh"} = "";         #--This will be created in ConnectDB.

    #--GNUplot related variables.
    $SystemPrefs{"gnuplotdata"} = "plot.data";


    $SystemPrefs{"pngdefaultwidth"} = "640";
    $SystemPrefs{"pngdefaultheight"} = "480";


    for ($y = 1; $y <= $SystemPrefs{"projects"}; $y++)
    {
	$SystemPrefs{"disable" . $SystemPrefs{"contestsuffix"}[$y] } = $false; 
	$SystemPrefs{"enable" . $SystemPrefs{"contestsuffix"}[$y] } = $false; 
    }

    @Months = 
	( "Jan", "Feb", "Mar", "Apr", "May", "Jun",
	  "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"
	);

    @Days = 
	( "Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", 
	  "Friday", "Saturday"
	);

    #--Available time zones.
    %TimeZones = 
	( "NZDT"   => "+13",     #--New Zealand Daylight Time
	  "IDLE"   => "+12",     #--International Date Line, East
	  "NZST"   => "+12",     #--New Zealand Std Time
	  "NZT"    => "+12",     #--New Zealand Time
	  "AESST"  => "+11",     #--Australia Eastern Summer Std Time
	  "ACSST"  => "+10.5",   #--Central Australia Summer Std Time
	  "CADT"   => "+10.5",   #--Central Australia Daylight Savings Time
	  "SADT"   => "+10.5",   #--South Australian Daylight Time
	  "AEST"   => "+10",     #--Australia Eastern Std Time
	  "EAST"   => "+10",     #--East Australian Std Time
	  "GST"    => "+10",     #--Guam Std Time, USSR Zone 9
	  "LIGT"   => "+10",     #--Melbourne, Australia
	  "ACST"   => "+9.5",    #--Central Australia Std Time
	  "CAST"   => "+9.5",    #--Central Australia Std Time
	  "SAT"    => "+9.5",    #--South Australian Std Time
	  "AWSST"  => "+9",      #--Australia Western Summer Std Time
	  "JST"    => "+9",      #--Japan Std Time,USSR Zone 8
	  "KST"    => "+9",      #--Korea Standard Time
	  "WDT"    => "+9",      #--West Australian Daylight Time
	  "MT"     => "+8.5",    #--Moluccas Time
	  "AWST"   => "+8",      #--Australia Western Std Time
	  "CCT"    => "+8",      #--China Coastal Time
	  "WADT"   => "+8",      #--West Australian Daylight Time
	  "WST"    => "+8",      #--West Australian Std Time
	  "JT"     => "+7.5",    #--Java Time
	  "WAST"   => "+7",      #--West Australian Std Time
	  "IT"     => "+3.5",    #--Iran Time
	  "BT"     => "+3",      #--Baghdad Time
	  "EETDST" => "+3",      #--Eastern Europe Daylight Savings Time
	  "CETDST" => "+2",      #--Central European Daylight Savings Time
	  "EET"    => "+2",      #--Eastern Europe, USSR Zone 1
	  "FWT"    => "+2",      #--French Winter Time
	  "IST"    => "+2",      #--Israel Std Time
	  "MEST"   => "+2",      #--Middle Europe Summer Time
	  "METDST" => "+2",      #--Middle Europe Daylight Time
	  "SST"    => "+2",      #--Swedish Summer Time
	  "BST"    => "+1",      #--British Summer Time
	  "CET"    => "+1",      #--Central European Time
	  "DNT"    => "+1",      #--Dansk Normal Tid
	  "DST"    => "+1",      #--Dansk Standard Time (?)
	  "FST"    => "+1",      #--French Summer Time
	  "MET"    => "+1",      #--Middle Europe Time
	  "MEWT"   => "+1",      #--Middle Europe Winter Time
	  "MEZ"    => "+1",      #--Middle Europe Zone
	  "NOR"    => "+1",      #--Norway Standard Time
	  "SET"    => "+1",      #--Seychelles Time
	  "SWT"    => "+1",      #--Swedish Winter Time
	  "WETDST" => "+1",      #--Western Europe Daylight Savings Time
	  "GMT"    => "+0",      #--Greenwish Mean Time
	  "WET"    => "+0",      #--Western Europe
	  "WAT"    => "-1",      #--West Africa Time
	  "NDT"    => "-2.5",    #--Newfoundland Daylight Time
	  "ADT"    => "-3",      #--Atlantic Daylight Time
	  "NFT"    => "-3.5",    #--Newfoundland Standard Time
	  "NST"    => "-3.5",    #--Newfoundland Standard Time
	  "AST"    => "-4",      #--Atlantic Std Time (Canada)
	  "EDT"    => "-4",      #--Eastern Daylight Time
	  "ZP4"    => "-4",      #--GMT +4 hours
	  "CDT"    => "-5",      #--Central Daylight Time
	  "EST"    => "-5",      #--Eastern Standard Time
	  "ZP5"    => "-5",      #--GMT +5 hours
	  "CST"    => "-6",      #--Central Std Time
	  "MDT"    => "-6",      #--Mountain Daylight Time
	  "ZP6"    => "-6",      #--GMT +6 hours
	  "MST"    => "-7",      #--Mountain Standard Time
	  "PDT"    => "-7",      #--Pacific Daylight Time
	  "PST"    => "-8",      #--Pacific Std Time
	  "YDT"    => "-8",      #--Yukon Daylight Time
	  "HDT"    => "-9",      #--Hawaii/Alaska Daylight Time
	  "YST"    => "-9",      #--Yukon Standard Time
	  "AHST"   => "-10",     #--Alaska-Hawaii Std Time
	  "CAT"    => "-10",     #--Central Alaska Time
	  "NT"     => "-11",     #--Nome Time
	  "IDLW"   => "-12"      #--International Date Line, West
	);

    #--List of required user preferences.
    @AvailableUserPrefs = 
	( 
	  #--Enable/Disable Projects/Pages
	  "do_rc5",
	  "do_des",
	  "do_csc",
	  "do_ogr",
	  "disable_bydate",    
	  "disable_byweek",    
	  "disable_bymonth",    
	  "disable_byyear",    
	
	  #--Team Information
	  "team",
	  "teamhome",
	  "teamid",
	  "uselogo",            
          "logourl",
	  "admin",
	  "adminemail",

          #--Database Options
	  "dbtype",
	  "dbip",
	  "dbport",

	  #--Input Options
	  "do_import",
	  "logsuf",
	  "logdirrc5",
	  "logprerc5", 
	  "logdirdes",
	  "logpredes", 
	  "logdircsc",
	  "logprecsc", 
	  "logdirogr",
	  "logpreogr", 
	  "processziplogs",
	  "logzipsuf", 
	  "logtmpsuf", 
	  "logdecompressor", 
	  "htmldir",
	  "htmlroot", 

	  #--Output Options
	  "do_html",
          "htmlteam",
	  "htmlexec", 
	  "htmlmail", 
	  "htmlhost",
          "htmlos",   
	  "htmlcpu",  
	  "htmlver",  
	  "htmlfull", 
	  "htmldomain", 
	  "htmlhour", 
	  "htmldate", 
	  "htmlweek", 
	  "htmlmonth",  
	  "htmlyear",
	  "htmldayofweek", 
	  "htmlindvemail", 
	  "htmlemailhist", 
	  "htmlhostsbyemail", 
	  "htmlindvhost",  
	  "htmlhosthist",
	  "htmlemailsbyhost", 
	  "css", 
	  "cachedays", 
	  "gnuplot", 
	  "usecounter",   
	  "counter",   
	  "showbyhost",
	  "disableemailranksearch",
	  
	  #--Format Options
	  "chartlinestyle", 
	  "chartwidth", 
	  "chartheight", 
	  "dayaverage", 
	  "movementdays",    
	  "maxrows",  
	  "dayshistory", 
	  "blockhistory",
	  "timezone", 
	  "useheaderbanner", 
	  "usefooterbanner",
	  "conshosts",    
	  "colorlastseen",   
	  "lastseenwarningtime",
	  "lastseencriticaltime", 
	  "lastseenwarningcolor", 
	  "lastseencriticalcolor",
	  "usemetarefresh",  
	  "metarefreshperiod"
	);
}  #--SystemInit


#--- Method Prolog -----------------------------------------------------------
#  Name: ProcessCommandLine
#
#  Parameters: None.
#
#  Return Value: None.
#
#  Description:
#    Process command line arguements supplied by the user.
#
#--- End Method Prolog -------------------------------------------------------
sub ProcessCommandLine
{
    my $Contest;
    #--Parse the command line variables.
    &ParseCommandLine(
	"-help", 0,        #--Display useage information.
	"-rc5",0 ,
	"+rc5",0 ,
	"-des",0 ,
	"+des",0 ,
	"-csc",0 ,
	"+csc",0 ,
	"-ogr",0 ,
	"+ogr",0 ,
	"-sqldebug", 0,     #--Enable debug mode for all SQL-Queries
	"-debug", 0,       #--Enable debug mode.
	"-version", 0     #--Display version information.
    );

    &EnableDebugMessages() if ($opt{"-debug"});
    &EnableSQLDebugMessages() if ($opt{"-sqldebug"});
    &DisplayUsage() if ($opt{"-help"});
    &DisplayVersion() if ($opt{"-version"});
    for ($Contest = 1; $Contest <= $SystemPrefs{"projects"}; $Contest++)
    {
	if ($opt{"-" . $SystemPrefs{"contestsuffix"}[$Contest]}) { $SystemPrefs{"disable" . $SystemPrefs{"contestsuffix"}[$Contest]} = $true; };
	if ($opt{"+" . $SystemPrefs{"contestsuffix"}[$Contest]}) { $SystemPrefs{"enable" . $SystemPrefs{"contestsuffix"}[$Contest]} = $true; };
    }
}  #--ProcessCommandLine


#--- Method Prolog -----------------------------------------------------------
#  Name: ImportLogfiles
#
#  Parameters: None.
#
#  Return Value: None.
#
#  Description:
#    Start &CollectData for all enabled Projects
#
#--- End Method Prolog -------------------------------------------------------
sub ImportLogfiles
{
    my $Contest;

    if ($UserPrefs{"do_import"} == 1) 
    {
	for ($Contest = 1; $Contest <= $SystemPrefs{"projects"}; $Contest++)
	{
	    if ($UserPrefs{"do_" . $SystemPrefs{"contestsuffix"}[$Contest] } == $true)
	    {
	        &CollectData($Contest);
	    }
	}
    }
    else
    {
	&Debugln("Logfile import disabled");
    }
}


#--- Method Prolog -----------------------------------------------------------
#  Name: ProcessUserPrefs
#
#  Parameters: None.
#
#  Return Value: None.
#
#  Description:
#    Read user defined values from the initialization file.
#
#--- End Method Prolog -------------------------------------------------------
sub ProcessUserPrefs
{
    my ($entry, $var, $value);
    my $Contest;

    #--Read in user initialization file.
    &Debugln("Reading initialization file " . $SystemPrefs{"appdir"} . $SystemPrefs{"appini"} . " ...");
    open(INIFILE, $SystemPrefs{"appdir"} . $SystemPrefs{"appini"}) or die $SystemPrefs{"appname"} . ": Cannot open " . $SystemPrefs{"appdir"} . $SystemPrefs{"appini"} . " ($!)\n";

    while (defined($entry = <INIFILE>)) {
	chomp($entry);               #--no newline
	$entry =~ s/#.*//;           #--no comments
	$entry =~ s/^\s+//;          #--no leading white
	$entry =~ s/\s+$//;          #--no trailing white
	next unless length($entry);  #--anything left?
	($var, $value) = split(/\s*=\s*/, $entry, 2);
	$value =~ s/\"//g;           #--remove "'s
	$var =~ tr/A-Z/a-z/;	     #--convert to lowercase
	$UserPrefs{$var} = $value    #--set var to value
    }

    close INIFILE;

    if (!defined($UserPrefs{"gnuplotfile"})) 
    {
     die "Personal Proxy Statistics : GNUPlotfile not defined (needed since PNG change)\n";     
    }
    if ($UserPrefs{"imagetype"} == 1) 
    {
     $UserPrefs{"imageext"} = "png";
     $UserPrefs{"gnuplotoutput"} = $UserPrefs{"gnuplotfile"} . "." . $UserPrefs{"imageext"};
    }
    else
    {
     $UserPrefs{"imageext"} = "gif";
     $UserPrefs{"gnuplotoutput"} = $UserPrefs{"gnuplotfile"} . "." . $UserPrefs{"imageext"};
    }       
     &Debugln("Image-Fileextention: " . $UserPrefs{"imageext"});
     &Debugln("GNUPlotoutput will be: " . $UserPrefs{"gnuplotoutput"});

    if (!defined($UserPrefs{"dbtype"}))
    {
     die "Personal Proxy Statistics : database type not defined\n";
    }
    if ($UserPrefs{"dbtype"} eq "postgresql")
    {
#     use DBD::Pg;       #--Database driver for PostgreSQL.
    }
    else
    {
     use DBD::mysql;    #--Database driver for MySQL
    }

    for ($Contest = 1; $Contest <= $SystemPrefs{"projects"}; $Contest++)
    {
	if (!defined($UserPrefs{"do_" . $SystemPrefs{"contestsuffix"}[$Contest] }))
	{
	    $UserPrefs{"do_" . $SystemPrefs{"contestsuffix"}[$Contest] } = $true;
	}
	if ($SystemPrefs{"disable" . $SystemPrefs{"contestsuffix"}[$Contest] } == $true)
	{
    	    $UserPrefs{"do_" . $SystemPrefs{"contestsuffix"}[$Contest] } = $false;
	}
	if ($SystemPrefs{"enable" . $SystemPrefs{"contestsuffix"}[$Contest] } == $true)
	{
    	    $UserPrefs{"do_" . $SystemPrefs{"contestsuffix"}[$Contest] } = $true;
	}
    }
}  #--ProcessUserPrefs


#--- Method Prolog -----------------------------------------------------------
#  Name: SanityCheck
#
#  Parameters: None.
#
#  Return Value: None.
#
#  Description:
#    Verify that each user defined variable from the initialization file has 
#    an appropriate value. More advanced verification occurs on variables that
#    users frequently enter incorrectly.
#
#--- End Method Prolog -------------------------------------------------------
sub SanityCheck
{
    my $up;
    my $msg = "Please verify " . $SystemPrefs{"appini"} . " file.\n";

    #--Verify each user preference, die if not defined.
    foreach $up (@AvailableUserPrefs) {
	if (!defined($UserPrefs{$up})) {
	    die $SystemPrefs{"appname"} . ": Required user variable '$up' not defined. $msg";
	}

	#--Print out each INI file variable and its value.
	&Debugln("$up=" . $UserPrefs{$up});
    }

    #--Check ranges for critical user defined variables.
    if (!defined($TimeZones{$UserPrefs{"timezone"}})) {
	die $SystemPrefs{"appname"} . ": Time zone not recognized. $msg\n";
    }
}  #--SanityCheck


#--- Method Prolog -----------------------------------------------------------
#  Name: CollectData
#
#  Parameters:
#    Contest - The contest to collect data for. 1=RC5, 2=DES, etc.
#
#  Return Value: None.
#
#  Description:
#    This function is responsible for collecting all data from the proxy
#    log files and inserting it into the database.
#
#    See http://ppstats.sourceforge.net/docs/import.php3 for details.
#
#--- End Method Prolog -------------------------------------------------------
sub CollectData
{
    my $Contest = shift @_;
    my $TempLogFile;
    my $entry;
    my $date; 
    my ($year, $month, $day);
    my ($time, $hour, $min, $sec);
    my ($mday, $wday);
    my $sql;
    my ($LastYear, $LastMonth, $LastDay);
    my @row;
    my $rst;
    my ($submitted, $host_id, $participant_id, $os_id, $cpu_id, $version, $wu);
    my ($subdate, $subtime);
    my $found;
    my $host;


    &Debugln("Entering CollectData() for " . $SystemPrefs{"contestname"}[$Contest] . " ...");

    #--Get the log directory and file mask for the given contest.
    my $LogDir = $UserPrefs{"logdir" . $SystemPrefs{"contestsuffix"}[$Contest]};
    my $LogPre = $UserPrefs{"logpre" . $SystemPrefs{"contestsuffix"}[$Contest]};

    #--Get logfiles in reverse order.
    my $LogGlob = $LogPre . "*." . $UserPrefs{"logsuf"};
    my @LogFiles = <$LogDir$LogGlob>;

    #--Handle compressed log files. The check for a valid log file suffix is
    #--to prevent problems where all files would be processed as if zipped.
    if ($UserPrefs{"processziplogs"}) 
    {
	$LogGlob = $LogPre . "*." . $UserPrefs{"logzipsuf"};
	push(@LogFiles, <$LogDir$LogGlob>);
    }

    @LogFiles = sort @LogFiles;

    #--Retrieve the last day logs were recorded in the database.
    $sql = "SELECT MAX(SUBMITTED_DATE) FROM " . $SystemPrefs{"contestwutable"}[$Contest];
    my ($LastSubmitted) = &DLookup($sql);
    
    #--Delete all log entries that were submitted on the last day.
    if (defined($LastSubmitted)) {
	my ($Date, $Time) = split(/ /, $LastSubmitted);
	($LastYear, $LastMonth, $LastDay) = split(/-/, $Date);

	$sql = "DELETE FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE SUBMITTED_DATE >= '" . $Date . "'";
	&RunSQL($sql);
    }	

    #--Process each log file, normal and zipped.
    my $UnifiedLogFile;
    foreach $UnifiedLogFile (@LogFiles) 
    {
	my $LogFile = $UnifiedLogFile;

	#--If there is data loaded already, only process those log files that
        #--haven't been seen yet (including the last day submitted).
	if (defined($LastSubmitted)) {
	    if ($UserPrefs{"processziplogs"} && (substr($LogFile, -length($UserPrefs{"logzipsuf"}), length($UserPrefs{"logzipsuf"})) eq $UserPrefs{"logzipsuf"})) {
		$year = substr($LogFile, length($LogFile) - length($UserPrefs{"logzipsuf"}) - 9, 4);
		$month = substr($LogFile, length($LogFile) - length($UserPrefs{"logzipsuf"}) - 5, 2);
		$day = substr($LogFile, length($LogFile) - length($UserPrefs{"logzipsuf"}) - 3, 2);
	    } else {
		$year = substr($LogFile, length($LogFile) - length($UserPrefs{"logsuf"}) - 9, 4);
		$month = substr($LogFile, length($LogFile) - length($UserPrefs{"logsuf"}) - 5, 2);
		$day = substr($LogFile, length($LogFile) - length($UserPrefs{"logsuf"}) - 3, 2);
	    }

	    #--If we already have this log file in the database, skip to the next one.
	    if (int("$year$month$day") < int("$LastYear$LastMonth$LastDay")) {
		next;
	    }
	}



	#--Handle compressed log files.
	my $LogFileSuffix = $UserPrefs{"logsuf"};
	my $IsCompressed = 0;

	#--If the file is compressed then extract a copy to the working app directory.
	if ($UserPrefs{"processziplogs"} && (substr($LogFile, -length($UserPrefs{"logzipsuf"}), length($UserPrefs{"logzipsuf"})) eq $UserPrefs{"logzipsuf"})) 
	{
	    $TempLogFile = $SystemPrefs{"appdir"} . substr($LogFile, -(length($LogPre)+8+1+length($UserPrefs{"logzipsuf"})), length($LogPre)+8+1) . $UserPrefs{"logtmpsuf"};
	    $LogFileSuffix = $UserPrefs{"logtmpsuf"};
	    system($UserPrefs{"logdecompressor"} . " $LogFile > $TempLogFile") == 0 or die $SystemPrefs{"appname"} . ": Cannot execute " . $UserPrefs{"logdecompressor"} . " $LogFile > $TempLogFile ($?)\n";
	    $LogFile = $TempLogFile;
	    $IsCompressed = 1;
	}


	open(LOGFILE, $LogFile) or die $SystemPrefs{"appname"} . ": Cannot open $LogFile ($!)\n";

	#--Start with a fresh hour and clean consolidation buffer.
	my $LastHour = undef;
	my $CurrHour = undef;
	my %Cons = ();
	my %Minutes = (); # Sorry, but I don't want to write a whole new function.
	my $con;
	my ($subyear, $submonth, $subday, $subtemp);

	#--Process each line in the log file.
	while (defined($entry = <LOGFILE>)) 
	{
	    chomp($entry);               #--no newline
	    $entry =~ tr/A-Z/a-z/;       #--convert to lowercase
	    $entry =~ s/^\s+//;          #--no leading white
	    $entry =~ s/\s+$//;          #--no trailing white
	    next unless length($entry);  #--anything left?   

	    #--Format of the proxy log file: 
            #--"Date/Time HostIP, Email, Block, Block Size, OS, CPU, Client Version"
	    my ($datetime, $ipaddr, $email, $block, $nkeys, $os, $cpu, $ver) = split(/,/, $entry);
	    $email = lc($email);

	    #--Parse the log file date and time field.
	    ($date, $time) = split(/ /, $datetime);
	    if (index ($date, "/") >= 0)
	    {
	     ($month, $day, $year) = split(/\//, $date);
 	    }
	    else
	    {
	     ($year, $month, $day) = split(/-/, $date);
	    }
	    ($hour, $min, $sec) = split(/:/, $time);

	    if (!defined($LastHour)) {		
		$LastHour = $hour;
	    }
	    $CurrHour = $hour;
	    

	    #--If we have started a new hour, flush out the consolidation buffer.
	    if ($LastHour ne $CurrHour) {
		foreach $con (keys %Cons) 
		{
		    ($submitted, $host_id, $participant_id, $os_id, $cpu_id, $version) = split(/,/, $con);
		    ($subdate, $subtemp) = split(/ /, $submitted);
		    $subtime = "$subtemp:" . $Minutes{$con} . ":00";
		    $wu = $Cons{$con};
		    ($subyear, $submonth, $subday) = split(/-/, $subdate);
		    if (($submonth != "00") && ($subday != "00"))
		    {
			$sql = "INSERT INTO " . $SystemPrefs{"contestwutable"}[$Contest] . " (\n" .
			   " SUBMITTED_DATE,\n" .
			   " SUBMITTED_TIME,\n" .
			   " HOST_ID,\n" . 
			   " PARTICIPANT_ID,\n" . 
		           " WU_SIZE,\n" . 
	        	   " OS_ID,\n" .  
		      	   " CPU_ID,\n" . 
			   " VERSION\n" . 
			   ") VALUES (\n" . 
			   " '$subdate',\n" .
			   " '$subtime',\n" .
			   " $host_id,\n" .
			   " $participant_id,\n" . 
			   " $wu,\n" .
		           " $os_id,\n" .
		           " $cpu_id,\n" .
		           " '$version'\n" .
		           ")";
			&RunSQL($sql);
		    }
		}

		$LastHour = $CurrHour;
		%Cons = ();
	    }

	    #--If this host address hssn't been seen before, add it to the database.
#	    &InsertHost($ipaddr) if (!defined($Hosts{$ipaddr}));
#	    my $host = $Hosts{$ipaddr}{"host_id"};	 

	    my ($testipa, $testipb, $testipc, $testipd) = split (/\./, $ipaddr);
	    my $test_id;

	    undef $found;
	    foreach $test_id (keys %Hosts)
	    {
		if (($Hosts{$test_id}{"ipa"} eq (sprintf ("%08d",&dec2bin($testipa)) & $Hosts{$test_id}{"maska"})) && ($Hosts{$test_id}{"ipb"} eq (sprintf ("%08d", &dec2bin($testipb)) & $Hosts{$test_id}{"maskb"})) && ($Hosts{$test_id}{"ipc"} eq (sprintf ("%08d", &dec2bin($testipc)) & $Hosts{$test_id}{"maskc"})) && ($Hosts{$test_id}{"ipd"} eq (sprintf ("%08d", &dec2bin($testipd)) & $Hosts{$test_id}{"maskd"})) && !defined($found))
		{
		$found = 1;
		$host = $Hosts{$test_id}{"host_id"};
		}
	    }
	    if ($found != 1)
	    {
		&InsertHost($ipaddr);
		$host = $Hosts{$ipaddr}{"host_id"};
	    }



	    #--If this email address hasn't been seen before, add it to the database.
	    &InsertEmail($email) if (!defined($Emails{$email}));

	    #--Skip this line if the email address is not marked as a member.
	    next unless $Emails{$email}{"member"} eq $true;

	    #--Determine who gets credit for this work unit.
	    my $participant;
	    if ($Emails{$email}{"retire_to_id"} > 0) 
	    {
		$participant = $Emails{$email}{"retire_to_id"};
	    } 
	    else 
	    {
		$participant = $Emails{$email}{"participant_id"};
	    }

	    #--Uncomment this to convert nkeys to proper format, 
            #--only used for compatibility with older logfiles.
	    #if ($nkeys == (2**28)) { $nkeys = 1; }

	    #--Construct the consolidation buffer for this record. Group by hour.
	    $con = "$year-$month-$day $hour,$host,$participant,$os,$cpu,$ver";	    
	    if (defined($Cons{$con})) { $Cons{$con} += $nkeys; }
	    else { $Cons{$con} = $nkeys; }
	    if (defined($Minutes{$con})) 
	    {
	        if ($Minutes{$con} < $min) 
	        { 
	    	    $Minutes{$con} = $min; 
	        }
	    }
	    else
	    {
	        $Minutes{$con} = $min;
	    }
	}

	#--Flush out the consolidation buffer since its likely we have reached EOF of
        #--the log file while processing the current hour.
	foreach $con (keys %Cons) 
	{
	    ($submitted, $host_id, $participant_id, $os_id, $cpu_id, $version) = split(/,/, $con);
	    ($subdate, $subtemp) = split(/ /, $submitted);
	    $subtime = "$subtemp:" . $Minutes{$con} . ":00";
	    $wu = $Cons{$con};
		
	    ($subyear, $submonth, $subday) = split(/-/, $subdate);
	    if (($submonth != "00") && ($subday != "00"))
	    {
		$sql = "INSERT INTO " . $SystemPrefs{"contestwutable"}[$Contest] . " (\n" .
		   " SUBMITTED_DATE,\n" .
		   " SUBMITTED_TIME,\n" .
		   " HOST_ID,\n" . 
		   " PARTICIPANT_ID,\n" . 
	           " WU_SIZE,\n" . 
        	   " OS_ID,\n" .  
	      	   " CPU_ID,\n" . 
		   " VERSION\n" . 
		   ") VALUES (\n" . 
		   " '$subdate',\n" .
		   " '$subtime',\n" .
		   " $host_id,\n" .
		   " $participant_id,\n" . 
		   " $wu,\n" .
	           " $os_id,\n" .
	           " $cpu_id,\n" .
		   " '$version'\n" .
                   ")";
		&RunSQL($sql);
	    }
	}

	close LOGFILE;


	#--If the file compressed, delete the temporary uncompressed file.
	if ($IsCompressed) {
	    unlink("$TempLogFile") == 1 or die $SystemPrefs{"appname"} . ": Cannot delete $TempLogFile ($?)\n";
	    $IsCompressed = 0;
	}

	&Debug(".");
    }

    &Debugln("");

    &Debugln("Exiting CollectData() for " . $SystemPrefs{"contestname"}[$Contest] . " ...");
}  #--CollectData

#--- Method Prolog -----------------------------------------------------------
#  Name: write_css
#
#  Parameters: 
#      Contest - The contest to collect data for. 1=RC5, 2=DES, etc.
#
#  Return Value: None.
#
#  Description:
#    Write the cascading style sheet file
#
#--- End Method Prolog -------------------------------------------------------
sub write_css
{
    my $Contest = shift @_;
    &Debugln("Entering write_css($Contest) ...");
    mkdir($UserPrefs{"htmldir"} . $SystemPrefs{"contestsuffix"}[$Contest] , 0755);
    open(OUTFILE, ">" . $UserPrefs{"htmldir"} . $SystemPrefs{"contestsuffix"}[$Contest] . "/" . $UserPrefs{"css"}) or die "Personal Proxy Statistics : Cannot open " . $UserPrefs{"htmldir"} . $SystemPrefs{"contestsuffix"}[$Contest] . "/" . $UserPrefs{"css"} . " ($!)\n";
    print OUTFILE "BODY {\n",
                  " font-family: " . $UserPrefs{"fontstyle"} . ";\n",
                  " font-size: 10pt;\n",
                  " color: #" . $UserPrefs{"textcolor"} . ";\n",
                  " background: #" . $UserPrefs{"backgroundcolor"} . ";\n",
                  "}\n\n";
    print OUTFILE "P {\n",
                  " font-family: " . $UserPrefs{"fontstyle"} . ";\n",
                  " font-size: 10pt;\n",
                  " color: #" . $UserPrefs{"textcolor"} . ";\n",
                  "}\n\n";
    print OUTFILE "TH {\n",
                  " font-family: " . $UserPrefs{"fontstyle"} . ";\n",
                  " font-size: 10pt;\n",
                  " color: #" . $UserPrefs{"linkbartitlefontcolor"} . ";\n",
                  "}\n\n";
    print OUTFILE "TD {\n",
                  " font-family: " . $UserPrefs{"fontstyle"} . ";\n",
                  " font-size: 10pt;\n",
                  " color: #" . $UserPrefs{"textcolor"} . ";\n",
                  "}\n\n";
    print OUTFILE "TD.section {\n",
                  " color: #" . $UserPrefs{"sectiontitlefontcolor"} . ";\n",
                  " background: #" . $UserPrefs{"sectiontitlebarcolor"} . ";\n",
                  "}\n\n";
    print OUTFILE "TD.linkbar {\n",
                  " background: #" . $UserPrefs{"linkbarcolor"} . ";\n",
                  "}\n\n";

    print OUTFILE "TD.title {\n",
                  " font-size: 12pt;\n",
                  " color: #" . $UserPrefs{"sectiontitlefontcolor"} . ";\n",
                  " background: #" . $UserPrefs{"sectiontitlebarcolor"} . ";\n",
                  "}\n\n";
    print OUTFILE "TD.subtitle {\n",
                  " font-size: 12pt;\n",
                  " color: #" . $UserPrefs{"linkbartitlefontcolor"} . ";\n",
                  "}\n\n";
    print OUTFILE "TD.note { color: #" . $UserPrefs{"sectionnotefontcolor"} . "; }\n\n";
    print OUTFILE "TD.history { background: #" . $UserPrefs{"historycolor"} . "; }\n\n";
    print OUTFILE "TD.normal { color: #" . $UserPrefs{"textcolor"} . "; }\n\n";
    print OUTFILE "TD.warning { color: #" . $UserPrefs{"lastseenwarningcolor"} . "; }\n\n";
    print OUTFILE "TD.critical { color: #" . $UserPrefs{"lastseencriticalcolor"} . "; }\n\n";
    print OUTFILE "TR.odd { background: #" . $UserPrefs{"sectionlowlightcolor"} . "; }\n\n";
    print OUTFILE "TR.even { background: #" . $UserPrefs{"sectionhighlightcolor"} . "; }\n\n";
    print OUTFILE "SPAN.moveup { color: #" . $UserPrefs{"moveupcolor"} . "; }\n\n";
    print OUTFILE "SPAN.movedown { color: #" . $UserPrefs{"movedowncolor"} . "; }\n\n";
    print OUTFILE "A:link {\n", 
                  " color: #" . $UserPrefs{"linkcolor"} . ";\n", 
                  " text-decoration: none;\n", 
                  "}\n\n";
    print OUTFILE "A:active {\n", 
                  " color: #" . $UserPrefs{"activelinkcolor"} . ";\n", 
                  " text-decoration: none;\n", 
                  "}\n\n";
    print OUTFILE "A:visited {\n", 
                  " color: #" . $UserPrefs{"visitedlinkcolor"} . ";\n", 
                  " text-decoration: none;\n", 
                  "}\n\n";
    print OUTFILE "A:hover {\n", 
                  " color: #" . $UserPrefs{"activelinkcolor"} . ";\n",
                  "}\n\n";
    close OUTFILE;
    &Debugln("Exiting write_css($Contest) ...");
}  # write_css


#--- Method Prolog -----------------------------------------------------------
#  Name: write_header
#
#  Parameters:
#      Contest - The contest to collect data for. 1=RC5, 2=DES, etc.
#
#  Return Value: None.
#
#  Description:
#    Write header in open HTML files
#
#
#--- End Method Prolog -------------------------------------------------------
sub write_header
{
    my $Contest = shift @_;

    print OUTFILE "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0 Transitional//EN\" \"http://www.w3.org/TR/REC-html40/loose.dtd\">\n",
                  "<HTML>\n",
                  " <HEAD>\n",
                  "  <LINK REL=\"stylesheet\" TYPE=\"text/css\" HREF=\"" . $UserPrefs{"htmlroot"} . $SystemPrefs{"contestsuffix"}[$Contest]. "/" . $UserPrefs{"css"} . "\">\n",
                  "  <TITLE>Team " . $UserPrefs{"team"} . " " . $SystemPrefs{"contestname"}[$Contest] . " Statistics</TITLE>\n";

    if ($UserPrefs{"usemetarefresh"}) {
        print OUTFILE "  <META HTTP-EQUIV=\"refresh\" CONTENT=\"" . $UserPrefs{"metarefreshperiod"} . "\">\n";
    }

    print OUTFILE "  <META HTTP-EQUIV=\"Content-Type\" CONTENT=\"text/html; charset=iso-8859-1\">\n",
                  "  <META NAME=\"description\" CONTENT=\"Distributed.Net " . $SystemPrefs{"contestname"}[$Contest] . " Personal Proxy Statistics for Team " . $UserPrefs{"team"} . "\">\n",
                  "  <META NAME=\"keywords\" CONTENT=\"Distributed.Net, RC5, RC5-64, DES, DES-II, DES-III, OGR, Encryption, Computers, Parallel Processing\">\n",
                  "  <META NAME=\"robots\" CONTENT=\"nofollow\">\n",
                  " </HEAD>\n",
                  " <BODY>\n";

    &write_banner() if ($UserPrefs{"useheaderbanner"});

    print OUTFILE "  <TABLE BORDER=0 CELLPADDING=0 CELLSPACING=0 WIDTH=\"100%\">\n",
                  "   <TR>\n",
                  "    <TD VALIGN=TOP>\n",
                  "     <TABLE BORDER=0 CELLPADDING=1 CELLSPACING=0 WIDTH=130>\n",
                  "      <TR><TH ALIGN=LEFT>\n",
                  "       &nbsp;&nbsp;Team " . $UserPrefs{"team"} . "\n",
                  "      </TH></TR>\n",
                  "      <TR><TD CLASS=linkbar NOWRAP>\n",
                  "       &nbsp;&nbsp;<A HREF=\"" . $UserPrefs{"teamhome"} . "\">Team Home</A><BR>\n",
                  "       &nbsp;&nbsp;<A HREF=\"" . $UserPrefs{"htmlroot"} . $SystemPrefs{"contestsuffix"}[$Contest]. "/" . $UserPrefs{"htmlteam"} . "\">Project Stats</A><BR>\n",
                  "       &nbsp;&nbsp;<A HREF=\"" . $UserPrefs{"htmlroot"} . $SystemPrefs{"contestsuffix"}[$Contest]. "/" . $UserPrefs{"htmlexec"} . "\">Team Summary</A><BR>\n",
                  "       &nbsp;&nbsp;<A HREF=\"" . $UserPrefs{"htmlroot"} . $SystemPrefs{"contestsuffix"}[$Contest]. "/" . $UserPrefs{"htmlmail"} . "\">By Email</A><BR>\n";

    if ($UserPrefs{"showbyhost"}) {
	print OUTFILE "       &nbsp;&nbsp;<A HREF=\"" . $UserPrefs{"htmlroot"} . $SystemPrefs{"contestsuffix"}[$Contest]. "/" . $UserPrefs{"htmlhost"} . "\">By Host</A><BR>\n";
    }

    print OUTFILE "       &nbsp;&nbsp;<A HREF=\"" . $UserPrefs{"htmlroot"} . $SystemPrefs{"contestsuffix"}[$Contest]. "/" . $UserPrefs{"htmlos"} . "\">By OS</A><BR>\n",
                  "       &nbsp;&nbsp;<A HREF=\"" . $UserPrefs{"htmlroot"} . $SystemPrefs{"contestsuffix"}[$Contest]. "/" . $UserPrefs{"htmlcpu"} . "\">By CPU</A><BR>\n",
                  "       &nbsp;&nbsp;<A HREF=\"" . $UserPrefs{"htmlroot"} . $SystemPrefs{"contestsuffix"}[$Contest]. "/" . $UserPrefs{"htmlver"} . "\">By Client ver</A><BR>\n",
                  "       &nbsp;&nbsp;<A HREF=\"" . $UserPrefs{"htmlroot"} . $SystemPrefs{"contestsuffix"}[$Contest]. "/" . $UserPrefs{"htmlfull"} . "\">By Full Detail</A><BR>\n",
                  "       &nbsp;&nbsp;<A HREF=\"" . $UserPrefs{"htmlroot"} . $SystemPrefs{"contestsuffix"}[$Contest]. "/" . $UserPrefs{"htmldomain"} . "\">By Domain</A><BR>\n",
                  "       &nbsp;&nbsp;<A HREF=\"" . $UserPrefs{"htmlroot"} . $SystemPrefs{"contestsuffix"}[$Contest]. "/" . $UserPrefs{"htmlhour"} . "\">By Hour of Day</A><BR>\n";

    if ($UserPrefs{"disable_bydate"} != $true)
    {
	print OUTFILE "       &nbsp;&nbsp;<A HREF=\"" . $UserPrefs{"htmlroot"} . $SystemPrefs{"contestsuffix"}[$Contest]. "/" . $UserPrefs{"htmldate"} . "\">By Date</A><BR>\n";
    }
    if ($UserPrefs{"disable_bydate"} != $true)
    {
	print OUTFILE "       &nbsp;&nbsp;<A HREF=\"" . $UserPrefs{"htmlroot"} . $SystemPrefs{"contestsuffix"}[$Contest]. "/" . $UserPrefs{"htmlweek"} . "\">By Week</A><BR>\n";
    }
    if ($UserPrefs{"disable_bydate"} != $true)
    {
	print OUTFILE "       &nbsp;&nbsp;<A HREF=\"" . $UserPrefs{"htmlroot"} . $SystemPrefs{"contestsuffix"}[$Contest]. "/" . $UserPrefs{"htmlmonth"} . "\">By Month</A><BR>\n";
    }
    if ($UserPrefs{"disable_bydate"} != $true)
    {
	print OUTFILE "       &nbsp;&nbsp;<A HREF=\"" . $UserPrefs{"htmlroot"} . $SystemPrefs{"contestsuffix"}[$Contest]. "/" . $UserPrefs{"htmlyear"} . "\">By Year</A><BR>\n";
    }
    		  
    print OUTFILE "       &nbsp;&nbsp;<A HREF=\"" . $UserPrefs{"htmlroot"} . $SystemPrefs{"contestsuffix"}[$Contest]. "/" . $UserPrefs{"htmldayofweek"} . "\">By Day of Week</A><BR>\n";

    print OUTFILE "      </TD></TR>\n",
                  "      <TR><TH ALIGN=LEFT>\n",
                  "       &nbsp;&nbsp;" . $SystemPrefs{"contestname"}[$Contest] . "\n",
                  "      </TH></TR>\n";

    print OUTFILE "      <TR><TD CLASS=linkbar NOWRAP>\n",
                  "       &nbsp;&nbsp;<A HREF=\"http://www.distributed.net/\">Distributed.Net</A><BR>\n",
                  "       &nbsp;&nbsp;<A HREF=\"http://www.distributed.net/" . $SystemPrefs{"contesthome"}[$Contest] . "/\">Project Home</A><BR>\n",
                  "       &nbsp;&nbsp;<A HREF=\"http://www.distributed.net/FAQ/\">Distributed FAQs</A><BR>\n",
                  "       &nbsp;&nbsp;<A HREF=\"http://www.distributed.net/clients.html\">Client Download</A><BR>\n",
                  "       &nbsp;&nbsp;<A HREF=\"http://www.distributed.net/proxies.html\">Proxy Download</A><BR>\n",
                  "       &nbsp;&nbsp;<A HREF=\"http://www.distributed.net/cgi/dnet-finger.cgi\">Finger Gateway</A><BR>\n",
                  "       &nbsp;&nbsp;<A HREF=\"http://www.distributed.net/lists/\">Mailing Lists</A><BR>\n",
                  "       &nbsp;&nbsp;<A HREF=\"" . $SystemPrefs{"stats-server"} . "/team/tmsummary.php" .  $SystemPrefs{"conteststats"}[$Contest]  . "&team=" . $UserPrefs{"teamid"} . "\">Team Summary</A><BR>\n",
                  "       &nbsp;&nbsp;<A HREF=\"" . $SystemPrefs{"stats-server"} . "/team/tmember.php" . $SystemPrefs{"conteststats"}[$Contest]  . "&team=" . $UserPrefs{"teamid"} . "\">Top 100 Overall</A><BR>\n",
                  "       &nbsp;&nbsp;<A HREF=\"" . $SystemPrefs{"stats-server"} . "/team/tmember.php" . $SystemPrefs{"conteststats"}[$Contest]  . "&team=" . $UserPrefs{"teamid"} . "&source=y\">Top 100 Yesterday</A><BR>\n",
                  "       &nbsp;&nbsp;<A HREF=\"" . $SystemPrefs{"stats-server"} . "/projects.php" . $SystemPrefs{"conteststats"}[$Contest]  . "\">Project Stats</A><BR>\n",
                  "       &nbsp;&nbsp;<A HREF=\"http://www.distributed.net/statistics/\">Graphical Stats</A><BR>\n",
                  "       &nbsp;&nbsp;<A HREF=\"http://www.distributed.net/" . $SystemPrefs{"contestsuffix"}[$Contest] . "/proxyinfo.html\">Keyservers</A><BR>\n",
                  "      </TD></TR>\n";

    print OUTFILE "      <TR><TH ALIGN=LEFT>\n",
                  "       &nbsp;&nbsp;Teams & Groups\n",
                  "      </TH></TR>\n";

    print OUTFILE "      <TR><TD CLASS=linkbar NOWRAP>\n",
                  "       &nbsp;&nbsp;<A HREF=\"" . $SystemPrefs{"stats-server"} . "/team/tlist.php" . $SystemPrefs{"conteststats"}[$Contest] . "&low=1&limit=100\">Top 100 Overall</A><BR>\n",
                  "       &nbsp;&nbsp;<A HREF=\"" . $SystemPrefs{"stats-server"} . "/team/tlist.php" . $SystemPrefs{"conteststats"}[$Contest] . "&low=1&limit=100&source=y\">Top 100 Yesterday</A>\n",
                  "       &nbsp;&nbsp;<FORM ACTION=\"" . $SystemPrefs{"stats-server"} . "/team/tsearch.php" . $SystemPrefs{"conteststats"}[$Contest] . "\" METHOD=GET>\n",
                  "       &nbsp;&nbsp;Search for team:<BR>\n",
                  "       &nbsp;&nbsp;<INPUT TYPE=\"text\" NAME=\"st\" VALUE=\"\" SIZE=6 MAXLENGTH=60>\n",
                  "                   <INPUT TYPE=\"submit\" VALUE=\"Go!\">\n",
                  "                   </FORM>\n",
                  "       &nbsp;&nbsp;<A HREF=\"http://stats.distributed.net/newteam1.php\">Register New Team</A><BR>\n",
                  "      </TD></TR>\n";

    print OUTFILE "      <TR><TH ALIGN=LEFT>\n",
                  "       &nbsp;&nbsp;Individuals\n",
                  "      </TH></TR>\n";

    print OUTFILE "      <TR><TD CLASS=linkbar NOWRAP>\n",
                  "       &nbsp;&nbsp;<A HREF=\"" . $SystemPrefs{"stats-server"} . "/participant/plist.php" . $SystemPrefs{"conteststats"}[$Contest] . "&low=1&limit=100\">Top 100 Overall</A><BR>\n",
                  "       &nbsp;&nbsp;<A HREF=\"" . $SystemPrefs{"stats-server"} . "/participant/plist.php" . $SystemPrefs{"conteststats"}[$Contest] . "&low=1&limit=100&source=y\">Top 100 Yesterday</A>\n",
                  "       &nbsp;&nbsp;<FORM ACTION=\"" . $SystemPrefs{"stats-server"} . "/participant/psearch.php" . $SystemPrefs{"conteststats"}[$Contest] . "\" METHOD=GET>\n",
                  "       &nbsp;&nbsp;Search for email:<BR>\n",
                  "       &nbsp;&nbsp;<INPUT TYPE=\"text\" NAME=\"st\" VALUE=\"\" SIZE=6 MAXLENGTH=60>\n",
                  "                   <INPUT TYPE=\"submit\" VALUE=\"Go!\">\n",
                  "                   </FORM>\n",
                  "       &nbsp;&nbsp;<A HREF=\"http://stats.distributed.net/participant/pedit.php\">Participant Details</A><BR>\n",
                  "       &nbsp;&nbsp;<A HREF=\"" . $SystemPrefs{"stats-server"} . "/participant/pjointeam.php?team=" . $UserPrefs{"teamid"} . "\">Join This Team</A><BR>\n",
                  "      </TD></TR>\n";

    print OUTFILE "      <TR><TD ALIGN=CENTER NOWRAP>\n",
                  "       Last Updated:<BR>\n",
                  "       ". $SystemPrefs{"timestamp"} ."<BR>\n";

    if ($UserPrefs{"usecounter"}) {
        print OUTFILE "       Hit Count: <IMG BORDER=0 SRC=\"" . $UserPrefs{"counter"} . "\" ALT=\"Hit Count\" VSPACE=0 HSPACE=0><BR>\n";
    }

    print OUTFILE "      </TD></TR>\n",
                  "     </TABLE>\n",
                  "    </TD>\n",
                  "    <TD VALIGN=TOP WIDTH=\"100%\">\n";

    if ($UserPrefs{"uselogo"}) {
        print OUTFILE "     <P ALIGN=CENTER>\n",
                      "      <A HREF=\"" . $UserPrefs{"teamhome"} . "\"><IMG BORDER=0 SRC=\"" . $UserPrefs{"logourl"} . "\" ALT=\"" . $UserPrefs{"team"} . "\"></A>\n",
                      "     </P>\n";
    }
}  # write_header


#--- Method Prolog -----------------------------------------------------------
#  Name: write_footer
#
#  Parameters: None
#
#  Return Value: None.
#
#  Description:
#    Write footer in open HTML files
#
#
#--- End Method Prolog -------------------------------------------------------
sub write_footer
{
    print OUTFILE "    </TD>\n",
                  "   </TR>\n",
                  "  </TABLE>\n",
                  "  <HR SIZE=1>\n";

    &write_banner if ($UserPrefs{"usefooterbanner"});

    print OUTFILE "  <P ALIGN=CENTER>\n",
    "   Personal Proxy Statistics\n",
    "   <A HREF=\"http://www.intrepid.de/ppstats/ppstats-" . $SystemPrefs{"appver"} . ".tar.gz\">ppstats-" . $SystemPrefs{"appver"} . ".tar.gz</A><BR>\n",
    "   Copyright (C) <A HREF=\"mailto:kpesce\@netscape.net\">Kevin Pesce</A> 1998, 1999<BR>\n",
    "   Modified by <A HREF=\"mailto:olli\@intrepid.de\">Oliver Fa&szlig;bender</A> 2001,2002,2003<BR>\n",
    "   This program was written under GNU General Public License",
    "  </P>\n",
    " </BODY>\n",
    "</HTML>\n";


#    print OUTFILE "  <P ALIGN=CENTER>\n",
#                  "   Personal Proxy Statistics\n",
#                  "   <A HREF=\"http://usmcug.usm.maine.edu/rc5/files/ppstats-$contestsuffix.zip\">ppstats-$contestsuffix.zip</A> v$appver<BR>\n",
#                  "   Copyright (C) <A HREF=\"mailto:kpesce\@netscape.net\">Kevin Pesce</A> 1998, 1999<BR>\n",
#                  "   University of Southern Maine Computer Users Group\n",
#                  "  </P>\n",
#                  " </BODY>\n",
#                  "</HTML>\n";
}  # write_footer



#--- Method Prolog -----------------------------------------------------------
#  Name: write_section_header
#
#  Parameters: 
#      Page - Number of page to write 
#      Title - Page Title/Headline
#      htmlfile - basefilename for page to write (without pagenumber)
#      Contest - The contest to collect data for. 1=RC5, 2=DES, etc.
#
#  Return Value: None.
#
#  Description:
#    Calling the procedures that write the html-header (done by 
#     calling &write_header) and write the tableheader for section
#
#--- End Method Prolog -------------------------------------------------------
sub write_section_header
{
    my $page = shift @_;
    my $title = shift @_;
    my $htmlfile = shift @_;
    my $Contest = shift @_;
    my $y;
    my ($year, $month, $mday, $hour, $min, $sec, $wday); 

    open(OUTFILE, ">" . $UserPrefs{"htmldir"} . $SystemPrefs{"contestsuffix"}[$Contest] . "/" . $page . $htmlfile) or die "Personal Proxy Statistics : Cannot open " . $UserPrefs{"htmldir"} . $SystemPrefs{"contestsuffix"}[$Contest] . "/" . $page . $htmlfile . " ($!)\n";
    &write_header($Contest);
    print OUTFILE "     <TABLE BORDER=0 CELLPADDING=0 CELLSPACING=0 WIDTH=\"100%\">\n";
    print OUTFILE "      <TR ALIGN=CENTER>\n",
                  "       <TD CLASS=section>Rank</TD>\n",
                  "       <TD CLASS=section NOWRAP>$title</TD>\n";

    for ($y = $UserPrefs{"dayshistory"} - 3; $y > 1; $y--) 
    {
	($year, $month, $mday, $hour, $min, $sec, $wday) = &nice_gmtime($SystemPrefs{"timebegin"} - $y * (24*60*60));
	print OUTFILE "       <TD CLASS=section>" . $SystemPrefs{"contestheaderblocktype"}[$Contest] . "s<BR>$Days[$wday]</TD>\n";
    }
    if ($UserPrefs{"dayshistory"} - 3 >= 1) {
	print OUTFILE "       <TD CLASS=section>" . $SystemPrefs{"contestheaderblocktype"}[$Contest] . "s<BR>Yesterday</TD>\n";
    }
    if ($UserPrefs{"dayshistory"} - 3 >= 0) {
	print OUTFILE "       <TD CLASS=section>" . $SystemPrefs{"contestheaderblocktype"}[$Contest] . "s<BR>Today</TD>\n";
    }
    print OUTFILE "       <TD CLASS=section>" . $SystemPrefs{"contestheaderblocktype"}[$Contest] . "s<BR>Total</TD>\n",
                  "       <TD CLASS=section>Last<BR>Seen</TD>\n",
                  "       <TD CLASS=section>Percent</TD>\n",
                  "      </TR>\n";
}  # write_section_header


#--- Method Prolog -----------------------------------------------------------
#  Name: write_section_footer
#
#  Parameters: 
#      Page - Number of page to write
#      totalrows - totalrows in this section
#      offset - Offset for '$UserPrefs{"dayshistory"}'
#      htmlfile - base filname for this section (for links between pages if more than one)
#   DISABLED -- pageblocks - blocks on this page (was an array in the past, but is not valid anymore)
#      Contest - The contest to collect data for. 1=RC5, 2=DES, etc.
#      domain - for the domainpage is one colum extra needed
#
#  Return Value: None.
#
#  Description:
#    Calling the procedures that closes table with summary and
#     writes the html-footers (done by &write_footer)
#
#--- End Method Prolog -------------------------------------------------------
sub write_section_footer
{
    my $page = shift @_;
    my $totalrows = shift @_;
    my $offset = shift @_;
    my $htmlfile = shift @_;
    my $msg = shift @_;
    my $Contest = shift @_;
    my $domain = shift @_;
    my $percentofteam;
    my $colsp;
    my $y;
    my ($year, $month, $mday, $hour, $min, $sec, $wday); 
    my $sql;
    my $printblocks;

    if ($totalrows > $UserPrefs{"maxrows"}) {
	print OUTFILE "      <TR ALIGN=RIGHT>\n",
                      "       <TD CLASS=section>&nbsp;</TD>\n",
                      "       <TD CLASS=section>Page Total</TD>\n";
    
	for ($y = $UserPrefs{"dayshistory"} - 2 - $offset; $y > 0; $y--) 
	{
	    if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
	    {
		print OUTFILE "       <TD CLASS=section>$pageblocks[$y]</TD>\n";
	    }
	    else
	    {
		print OUTFILE "       <TD CLASS=section>" . &ratef($pageblocks[$y] * $SystemPrefs{"contestfactor"}[$Contest]) . "</TD>\n";
	    }
	}
        if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
        {
	    print OUTFILE "       <TD CLASS=section>$pageblocks[0]</TD>\n";
	}
	else
	{
	    print OUTFILE "       <TD CLASS=section>" . &ratef($pageblocks[0] * $SystemPrefs{"contestfactor"}[$Contest]) . "</TD>\n";
	}
    
	for ($y = 0; $y <= $offset + 1; $y++) {
	    print OUTFILE "       <TD CLASS=section>&nbsp;</TD>\n";
	}

        if ($domain != 0)
        {
	    print OUTFILE "       <TD CLASS=section>&nbsp;</TD>\n";
	}

	print OUTFILE "      </TR>\n";
    }

    print OUTFILE "      <TR ALIGN=RIGHT>\n",
                  "       <TD CLASS=section>&nbsp;</TD>\n",
                  "       <TD CLASS=section>" . $SystemPrefs{"contestheaderblocktype"}[$Contest] . "s Total</TD>\n";
    
    for ($y = $UserPrefs{"dayshistory"} - 3 - $offset; $y >= 0; $y--) 
    {
	($year, $month, $mday, $hour, $min, $sec, $wday) = &nice_gmtime($SystemPrefs{"timebegin"} - ($y * (24*60*60)));
#SELECT SUM(WU_SIZE) FROM `TBL_WU_CSC` WHERE DATE_FORMAT(SUBMITTED,"%Y-%m-%d") = "1999-07-11"
        $sql = "SELECT IFNULL(SUM(WU_SIZE),\"-\") FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE SUBMITTED_DATE = \"$year-$month-$mday\"";
        my ($blocksthatday) = &DLookup($sql);
	if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
	{
	    print OUTFILE "       <TD CLASS=section>$blocksthatday</TD>\n";
	}
	else
	{
	    if ($blocksthatday ne "-")
	    {
	        $printblocks = &ratef($blocksthatday * $SystemPrefs{"contestfactor"}[$Contest]);
	    }
	    else
	    {
		$printblocks = $blocksthatday;
	    }
	    print OUTFILE "       <TD CLASS=section>$printblocks</TD>\n";
	}
    }

    ($year, $month, $mday, $hour, $min, $sec, $wday) = &nice_gmtime($SystemPrefs{"timebegin"} - $UserPrefs{"dayshistory"} * (24*60*60));
#SELECT SUM(WU_SIZE) FROM TBL_WU_RC564 
    $sql = "SELECT IFNULL(SUM(WU_SIZE),\"-\") FROM " . $SystemPrefs{"contestwutable"}[$Contest];
    my ($blocksoverall) = &DLookup($sql);
    
    if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
    {
	print OUTFILE "       <TD CLASS=section>$blocksoverall</TD>\n";
    }
    else
    {
	if ($blocksoverall > "0")
	{
	    $printblocks = &ratef($blocksoverall * $SystemPrefs{"contestfactor"}[$Contest]);
	}
	else
	{
	    $printblocks = $blocksoverall;
	}
	print OUTFILE "       <TD CLASS=section>$printblocks</TD>\n";
    }

    for ($y = 0; $y <= $offset + 1; $y++) {
	print OUTFILE "       <TD CLASS=section>&nbsp;</TD>\n";
    }

    if ($domain != 0)
    {
	print OUTFILE "       <TD CLASS=section>&nbsp;</TD>\n";
    }

    print OUTFILE "      </TR>\n";
    
    if ($totalrows > $UserPrefs{"maxrows"}) {
	print OUTFILE "      <TR ALIGN=RIGHT>\n",
                      "       <TD CLASS=section>&nbsp;</TD>\n",
                      "       <TD CLASS=section>Percent</TD>\n";
    
	for ($y = $UserPrefs{"dayshistory"} - 2 - $offset; $y > 0; $y--) 
	{
	    ($year, $month, $mday, $hour, $min, $sec, $wday) = &nice_gmtime($SystemPrefs{"timebegin"} - (($y-1) * (24*60*60)));
#SELECT SUM(WU_SIZE) FROM `TBL_WU_CSC` WHERE DATE_FORMAT(SUBMITTED,"%Y-%m-%d") = "1999-07-11"
    	    $sql = "SELECT IFNULL(SUM(WU_SIZE),\"-\") FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE SUBMITTED_DATE = \"$year-$month-$mday\"";
    	    my ($blocksthatday) = &DLookup($sql);
	    if ($blocksthatday eq "-") 
	    {
		$percentofteam = sprintf("%.2f", 100);
	    } 
	    else 
	    {
		$percentofteam = sprintf("%.2f", 100 * $pageblocks[$y] / $blocksthatday);
	    }
	    print OUTFILE "       <TD CLASS=section>$percentofteam%</TD>\n",
	}

	if ($blocksoverall eq "-")
	{
	    $percentofteam = sprintf("%.2f", 100);
	} 
	else 
	{
	    $percentofteam = sprintf("%.2f", 100 * $pageblocks[0] / $blocksoverall);
	}
	print OUTFILE "       <TD CLASS=section>$percentofteam%</TD>\n";
    
	for ($y = 0; $y <= $offset + 1; $y++) {
	    print OUTFILE "       <TD CLASS=section>&nbsp;</TD>\n";
	}

	if ($domain != 0)
	{
    	    print OUTFILE "       <TD CLASS=section>&nbsp;</TD>\n";
	}

	print OUTFILE "      </TR>\n";
    }

    $colsp = 5 + $offset;
    if ($UserPrefs{"dayshistory"} > 2 + $offset) {
	$colsp = $UserPrefs{"dayshistory"} + 2;
    }

    if ($totalrows > $UserPrefs{"maxrows"}) {
	print OUTFILE "      <TR>\n",
                      "       <TD COLSPAN=$colsp>\n",
                      "        &nbsp;\n",
                      "       </TD>\n",
                      "      </TR>\n";

	print OUTFILE "      <TR ALIGN=CENTER>\n",
                      "       <TD COLSPAN=$colsp>\n",
                      "        Goto Page:" . &goto_page($page, $totalrows, $htmlfile, $Contest) . "\n",
		      "       </TD>\n",
                      "      </TR>\n";
    }

    if ($msg ne "") {
	print OUTFILE "      <TR>\n",
                      "       <TD COLSPAN=$colsp>\n",
                      "        &nbsp;\n",
                      "       </TD>\n",
                      "      </TR>\n";

	print OUTFILE "      <TR ALIGN=CENTER>\n",
                      "       <TD CLASS=note COLSPAN=$colsp>\n",
                      "        $msg\n",
                      "       </TD>\n",
                      "      </TR>\n";
    }

    print OUTFILE "     </TABLE>\n";
    &write_footer();
    close OUTFILE;
}  # write_section_footer


#--- Method Prolog -----------------------------------------------------------
#  Name: write_banner
#
#  Parameters: none
#
#  Return Value: None.
#
#  Description:
#    Writes distributed.net bannerlink to open htmlfile
#
#--- End Method Prolog -------------------------------------------------------
sub write_banner
{
    print OUTFILE "  <P ALIGN=CENTER>\n",
                  "   <A HREF=\"http://www.distributed.net/\"><IMG BORDER=0 ALT=\"Distributed Computing\" WIDTH=400 HEIGHT=40 SRC=\"http://www.distributed.net/banners/image.cgi\"></A>\n",
	          "  </P>\n";
}  # write_banner


#--- Method Prolog -----------------------------------------------------------
#  Name: rate_day
#
#  Parameters: 
#      blocks - "Blocks" aka "Workunits" 
#      divide - To divide by $x
#      Contest - The contest to collect data for. 1=RC5, 2=DES, etc.
#
#  Return Value: 
#      Number of Keys/Nodes per second
#
#  Description:
#      Calculates to Keys or Nodes per second from number Workunits per day
#    
#
#--- End Method Prolog -------------------------------------------------------
sub rate_day
{
    my $blocks = shift @_;
    my $factor = shift @_;
    my $Contest = shift @_;
    

    if (!defined($blocks)) { $blocks = 0; }
    if ($blocks eq "-") { $blocks = 0; }
    if ($factor eq "-") { $factor = 0; }

    return ($SystemPrefs{"contestblockweight"}[$Contest] * $factor * $blocks / $SystemPrefs{"contestdivide"}[$Contest]); 
#    return ($SystemPrefs{"contestblockweight"}[$Contest] * $factor * $blocks / 322);
    #   (Seconds per day / keys in one workunit) 
    # = ( (24*60*60) / (2^28 / 1000000) )
    # = (86400 / 268,435456)
    # = 321,865081787109375
    # ~ 322
}  # rate_day


#--- Method Prolog -----------------------------------------------------------
#  Name: ratef
#
#  Parameters: 
#      rate - Keys or Nodes per second (value)
#
#  Return Value:
#      Formated rate as number and factor (2.5M)
#
#  Description:
#    Formating the incoming number to xxx.yyZ (xxx.yy<STRONG>Z</STRONG>)
#
#--- End Method Prolog -------------------------------------------------------
sub ratef
{
    my $rate = shift @_;

    if (!defined($rate)) { $rate = 0; }


    if ($rate < 10**3) {
        $rate = sprintf("%.2f", $rate) . " <STRONG>k</STRONG>";
    } elsif ($rate < 10**6) {
        $rate = sprintf("%.2f", $rate / 10**3) . " <STRONG>M</STRONG>";
    } elsif ($rate < 10**9) {
        $rate = sprintf("%.2f", $rate / 10**6) . " <STRONG>G</STRONG>";
    } elsif ($rate < 10**12) {
        $rate = sprintf("%.2f", $rate / 10**9) . " <STRONG>T</STRONG>";
    } elsif ($rate < 10**15) {
        $rate = sprintf("%.2f", $rate / 10**12) . " <STRONG>P</STRONG>";
    } elsif ($rate < 10**18) {
        $rate = sprintf("%.2f", $rate / 10**15) . " <STRONG>E</STRONG>";
    } elsif ($rate < 10**21) {
        $rate = sprintf("%.2f", $rate / 10**18) . " <STRONG>Z</STRONG>";
    } else {
        $rate = sprintf("%.2f", $rate / 10**21) . " <STRONG>Y</STRONG>";
    }

    return $rate;
}  # ratef


#--- Method Prolog -----------------------------------------------------------
#  Name: nice_gmtime
#
#  Parameters: 
#      @_ - UnixSeconds (Seconds since 1.1.1970 00:00 GMT)
#
#  Return Value: 
#      @gmt = TimeArray ($year, $month, $mday, $hour, $min, $sec, $wday)
#
#  Description:
#    Calling the procedures that write the html-pages
#
#--- End Method Prolog -------------------------------------------------------
sub nice_gmtime
{
    my @gmt = gmtime shift @_;
    my $i;

    $gmt[4]++;
    $gmt[5] += 1900;
    for ($i = 0; $i < 5; $i++) {
        while (length($gmt[$i]) < 2) {
            $gmt[$i] = "0" . $gmt[$i];
        }
    }

    return ($gmt[5], $gmt[4], $gmt[3], $gmt[2], $gmt[1], $gmt[0], $gmt[6]);
}  # nice_gmtime

#--- Method Prolog -----------------------------------------------------------
#  Name: goto_page
#
#  Parameters: 
#      page - Number of actual page
#      totalrows - totalrows for this section
#      htmlfile - filebasename (without path or pagenumber)
#
#  Return Value: 
#      Returns String with Link forward and backward (what is needed or both)
#
#  Description:
#    Generates Links back and forward depending on actual pagenumber
#
#--- End Method Prolog -------------------------------------------------------
sub goto_page
{
    my ($page, $totalrows, $htmlfile, $Contest) = @_;
    my $retval = "";
    my ($url, $x, $y);

    if ($page > 1) {
	if ($page == 2) {
	    $url = $UserPrefs{"htmlroot"} . $SystemPrefs{"contestsuffix"}[$Contest] . "/" . $htmlfile;
	} else {
	    $url = $UserPrefs{"htmlroot"} . $SystemPrefs{"contestsuffix"}[$Contest] . "/" . ($page - 1) . $htmlfile;
	}
        $retval .= " [<A HREF=\"$url\"><< Prev</A>]";
    }

    $x = 0;
    $y = 0;
    while ($y < $totalrows) {
	$x += 1;
	$y += $UserPrefs{"maxrows"};

	if ($x == 1) {
	    $url = $UserPrefs{"htmlroot"} . $SystemPrefs{"contestsuffix"}[$Contest] . "/" . $htmlfile;
	} else {
	    $url = $UserPrefs{"htmlroot"} . $SystemPrefs{"contestsuffix"}[$Contest] . "/" . $x . $htmlfile;
	}
	if ($x == $page) {
	    $retval .= " ($x)";
	} else {
	    $retval .= " <A HREF=\"$url\">$x</A>";
	}
    }

    if ($page != $x) {
	$url = $UserPrefs{"htmlroot"} . $SystemPrefs{"contestsuffix"}[$Contest] . "/" . ($page + 1) . $htmlfile;
	$retval .= " [<A HREF=\"$url\">Next >></A>]";
    }

    return $retval;
}  # goto_page





#--- Method Prolog -----------------------------------------------------------
#  Name: write_index
#
#  Parameters: 
#      Contest - The contest to collect data for. 1=RC5, 2=DES, etc.
#
#  Return Value: None.
#
#  Description:
#    Write main team index file
#
#
#--- End Method Prolog -------------------------------------------------------
sub write_index
{
    my $Contest = shift @_;
    my $sql;
    my $x;
    my $y;
    my ($date, $month, $day, $year);
    my ($time, $hour, $min, $sec);
    my ($mday, $wday);
    my @rate;
    my $printblocks;

    &Debugln("Entering write_index($Contest) ...");



    open(OUTFILE, ">" . $UserPrefs{"htmldir"} . $SystemPrefs{"contestsuffix"}[$Contest] . "/" . $UserPrefs{"htmlteam"}) or die "Personal Proxy Statistics : Cannot open " . $UserPrefs{"htmldir"} . $SystemPrefs{"contestsuffix"}[$Contest] . "/" . $UserPrefs{"htmlteam"} . " ($!)\n";
    &write_header($Contest);
    print OUTFILE "     <TABLE BORDER=0 CELLPADDING=0 CELLSPACING=0 WIDTH=\"100%\">\n";
    print OUTFILE "      <TR ALIGN=CENTER>\n";
    for ($y = $UserPrefs{"dayshistory"}; $y > 1; $y--) 
    {
	($year, $month, $mday, $hour, $min, $sec, $wday) = &nice_gmtime($SystemPrefs{"timebegin"} - ($y * (24*60*60)));
	print OUTFILE "       <TD CLASS=section>" . $SystemPrefs{"contestheaderblocktype"}[$Contest] . "s<BR>$Days[$wday]</TD>\n";
    }
    if ($UserPrefs{"dayshistory"} > 0) {
	print OUTFILE "       <TD CLASS=section>" . $SystemPrefs{"contestheaderblocktype"}[$Contest] . "s<BR>Yesterday</TD>\n",
    }
    if ($UserPrefs{"dayshistory"} >= 0) {
	print OUTFILE "       <TD CLASS=section>" . $SystemPrefs{"contestheaderblocktype"}[$Contest] . "s<BR>Today</TD>\n";
    }
    print OUTFILE "       <TD CLASS=section>" . $SystemPrefs{"contestheaderblocktype"}[$Contest] . "s<BR>Total</TD>\n",
                  "      </TR>\n";
    
    print OUTFILE "      <TR ALIGN=CENTER>\n";

    for ($y = $UserPrefs{"dayshistory"}; $y >= 0; $y--) 
    {
	($year, $month, $mday, $hour, $min, $sec, $wday) = &nice_gmtime($SystemPrefs{"timebegin"} - ($y * (24*60*60)));
#SELECT SUM(WU_SIZE) FROM `TBL_WU_CSC` WHERE DATE_FORMAT(SUBMITTED,"%Y-%m-%d") = "1999-07-11"
        $sql = "SELECT IFNULL(SUM(WU_SIZE),0) FROM " . $SystemPrefs{"contestwutable"}[$Contest] . 
		" WHERE SUBMITTED_DATE = \"$year-$month-$mday\"";
        my ($blocksthatday) = &DLookup($sql);
	if ($y > 0)
	{ 
	 $rate[$y] = &ratef(&rate_day($blocksthatday, $SystemPrefs{"contestfactor"}[$Contest], $Contest));
	}
	else
	{
	 $rate[$y] = &ratef(&rate_day($blocksthatday / ($SystemPrefs{"secstoday"} / (60*60*24)), $SystemPrefs{"contestfactor"}[$Contest], $Contest));
	}
	if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
	{
	    print OUTFILE "       <TD>$blocksthatday</TD>\n";
	}
	else
	{
	    if ($blocksthatday > "0")
	    {
		$printblocks = &ratef($blocksthatday * $SystemPrefs{"contestfactor"}[$Contest]);
	    }
	    else
	    {
		$printblocks = $blocksthatday;
	    }
	    print OUTFILE "       <TD>$printblocks</TD>\n";
	}
    }

    ($year, $month, $mday, $hour, $min, $sec, $wday) = &nice_gmtime($SystemPrefs{"timebegin"} - $UserPrefs{"dayshistory"} * (24*60*60));
#SELECT SUM(WU_SIZE) FROM TBL_WU_RC564 
    $sql = "SELECT SUM(WU_SIZE) FROM " . $SystemPrefs{"contestwutable"}[$Contest];
    my ($blocksoverall) = &DLookup($sql);
    
    if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
    {
	print OUTFILE "       <TD>$blocksoverall</TD>\n";
    }
    else
    {
	print OUTFILE "       <TD>" . &ratef($blocksoverall * $SystemPrefs{"contestfactor"}[$Contest]) . "</TD>\n";
    }
    print OUTFILE "      </TR>\n";
    print OUTFILE "      <TR ALIGN=CENTER>\n";

    # Print out day history list for all days before today
    for ($y = $UserPrefs{"dayshistory"}; $y >= 0; $y--)
    {
	print OUTFILE "       <TD NOWRAP>$rate[$y]" . $SystemPrefs{"contestblocktypeabbreviation"}[$Contest] . "/s</TD>\n";
    }


    $sql = "SELECT UNIX_TIMESTAMP(MAX(SUBMITTED_DATE)) FROM " . $SystemPrefs{"contestwutable"}[$Contest];
    my ($LastSubmitted) = &DLookup($sql);
    $sql = "SELECT UNIX_TIMESTAMP(MIN(SUBMITTED_DATE)) FROM " . $SystemPrefs{"contestwutable"}[$Contest];
    my ($FirstSubmitted) = &DLookup($sql);
    
    &Debugln("FirstSubmitted : $FirstSubmitted");
    &Debugln("LastSubmitted : $LastSubmitted");
        
    my $totaldays = int((($LastSubmitted - $FirstSubmitted) / (60*60*24)) +1);
    my $aveblocks;
    if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
    {
	$aveblocks = int($blocksoverall / $totaldays);
    }
    else
    {
	$aveblocks = &ratef($blocksoverall / $totaldays * $SystemPrefs{"contestfactor"}[$Contest]);
    }
    my $averate = &rate_day($blocksoverall / $totaldays, $SystemPrefs{"contestfactor"}[$Contest], $Contest);
    my $averatef = &ratef($averate);

    print OUTFILE "       <TD NOWRAP>$averatef" . $SystemPrefs{"contestblocktypeabbreviation"}[$Contest] . "/s</TD>\n";
    print OUTFILE "      </TR>\n";
    
    my $colsp = $UserPrefs{"dayshistory"} + 2;
    print OUTFILE "      <TR>\n",
                  "       <TD COLSPAN=$colsp>\n",
                  "        &nbsp;\n",
                  "       </TD>\n",
                  "      </TR>\n";

    print OUTFILE "      <TR><TD CLASS=title NOWRAP COLSPAN=$colsp ALIGN=CENTER>\n",
                  "       <STRONG>Graphical Team History for " . $SystemPrefs{"contestname"}[$Contest] . "</STRONG><BR>\n",
                  "       Rate (" . $SystemPrefs{"contestblockmeasure"}[$Contest] . "s/sec) by Day of Month\n",
                  "      </TD></TR>\n";

    print OUTFILE "      <TR><TD NOWRAP COLSPAN=$colsp ALIGN=CENTER>\n",
                  "       <BR>Total Days: " . $totaldays . " days - Average " . $SystemPrefs{"contestheaderblocktype"}[$Contest] . 
		  "s: $aveblocks " . $SystemPrefs{"contestblocktype"}[$Contest] . "s - Average Rate: $averatef" . $SystemPrefs{"contestblockmeasure"}[$Contest] . "s/sec\n",
                  "      </TD></TR>\n";

    print OUTFILE "      <TR>\n",
                  "       <TD COLSPAN=$colsp>\n",
                  "        &nbsp;\n",
                  "       </TD>\n",
                  "      </TR>\n";

    print OUTFILE "      <TR>\n",
                  "       <TD COLSPAN=$colsp ALIGN=CENTER>\n",
                  "        <IMG SRC=\"" . $UserPrefs{"htmlroot"} . $SystemPrefs{"contestsuffix"}[$Contest]. "/" . $UserPrefs{"gnuplotoutput"} . "\" WIDTH=" . $UserPrefs{"chartwidth"} . " HEIGHT=" . $UserPrefs{"chartheight"} . " ALT=\"" . $UserPrefs{"team"} . "\">\n",
                  "       </TD>\n",
                  "      </TR>\n";

    print OUTFILE "      <TR>\n",
                  "       <TD COLSPAN=$colsp>\n",
                  "        &nbsp;\n",
                  "       </TD>\n",
                  "      </TR>\n";

    print OUTFILE "      <TR><TD CLASS=note NOWRAP COLSPAN=$colsp ALIGN=CENTER>\n",
                  "       For past days: " . $SystemPrefs{"contestblockmeasure"}[$Contest] . "s/sec = " . 
		  $SystemPrefs{"contestblockweight"}[$Contest] . " * " . $SystemPrefs{"contestblocktype"}[$Contest] . "s / " . $SystemPrefs{"contestdivide"}[$Contest] . "<BR>\n",
                  "       For today: " . $SystemPrefs{"contestblockmeasure"}[$Contest] . "s/sec = (" . 
		  $SystemPrefs{"contestblockweight"}[$Contest] . " * " . $SystemPrefs{"contestblocktype"}[$Contest] . "s / " . $SystemPrefs{"contestdivide"}[$Contest] . ") / (seconds today / (60*60*24))\n",
                  "      </TD></TR>\n";
    print OUTFILE "     </TABLE>\n";
    &write_footer();
    close OUTFILE;


    open(GNUPLOTDATA, ">" . $UserPrefs{"htmldir"} . $SystemPrefs{"contestsuffix"}[$Contest] . "/" . $SystemPrefs{"gnuplotdata"}) or die "Personal Proxy Statistics : Cannot open " . $UserPrefs{"htmldir"} . $SystemPrefs{"contestsuffix"}[$Contest] . "/" . $SystemPrefs{"gnuplotdata"} . " ($!)\n";

    # Write out data file in format "Date, Rate".
#    &clear_nday_ave();

    my @row;
    $sql = "SELECT MIN(SUBMITTED_DATE) FROM " . $SystemPrefs{"contestwutable"}[$Contest];
    my $sth = &SelectSQL($sql);
    @row = $sth->fetchrow_array();
    my $Submitted = $row[0];
    my $WorkUnits = 0;
    $sql = "SELECT SUBMITTED_DATE, WU_SIZE FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " ORDER BY SUBMITTED_DATE";
    $sth = &SelectSQL($sql);
    my $line;
    my $averagerate;
    my ($rowyear, $rowmonth, $rowday);
    my ($subyear, $submonth, $subday);
    my ($yeartemp, $monthtemp, $daytemp);
    my ($datetemp, $timetemp);
    
    &clear_average;
    
    while (@row = $sth->fetchrow_array()) 
    {
    
	($subyear, $submonth, $subday) = split(/-/, $Submitted); 
	($rowyear, $rowmonth, $rowday) = split(/-/, $row[0]);

	if (($subyear == $rowyear) && ($submonth == $rowmonth) && ($subday == $rowday))
	{
	    $WorkUnits = $WorkUnits + $row[1];	
	}
	else
	{
    	    $averagerate = &calculate_average($WorkUnits, 2, $Contest);
	    $line = "$subyear/$submonth/$subday " . sprintf("%.2f", &rate_day($WorkUnits,$SystemPrefs{"contestfactor"}[$Contest], $Contest)) . " $averagerate";
	    print GNUPLOTDATA "$line\n";
    	    $Submitted = $row[0];
	    $WorkUnits = $row[1];
	    # Check if days are missing between writen line and new started day
    	    $sql = "SELECT DATE_FORMAT(DATE_ADD(\"$subyear-$submonth-$subday 00:00:00\", INTERVAL 1 DAY), \"%Y-%m-%d\")";
    	    ($datetemp) = &DLookup($sql);
	    ($yeartemp, $monthtemp, $daytemp) = split(/-/, $datetemp);
	    while (($yeartemp != $rowyear) || ($monthtemp != $rowmonth) || ($daytemp != $rowday))
	    {
		$averagerate = &calculate_average(0 , 2, $Contest);
	        $line = "$yeartemp/$monthtemp/$daytemp 0 $averagerate";
		print GNUPLOTDATA "$line\n";
    		$sql = "SELECT DATE_FORMAT(DATE_ADD(\"$yeartemp-$monthtemp-$daytemp 00:00:00\", INTERVAL 1 DAY), \"%Y-%m-%d\")";
    		($datetemp) = &DLookup($sql);
		($yeartemp, $monthtemp, $daytemp) = split(/-/, $datetemp);	     
	    }
	}
    }

    
    $averagerate = &calculate_average($WorkUnits, 2, $Contest);
    if ($Submitted ne "0000-00-00" && $Submitted ne "")
    {
     if ($Submitted eq "$GMTTS[0]-$GMTTS[1]-$GMTTS[2]")
     {
      $line = "$subyear/$submonth/$subday " . sprintf("%.2f", &rate_day($WorkUnits  / ($SystemPrefs{"secstoday"} / (60*60*24)),$SystemPrefs{"contestfactor"}[$Contest], $Contest)) . " $averagerate";
     }
     else
     {
      $line = "$subyear/$submonth/$subday " . sprintf("%.2f", &rate_day($WorkUnits,$SystemPrefs{"contestfactor"}[$Contest], $Contest)) . " $averagerate";
     }
     print GNUPLOTDATA "$line\n";
    }
    

#    for ($x = $day - 1; $x > 1; $x--) {
#	($year, $mon, $mday, $hour, $min, $sec, $wday) = &nice_gmtime($time_begin - ($x - 1) * (24*60*60));
#	$rate = sprintf("%.2f", &rate_day($blocks[$x],$SystemPrefs{"contestfactor"}[$Contest], $Contest));
#	$line = "$year/$mon/$mday $rate";
#	&push_nday_ave($rate);
#	$rate = &calc_nday_ave();
#	$line .= " $rate";
#	print GNUPLOTDATA "$line\n";
#    }
#
#    ($year, $mon, $mday, $hour, $min, $sec, $wday) = &nice_gmtime($time_begin);
#    $rate = sprintf("%.2f", &rate_day($blocks[1],$SystemPrefs{"contestfactor"}[$Contest], $Contest) / ($secs_today / (60*60*24)));
#    $line = "$year/$mon/$mday $rate";
#    &push_nday_ave($rate);
#    $rate = &calc_nday_ave();
#    $line .= " $rate";
#    print GNUPLOTDATA "$line\n";
#
    close GNUPLOTDATA;


    my $range = "*";
    if ($blocksoverall == 0) { $range = 1; };

    $sql = "SELECT DATE_FORMAT(MIN(SUBMITTED_DATE), \"%Y/%m/%d\") FROM " . $SystemPrefs{"contestwutable"}[$Contest];
    ($FirstSubmitted) =&DLookup($sql);

    &plot_gui_graph($UserPrefs{"htmldir"} . $SystemPrefs{"contestsuffix"}[$Contest] . "/", "History Plot for " . $SystemPrefs{"contestname"}[$Contest] . "\\nTeam " . $UserPrefs{"team"}, $FirstSubmitted, "$GMTTS[0]/$GMTTS[1]/$GMTTS[2]", $range, $averate, $Contest);

    &Debugln("Exiting write_index($Contest) ...");
}  # write_index




#--- Method Prolog -----------------------------------------------------------
#  Name: write_exec
#
#  Parameters: 
#      Contest - The contest to collect data for. 1=RC5, 2=DES, etc.
#
#  Return Value: None.
#
#  Description:
#      Write Executive Team Summary.
#
#
#--- End Method Prolog -------------------------------------------------------
sub write_exec
{
    my $Contest = shift @_;
    &Debugln("Entering write_exec($Contest) ...");
    my $sql;
    my @row;
    my ($yeartemp, $monthtemp, $daytemp);
    my $firstblockdate;
    my $lastblockdate;
    my ($LastSubmitted,$FirstSubmitted);
    my ($LastSubmittedTime,$FirstSubmittedTime);
    my ($totaldays, $aveblocks, $averate, $averatef);    
    my ($year, $month, $mday, $hour, $min, $sec, $wday);
    my ($blocksyesterday, $rateyesterday);
#    my $fblocksyesterday;
    my $percentcomplete;
    my $contestblocks;
    my $keyschecked;
    my ($y, $z);
    my ($numos, $ostemp);
    my ($numcpu, $cputemp);
    my ($numdomain, $domaintemp);
    my ($numcountry, $countrytemp);
    my ($numver, $vertemp, $maxversion);
    my $blockdate;
    my $rate;


    $sql = "SELECT MIN(SUBMITTED_DATE) FROM " . $SystemPrefs{"contestwutable"}[$Contest];
    ($FirstSubmitted) =&DLookup($sql);
    ($yeartemp, $monthtemp, $daytemp) = split(/-/, $FirstSubmitted);

    if (($yeartemp <= "0" ) && (($monthtemp <= "0") || ($daytemp <= "0")))
    {
	$firstblockdate = "No " . $SystemPrefs{"contestheaderblocktype"}[$Contest] . "s Submitted";
    } 
    else 
    {
	$firstblockdate = "$daytemp $Months[$monthtemp-1] $yeartemp GMT";
    }
    if ($firstblockdate eq "No " . $SystemPrefs{"contestheaderblocktype"}[$Contest] . "s Submitted") 
    {
	$lastblockdate = "No " . $SystemPrefs{"contestheaderblocktype"}[$Contest] . "s Submitted";
    } 
    else 
    {
	$sql = "SELECT MAX(SUBMITTED_DATE) FROM " . $SystemPrefs{"contestwutable"}[$Contest];
	($LastSubmitted) =&DLookup($sql);
	($yeartemp, $monthtemp, $daytemp) = split(/-/, $LastSubmitted);
	$lastblockdate = "$daytemp $Months[$monthtemp-1] $yeartemp GMT";
    }

    $sql = "SELECT TIME_TO_SEC(MAX(SUBMITTED_TIME)) FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE SUBMITTED_DATE = \"$LastSubmitted\"";
    ($LastSubmittedTime) = &DLookup($sql);
    $sql = "SELECT TIME_TO_SEC(MIN(SUBMITTED_TIME)) FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE SUBMITTED_DATE = \"$FirstSubmitted\"";
    ($FirstSubmittedTime) =&DLookup($sql);

    $sql = "SELECT UNIX_TIMESTAMP(\"$LastSubmitted 00:00:00\")";
    ($LastSubmitted) = &DLookup($sql);
    $sql = "SELECT UNIX_TIMESTAMP(\"$FirstSubmitted 00:00:00\")";
    ($FirstSubmitted) =&DLookup($sql);

    $sql = "SELECT SUM(WU_SIZE) FROM " . $SystemPrefs{"contestwutable"}[$Contest];
    my ($blocksoverall) = &DLookup($sql);

    if ($firstblockdate eq "No " . $SystemPrefs{"contestheaderblocktype"}[$Contest] . "s Submitted") 
    {
	$totaldays = "0";
	$aveblocks = "0";
	$averatef = "0 ";
    }
    else
    {
	$totaldays = int((($LastSubmitted - $FirstSubmitted) / (60*60*24)) + 1);
	if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
	{
    	    $aveblocks = int($blocksoverall / $totaldays);
	}
	else
	{
	    $aveblocks = &ratef($blocksoverall / $totaldays * $SystemPrefs{"contestfactor"}[$Contest]);
	}
	$averate = &rate_day($blocksoverall / $totaldays, $SystemPrefs{"contestfactor"}[$Contest], $Contest);
	$averatef = &ratef($averate);

    }


    # Print out the Last Seen column.
    my $lastseen;
    my $lastseencolor;
    if ($firstblockdate eq "No " . $SystemPrefs{"contestheaderblocktype"}[$Contest] . "s Submitted") {
	$lastseen = "Never";
	$lastseencolor = $UserPrefs{"textcolor"};
    } 
    else 
    {
	$lastseen = &ls($SystemPrefs{"timebegin"}-($LastSubmitted+$LastSubmittedTime));
	$lastseencolor = &ls_color($SystemPrefs{"timebegin"}-($LastSubmitted+$LastSubmittedTime));
    }


    ($year, $month, $mday, $hour, $min, $sec, $wday) = &nice_gmtime($SystemPrefs{"timebegin"} - (1 * (24*60*60)));
#SELECT SUM(WU_SIZE) FROM `TBL_WU_CSC` WHERE DATE_FORMAT(SUBMITTED,"%Y-%m-%d") = "1999-07-11"
    $sql = "SELECT SUM(WU_SIZE) FROM " . $SystemPrefs{"contestwutable"}[$Contest] . 
	   " WHERE SUBMITTED_DATE = \"$year-$month-$mday\"";
    ($blocksyesterday) = &DLookup($sql);

    $rateyesterday = &ratef(&rate_day($blocksyesterday,$SystemPrefs{"contestfactor"}[$Contest], $Contest));
    $keyschecked = &keysf($blocksoverall,$Contest);

    if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
    {
	$contestblocks = &get_contest_blocks($Contest);
	$percentcomplete = &percentf($blocksoverall, 6, $Contest);
    }
#    else
#    {
#	$fblocksyesterday = &ratef($blocksyesterday * $SystemPrefs{"contestfactor"}[$Contest]);
#    }

    open(OUTFILE, ">" . $UserPrefs{"htmldir"} . $SystemPrefs{"contestsuffix"}[$Contest] . "/" . $UserPrefs{"htmlexec"}) or die "Personal Proxy Statistics : Cannot open " . $UserPrefs{"htmldir"} . $SystemPrefs{"contestsuffix"}[$Contest] . "/" . $UserPrefs{"htmlexec"} . " ($!)\n";
    &write_header($Contest);
    print OUTFILE "     <TABLE BORDER=0 CELLPADDING=0 CELLSPACING=0 WIDTH=\"100%\">\n";
    print OUTFILE "      <TR ALIGN=CENTER>\n",
	          "       <TD CLASS=title NOWRAP>\n",
                  "        <STRONG>Team " . $UserPrefs{"team"} . "</STRONG><BR>\n",
                  "        Executive Team Summary\n",
                  "       </TD>\n",
                  "      </TR>\n";

    print OUTFILE "      <TR>\n",
                  "       <TD>\n",
                  "        &nbsp;\n",
                  "       </TD>\n",
                  "      </TR>\n";

    print OUTFILE "      <TR ALIGN=CENTER>\n",
                  "       <TD VALIGN=TOP WIDTH=\"100%\">\n",
                  "        <TABLE BORDER=0 CELLPADDING=2 CELLSPACING=0>\n";

    print OUTFILE "         <TR>\n",
                  "          <TD CLASS=subtitle NOWRAP COLSPAN=2 ALIGN=CENTER>\n",
                  "           Team Summary for " . $UserPrefs{"team"} . "\n",
                  "          </TD>\n",
                  "         </TR>\n";

    if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
    {
	print OUTFILE "         <TR>\n",
                  "          <TD NOWRAP>\n",
                  "           Total " . $SystemPrefs{"contestheaderblocktype"}[$Contest] . "s to Search\n",
                  "          </TD>\n",
                  "          <TD NOWRAP ALIGN=RIGHT>\n",
                  "           $contestblocks " . $SystemPrefs{"contestblocktype"}[$Contest] . "s\n",
                  "          </TD>\n",
                  "         </TR>\n";

	print OUTFILE "         <TR>\n",
                  "          <TD NOWRAP>\n",
                  "           Total " . $SystemPrefs{"contestheaderblocktype"}[$Contest] . "s Checked\n",
                  "          </TD>\n",
                  "          <TD NOWRAP ALIGN=RIGHT>\n",
                  "           $blocksoverall " . $SystemPrefs{"contestblocktype"}[$Contest] . "s\n",
                  "          </TD>\n",
                  "         </TR>\n";

	print OUTFILE "         <TR>\n",
                  "          <TD NOWRAP>\n",
                  "           Keyspace Exhausted\n",
                  "          </TD>\n",
                  "          <TD NOWRAP ALIGN=RIGHT>\n",
                  "           $percentcomplete%\n",
                  "          </TD>\n",
                  "         </TR>\n";
    }

    print OUTFILE "         <TR>\n",
                  "          <TD NOWRAP>\n",
                  "           Total " . $SystemPrefs{"contestheaderblockmeasure"}[$Contest] . "s Checked\n",
                  "          </TD>\n",
                  "          <TD NOWRAP ALIGN=RIGHT>\n",
                  "           $keyschecked\n",
                  "          </TD>\n",
                  "         </TR>\n";

    $sql = "SELECT COUNT(PARTICIPANT_ID) FROM TBL_PARTICIPANT WHERE RETIRE_TO_ID = 0";
    my ($numemail) =&DLookup($sql);
    print OUTFILE "         <TR>\n",
                  "          <TD NOWRAP>\n",
                  "           Total Participants\n",
                  "          </TD>\n",
                  "          <TD NOWRAP ALIGN=RIGHT>\n",
                  "           $numemail\n",
                  "          </TD>\n",
                  "         </TR>\n";

    if ($firstblockdate eq "No " . $SystemPrefs{"contestheaderblocktype"}[$Contest] . "s Submitted") 
    {
        if ($UserPrefs{"showbyhost"}) 
	{
#	print OUTFILE "         <TR>\n",
#                      "          <TD NOWRAP>\n",
#                      "           Total Unique IPs\n",
#                      "          </TD>\n",
#                      "          <TD NOWRAP ALIGN=RIGHT>\n",
#                      "           $numipaddr\n",
#                      "          </TD>\n",
#                      "         </TR>\n";

    	    $sql = "SELECT COUNT(HOST_ID) FROM TBL_HOST";
    	    my ($numhost) =&DLookup($sql);
    	    print OUTFILE "         <TR>\n",
                      "          <TD NOWRAP>\n",
                      "           Total Hosts\n",
                      "          </TD>\n",
                      "          <TD NOWRAP ALIGN=RIGHT>\n",
                      "           $numhost\n",
                      "          </TD>\n",
                      "         </TR>\n";
	}


	$sql = "SELECT MAX(OS_ID) FROM " . $SystemPrefs{"contestwutable"}[$Contest];
	my ($os) =&DLookup($sql);

	$numos = 0;
	for ($y = $os; $y > 0; $y--) 
	{
	    $sql = "SELECT IFNULL(OS_ID,\"-\") FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE OS_ID = \"$y\" LIMIT 1";
    	    ($ostemp) =&DLookup($sql);
	    if ($ostemp ne "-") 
	    {
		$numos++;
	    }	
	}

	print OUTFILE "         <TR>\n",
                  "          <TD NOWRAP>\n",
                  "           Total Operating Systems\n",
                  "          </TD>\n",
                  "          <TD NOWRAP ALIGN=RIGHT>\n",
                  "           $numos\n",
                  "          </TD>\n",
                  "         </TR>\n";


	$sql = "SELECT MAX(CPU_ID) FROM " . $SystemPrefs{"contestwutable"}[$Contest];
	my ($cpu) =&DLookup($sql);

	$numcpu = 0;
	for ($y = $cpu; $y > 0; $y--) 
	{
	    $sql = "SELECT IFNULL(CPU_ID,\"-\") FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE CPU_ID = \"$y\" LIMIT 1";
    	    ($cputemp) =&DLookup($sql);
	    if ($cputemp != "-") 
	    {
		$numcpu++;
	    }	
	}
	print OUTFILE "         <TR>\n",
                  "          <TD NOWRAP>\n",
                  "           Total CPUs\n",
                  "          </TD>\n",
                  "          <TD NOWRAP ALIGN=RIGHT>\n",
                  "           $numcpu\n",
                  "          </TD>\n",
                  "         </TR>\n";


	$sql = "SELECT MAX(VERSION) FROM " . $SystemPrefs{"contestwutable"}[$Contest];
	my ($maxversion) =&DLookup($sql);

	$numcpu = 0;
	$y = 0;
	while ($y < $maxversion) 
	{
	    $sql = "SELECT MIN(VERSION) FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE VERSION > \"$y\"";
    	    ($y) =&DLookup($sql);
	    $numver++;
	}
	print OUTFILE "         <TR>\n",
                  "          <TD NOWRAP>\n",
                  "           Total Client Versions\n",
                  "          </TD>\n",
                  "          <TD NOWRAP ALIGN=RIGHT>\n",
                  "           $numver\n",
                  "          </TD>\n",
                  "         </TR>\n";

	$sql = "SELECT MAX(COUNTRY_ID) FROM TBL_PARTICIPANT";
	my ($maxdomain) = &DLookup($sql);
	$numdomain = 0;
	$y = 0;
	while ($y ne $maxdomain) 
	{
	    $sql = "SELECT MIN(COUNTRY_ID) FROM TBL_PARTICIPANT WHERE (COUNTRY_ID > \"$y\")";
    	    ($y) =&DLookup($sql);
    	    $numdomain++;
        }

	print OUTFILE "         <TR>\n",
                  "          <TD NOWRAP>\n",
                  "           Total Countrys\n",
                  "          </TD>\n",
                  "          <TD NOWRAP ALIGN=RIGHT>\n",
                  "           $numdomain\n",
                  "          </TD>\n",
                  "         </TR>\n";
    }
    print OUTFILE "         <TR>\n",
                  "          <TD NOWRAP>\n",
                  "           Time Working\n",
                  "          </TD>\n",
                  "          <TD NOWRAP ALIGN=RIGHT>\n",
                  "           $totaldays days\n",
                  "          </TD>\n",
                  "         </TR>\n";

    print OUTFILE "         <TR>\n",
                  "          <TD NOWRAP>\n",
                  "           Last Seen\n",
                  "          </TD>\n",
                  "          <TD CLASS=$lastseencolor NOWRAP ALIGN=RIGHT>\n",
                  "           $lastseen\n",
                  "          </TD>\n",
                  "         </TR>\n";

    print OUTFILE "         <TR>\n",
                  "          <TD NOWRAP>\n",
                  "           First " . $SystemPrefs{"contestheaderblocktype"}[$Contest] . " Submitted\n",
                  "          </TD>\n",
                  "          <TD NOWRAP ALIGN=RIGHT>\n",
                  "           $firstblockdate\n",
                  "          </TD>\n",
                  "         </TR>\n";

    print OUTFILE "         <TR>\n",
                  "          <TD NOWRAP>\n",
                  "           Last " . $SystemPrefs{"contestheaderblocktype"}[$Contest] . " Submitted\n",
                  "          </TD>\n",
                  "          <TD NOWRAP ALIGN=RIGHT>\n",
                  "           $lastblockdate\n",
                  "          </TD>\n",
                  "         </TR>\n";

    print OUTFILE "         <TR>\n",
                  "          <TD NOWRAP>\n",
                  "           Overall Rate\n",
                  "          </TD>\n",
                  "          <TD NOWRAP ALIGN=RIGHT>\n",
                  "           $averatef" . $SystemPrefs{"contestblockmeasure"}[$Contest] . "s/sec\n",
                  "          </TD>\n",
                  "         </TR>\n";

    print OUTFILE "         <TR>\n",
                  "          <TD NOWRAP>\n",
                  "           Average " . $SystemPrefs{"contestheaderblocktype"}[$Contest] . "s\n",
                  "          </TD>\n",
                  "          <TD NOWRAP ALIGN=RIGHT>\n",
                  "           $aveblocks per day\n",
                  "          </TD>\n",
                  "         </TR>\n";

    print OUTFILE "         <TR>\n",
                  "          <TD NOWRAP>\n",
                  "           Distributed.Net Rank\n",
                  "          </TD>\n",
                  "          <TD NOWRAP ALIGN=RIGHT>\n",
                  "           <A HREF=\"" . $SystemPrefs{"stats-server"} . "/team/tmsummary.php" . $SystemPrefs{"conteststats"}[$Contest] . "&team=" . $UserPrefs{"teamid"} . "\">" . $UserPrefs{"team"} . "</A>\n",
                  "          </TD>\n",
                  "         </TR>\n";

    print OUTFILE "        </TABLE>\n",
                  "       </TD>\n",
                  "      </TR>\n";

    print OUTFILE "      <TR>\n",
                  "       <TD>\n",
                  "        &nbsp;\n",
                  "       </TD>\n",
                  "      </TR>\n";

    if ($blocksyesterday == 0) {
	print OUTFILE "      <TR ALIGN=CENTER>\n",
                      "       <TD NOWRAP>\n",
                      "        No one submitted any " . $SystemPrefs{"contestblocktype"}[$Contest] . "s yesterday, someone needs to get cracking!\n",
                      "       </TD>\n",
                      "      </TR>\n";
    } else {
	$percentcomplete = &percentf($blocksyesterday, 8, $Contest);
	print OUTFILE "      <TR ALIGN=CENTER>\n",
                      "       <TD NOWRAP>\n";
	if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
    	{
            print OUTFILE "        $blocksyesterday " . $SystemPrefs{"contestheaderblocktype"}[$Contest] . "s were completed yesterday ($percentcomplete% of the keyspace)\n";
	}
	else
	{
            print OUTFILE "        " . &ratef($blocksyesterday * $SystemPrefs{"contestfactor"}[$Contest]) . $SystemPrefs{"contestheaderblocktype"}[$Contest] . "s were completed yesterday\n";
	}
	print OUTFILE "        <BR>at a sustained rate of $rateyesterday" . $SystemPrefs{"contestblockmeasure"}[$Contest] ."s/sec!\n",
                      "       </TD>\n",
                      "      </TR>\n";
    }

    print OUTFILE "      <TR>\n",
                  "       <TD>\n",
                  "        &nbsp;\n",
                  "       </TD>\n",
                  "      </TR>\n";

    print OUTFILE "      <TR ALIGN=CENTER>\n",
                  "       <TD NOWRAP>\n",
                  "        Team History for the Last " . $UserPrefs{"blockhistory"} . " Days\n",
                  "       </TD>\n",
                  "      </TR>\n";

    print OUTFILE "      <TR>\n",
                  "       <TD>\n",
                  "        &nbsp;\n",
                  "       </TD>\n",
                  "      </TR>\n";

    print OUTFILE "      <TR ALIGN=CENTER>\n",
                  "       <TD VALIGN=TOP WIDTH=\"100%\">\n",
                  "        <TABLE BORDER=0 CELLPADDING=2 CELLSPACING=0>\n";

    print OUTFILE "         <TR ALIGN=CENTER>\n",
                  "          <TD CLASS=section NOWRAP>\n",
                  "           Day History\n",
                  "          </TD>\n",
                  "          <TD CLASS=section NOWRAP>\n",
                  "           " . $SystemPrefs{"contestheaderblocktype"}[$Contest] . "s Submited\n",
                  "          </TD>\n";
    if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
    {
	print OUTFILE "          <TD CLASS=section NOWRAP>\n",
                  "           " . $SystemPrefs{"contestheaderblockmeasure"}[$Contest] ."s Checked\n",
                  "          </TD>\n";
    }
    print OUTFILE "          <TD CLASS=section NOWRAP>\n",
                  "          " . $SystemPrefs{"contestheaderblockmeasure"}[$Contest] ."  Rate\n",
                  "          </TD>\n";
    if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
    {
        print OUTFILE "          <TD CLASS=section NOWRAP>\n",
                  "           " . $SystemPrefs{"contestheaderblockmeasure"}[$Contest] ."space\n";
    }
    print OUTFILE "          </TD>\n",
                  "         </TR>\n";

    for ($y = 0; $y <= $UserPrefs{"blockhistory"}; $y++) 
    {
	($year, $month, $mday, $hour, $min, $sec, $wday) = &nice_gmtime($SystemPrefs{"timebegin"} - ($y * (24*60*60)));
	$blockdate = "$mday $Months[$month-1] $year GMT";

#SELECT SUM(WU_SIZE) FROM `TBL_WU_CSC` WHERE DATE_FORMAT(SUBMITTED,"%Y-%m-%d") = "1999-07-11"
        $sql = "SELECT IFNULL(SUM(WU_SIZE),0) FROM " . $SystemPrefs{"contestwutable"}[$Contest] . 
		" WHERE SUBMITTED_DATE = \"$year-$month-$mday\"";
        my ($blocksthatday) = &DLookup($sql);

	if ($y > 0)
	{ 
	 $rate = &ratef(&rate_day($blocksthatday, $SystemPrefs{"contestfactor"}[$Contest], $Contest));
	}
	else
	{
	 $rate = &ratef(&rate_day($blocksthatday / ($SystemPrefs{"secstoday"} / (60*60*24)), $SystemPrefs{"contestfactor"}[$Contest], $Contest));
	}
	$keyschecked = &keysf($blocksthatday, $Contest);
	$percentcomplete = &percentf($blocksthatday, 8, $Contest);
	if ($y % 2 == 0) 
	{
	    print OUTFILE "         <TR CLASS=even>\n";
	} 
	else 
	{
	    print OUTFILE "         <TR CLASS=odd>\n";
	}
	print OUTFILE "          <TD NOWRAP ALIGN=LEFT>\n",
	              "           $blockdate\n",
                      "          </TD>\n",
                      "          <TD NOWRAP ALIGN=RIGHT>\n";
	if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
	{
    	    print OUTFILE "           $blocksthatday " . $SystemPrefs{"contestblocktype"}[$Contest] . "s\n";
	}
	else
	{
    	    print OUTFILE "           " . &ratef($blocksthatday * $SystemPrefs{"contestfactor"}[$Contest]) . $SystemPrefs{"contestblocktype"}[$Contest] . "s\n";
	}

	print OUTFILE "          </TD>\n";
	if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
	{
            print OUTFILE   "          <TD NOWRAP ALIGN=RIGHT>\n",
                      "           $keyschecked \n",
                      "          </TD>\n";
	}
        print OUTFILE "          <TD NOWRAP ALIGN=RIGHT>\n",
    	              "           $rate" . $SystemPrefs{"contestblockmeasure"}[$Contest] . "s/sec\n",
                      "          </TD>\n";
	if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
	{
    	    print OUTFILE "          <TD NOWRAP ALIGN=RIGHT>\n",
                      "           $percentcomplete%\n";
	}
        print OUTFILE "          </TD>\n",
                      "         </TR>\n";
    }

    print OUTFILE "        </TABLE>\n",
                  "       </TD>\n",
                  "      </TR>\n";

    print OUTFILE "     </TABLE>\n";
    &write_footer();
    close OUTFILE;

    &Debugln("Exiting write_exec($Contest) ...");
}  # write_exec

#--- Method Prolog -----------------------------------------------------------
#  Name: write_byhost
#
#  Parameters: 
#      Contest - The contest to collect data for. 1=RC5, 2=DES, etc.
#
#  Return Value: None.
#
#  Description:
#    Write stats ordered by host.
#
#--- End Method Prolog -------------------------------------------------------
sub write_byhost
{
    my $Contest = shift @_;
    &Debugln("Entering write_byhost($Contest) ...");

    my $count = 0;
    my $pagecount = 1;
    my $msg;
    my $displacement;
    my ($y, $z, $host);
    my $lastseen;
    my $lastseentime;
    my $lastseencolor;
    my $sql;
    my $sth;
    my @row;
    my $sth2;
    my @row2;
    my (@hosts,@blocks);
    my ($year, $month, $day, $hour, $min, $sec, $wday);
    my ($head, $tail);
    my ($hostdir, $hostlink);
    my $percentofteam;
    my $blocksthatday;
    my ($domain, $icon, $label);
    my $printblocks;

    &write_section_header("", "Hostname", $UserPrefs{"htmlhost"}, $Contest);

    for ($y = $UserPrefs{"dayshistory"}+1; $y >= 0; $y--) {
	$pageblocks[$y] = 0;
    }

    $sql = "SELECT SUM(WU_SIZE) FROM " . $SystemPrefs{"contestwutable"}[$Contest];
    my ($blocksoverall) = &DLookup($sql);

    my $numhosts = 0;
    
#SELECT HOST_ID,SUM(WU_SIZE) FROM `TBL_WU_RC564` GROUP BY 1 ORDER BY 2 DESC
    $sql = "SELECT HOST_ID,SUM(WU_SIZE) FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " GROUP BY 1 ORDER BY 2 DESC";
    $sth = &SelectSQL($sql);

    while (@row = $sth->fetchrow_array()) 
    {
	$numhosts++;
	$hosts[$numhosts] = $row[0];
	$blocks[$numhosts] = $row[1];
    }

    for ($z = 1; $z <= $numhosts; $z++)
    {
	$host = $hosts[$z];
	$count += 1;

	# Start a new file if maxinum number of rows has been reached.
	if ($count > $UserPrefs{"maxrows"} * $pagecount) 
	{
	    &write_section_footer($pagecount, $numhosts, 0, $UserPrefs{"htmlhost"}, "Please click on the highest level domain name for detailed host information.", $Contest, 0);

	    for ($y = $UserPrefs{"dayshistory"}+1; $y >= 0; $y--) {
		$pageblocks[$y] = 0;
	    }

	    $pagecount += 1;
	    &write_section_header($pagecount, "Hostname", $UserPrefs{"htmlhost"}, $Contest);
	}

	if ($count % 2 == 0) {
	    print OUTFILE "      <TR CLASS=even ALIGN=RIGHT>\n";
	} else {
	    print OUTFILE "      <TR CLASS=odd ALIGN=RIGHT>\n";
	}

# Print out the Rank and Movement column.
#	$displacement = $hosts_ry{$host} == 0 ? $numhost - $count : $hosts_ry{$host} - $count;
#	$msg = &create_move_msg($displacement);
#        print OUTFILE "       <TD NOWRAP ALIGN=LEFT>&nbsp;$count$msg</TD>\n";
        print OUTFILE "       <TD NOWRAP ALIGN=LEFT>&nbsp;$count</TD>\n";

	$domain = undef;
        $label = undef;
	$icon = undef;
	$sql = "SELECT HOSTNAME FROM TBL_HOST WHERE (HOST_ID = \"$host\") LIMIT 1";
	my ($hostname) = &DLookup($sql);

	$sql = "SELECT COUNTRY_ID FROM TBL_HOST WHERE (HOST_ID = \"$host\") LIMIT 1";
	my ($domain) = &DLookup($sql);

	if (defined($domain)) 
	{
	    $sql = "SELECT COUNTRY FROM TBL_COUNTRY WHERE (COUNTRY_ID = \"$domain\") LIMIT 1";
	    ($label) = &DLookup($sql);
	    if (defined($label))
	    {
	        $sql = "SELECT COUNTRY_ICON FROM TBL_COUNTRY WHERE (COUNTRY_ID = \"$domain\") LIMIT 1";
		($icon) = &DLookup($sql);
		$icon = ($icon . "." . $UserPrefs{"imageext"});
	    }
	    else
	    {
		$label = "Unknown";
		$icon = ("unknown." . $UserPrefs{"imageext"});
	    }
	}

	$hostdir = "h" . $host;
	($head, $tail) = split /\./, $hostname, 2;

	if (defined($tail)) 
	{ 
	    $hostlink = "<A HREF=\"" . $UserPrefs{"htmlroot"} . $SystemPrefs{"contestsuffix"}[$Contest]. "/" . "$hostdir/" . $UserPrefs{"htmlindvhost"} . "\">$head</A>.$tail";
	} 
	else 
	{
	    $hostlink = "<A HREF=\"" . $UserPrefs{"htmlroot"} . $SystemPrefs{"contestsuffix"}[$Contest]. "/" . "$hostdir/" . $UserPrefs{"htmlindvhost"} . "\">$head</A>";
	}

	print OUTFILE "       <TD NOWRAP ALIGN=LEFT><IMG SRC=\"" . $UserPrefs{"htmlroot"} . $UserPrefs{"icondir"} . "flags/$icon\" ALIGN=BOTTOM BORDER=0 WIDTH=14 HEIGHT=14 ALT=\"$label\">&nbsp;$hostlink</TD>\n";
 
	for ($y = $UserPrefs{"dayshistory"} - 3; $y >= 0; $y--) 
	{
	    ($year, $month, $day, $hour, $min, $sec, $wday) = &nice_gmtime($SystemPrefs{"timebegin"} - $y * (24*60*60));
	    $sql = "SELECT IFNULL(SUM(WU_SIZE),\"-\") FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE ((HOST_ID = \"$host\") AND (SUBMITTED_DATE = \"$year-$month-$day\"))";
	    ($blocksthatday) = &DLookup($sql);
	    if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
	    {
		print OUTFILE "       <TD>$blocksthatday</TD>\n";
	    }
	    else
	    {
		if ($blocksthatday ne "-")
		{
		    $printblocks = &ratef($blocksthatday * $SystemPrefs{"contestfactor"}[$Contest]);
		}
		else
		{
		    $printblocks = $blocksthatday;
		}
		print OUTFILE "       <TD>$printblocks</TD>\n";
	    }
	    if ($blocksthatday ne "-")
	    {
	    	$pageblocks[$y+1] += $blocksthatday;
	    }
	}

	
	$sql = "SELECT IFNULL(SUM(WU_SIZE),\"-\") FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE (HOST_ID = \"$host\")";
	($blocksthatday) = &DLookup($sql);
	if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
	{
	    print OUTFILE "       <TD>$blocksthatday</TD>\n";
	}
	else
	{
	    if ($blocksthatday > "0")
	    {
		$printblocks = &ratef($blocksthatday * $SystemPrefs{"contestfactor"}[$Contest]);
	    }
	    else
	    {
		$printblocks = $blocksthatday;
	    }
	    print OUTFILE "       <TD>$printblocks</TD>\n";
	}
	if ($blocksthatday > 0)
	{
    	    $pageblocks[0] += $blocksthatday;
	}

# Print out the Last Seen column.
        $sql = "SELECT MAX(SUBMITTED_DATE) FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE (HOST_ID = \"$host\")";
	my ($LastSubmittedDate) = &DLookup($sql);
        $sql = "SELECT MAX(SUBMITTED_TIME) FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE ((SUBMITTED_DATE = \"$LastSubmittedDate\") AND (HOST_ID = \"$host\"))";
	my ($LastSubmittedTime) = &DLookup($sql);
	$sql = "SELECT UNIX_TIMESTAMP(\"$LastSubmittedDate $LastSubmittedTime\")";
	my ($LastTime) = &DLookup($sql);

	$lastseentime = ($SystemPrefs{"timebegin"}-($LastTime));
	$lastseen = &ls($lastseentime);
	$lastseencolor = &ls_color($lastseentime);

	print OUTFILE "       <TD CLASS=$lastseencolor NOWRAP>$lastseen</TD>\n";

# Print out the Percent column.

	if (($blocks[$z] > 0) && ($blocksoverall > 0))
	{
	    $percentofteam = sprintf("%.2f", 100 * $blocks[$z] / $blocksoverall);
	}
	else
	{
	    $percentofteam = sprintf("%.2f", 0);
	}

	print OUTFILE "       <TD>$percentofteam%</TD>\n",
	              "      </TR>\n";
    }
    
    &write_section_footer($pagecount, $numhosts, 0, $UserPrefs{"htmlhost"}, "Please click on the highest level domain name for detailed host information.", $Contest, 0);
   
    &write_indvhost($Contest);

    &Debugln("Exiting write_byhost($Contest) ...");
}  # write_byhost


#--- Method Prolog -----------------------------------------------------------
#  Name: write_indvhost
#
#  Parameters: 
#      Contest - The contest to collect data for. 1=RC5, 2=DES, etc.
#
#  Return Value: None.
#
#  Description:
#     Routine to write individual host info.
#
#--- End Method Prolog -------------------------------------------------------
sub write_indvhost
{
    my $Contest = shift @_;
    my @hosts;
    my @blocks;
    my $count = 0;
    my $msg;
    my $displacement;
    my $numhosts;
    my ($y, $z);
    my ($firstblockdate, $lastblockdate);
    my ($lastseentime, $lastseen, $lastseencolor);
    my ($daysworking, $aveblocks, $averate, $averatef);    
    my ($year, $mon, $mday, $hour, $min, $sec, $wday);
    my ($host, $hostdir, $sql);
    my ($label, $icon, $domain);
    my ($contestblocks, $keyschecked, $percentcomplete, $percentofteam);
    my $sth;
    my @row;
    my ($blocksthathost, $rate);
    my ($blocksyesterday, $rateyesterday);

    $sql = "SELECT HOST_ID,SUM(WU_SIZE) FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " GROUP BY 1 ORDER BY 2 DESC";
    $sth = &SelectSQL($sql);

    while (@row = $sth->fetchrow_array()) 
    {
	$numhosts++;
	$hosts[$numhosts] = $row[0];
	$blocks[$numhosts] = $row[1];
    }

    $sql = "SELECT IFNULL(SUM(WU_SIZE),0) FROM " . $SystemPrefs{"contestwutable"}[$Contest];
    my ($blocksoverall) =&DLookup($sql);

    for ($z = 1; $z <= $numhosts; $z++)
    {
	$host = $hosts[$z];
	$count += 1;

	# Print out the Rank and Movement column.
#	$displacement = $hosts_ry{$host} == 0 ? $numhost - $count : $hosts_ry{$host} - $count;
#	$msg = &create_move_msg($displacement);


	$sql = "SELECT UNIX_TIMESTAMP(MAX(SUBMITTED_DATE)) FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE (HOST_ID = \"$host\")";
	my ($LastSubmitted) = &DLookup($sql);
	$sql = "SELECT UNIX_TIMESTAMP(MIN(SUBMITTED_DATE)) FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE (HOST_ID = \"$host\")";
	my ($FirstSubmitted) =&DLookup($sql);
    
	$daysworking = ((($LastSubmitted - $FirstSubmitted) / (60*60*24)) + 1 );
	$aveblocks = int($blocks[$z] / $daysworking);
	$averate = &rate_day($blocks[$z] / $daysworking, $SystemPrefs{"contestfactor"}[$Contest], $Contest);
	$averatef = &ratef($averate);

#	$prev_daysworking = $hosts_firstday{$host} - (defined($hosts_prev_lastday{$host}) ? $hosts_prev_lastday{$host} : 0) + 1;
	($year, $mon, $mday, $hour, $min, $sec, $wday) = &nice_gmtime($FirstSubmitted);
	$firstblockdate = "$mday $Months[$mon-1] $year GMT";
	($year, $mon, $mday, $hour, $min, $sec, $wday) = &nice_gmtime($LastSubmitted);
	$lastblockdate = "$mday $Months[$mon-1] $year GMT";

	# Print out the Last Seen column.
        $sql = "SELECT MAX(SUBMITTED_DATE) FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE (HOST_ID = \"$host\")";
	my ($LastSubmittedDate) = &DLookup($sql);
        $sql = "SELECT MAX(SUBMITTED_TIME) FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE ((SUBMITTED_DATE = \"$LastSubmittedDate\") AND (HOST_ID = \"$host\"))";
	my ($LastSubmittedTime) = &DLookup($sql);
	$sql = "SELECT UNIX_TIMESTAMP(\"$LastSubmittedDate $LastSubmittedTime\")";
	my ($LastTime) = &DLookup($sql);

	$lastseentime = ($SystemPrefs{"timebegin"}-($LastTime));
	$lastseen = &ls($lastseentime);
	$lastseencolor = &ls_color($lastseentime);


        ($year, $mon, $mday, $hour, $min, $sec, $wday) = &nice_gmtime($SystemPrefs{"timebegin"} - (1 * (24*60*60)));
#SELECT SUM(WU_SIZE) FROM `TBL_WU_CSC` WHERE DATE_FORMAT(SUBMITTED,"%Y-%m-%d") = "1999-07-11"
	$sql = "SELECT IFNULL(SUM(WU_SIZE),0) FROM " . $SystemPrefs{"contestwutable"}[$Contest] . 
	   " WHERE ((SUBMITTED_DATE = \"$year-$mon-$mday\") AND (HOST_ID = \"$host\"))";
	($blocksyesterday) = &DLookup($sql);
	$rateyesterday = &ratef(&rate_day($blocksyesterday, $SystemPrefs{"contestfactor"}[$Contest], $Contest));


	$contestblocks = &get_contest_blocks($Contest);
	$keyschecked = &keysf($blocks[$z], $Contest);
	$percentcomplete = &percentf($blocks[$z], 6, $Contest);
	if ($blocksoverall > 0)
	{
	    $percentofteam = sprintf("%.2f", 100 * $blocks[$z] / $blocksoverall);
	}
	else
	{
	    $percentofteam = sprintf("%.2f", 0);
	}
	$label = undef;
	$icon = undef;
	$sql = "SELECT HOSTNAME FROM TBL_HOST WHERE (HOST_ID = \"$host\") LIMIT 1";
	my ($hostname) = &DLookup($sql);

	$sql = "SELECT COUNTRY_ID FROM TBL_HOST WHERE (HOST_ID = \"$host\") LIMIT 1";
	my ($domain) = &DLookup($sql);

	if (defined($domain)) 
	{
	    $sql = "SELECT COUNTRY FROM TBL_COUNTRY WHERE (COUNTRY_ID = \"$domain\") LIMIT 1";
	    ($label) = &DLookup($sql);
	    if (defined($label))
	    {
	        $sql = "SELECT COUNTRY_ICON FROM TBL_COUNTRY WHERE (COUNTRY_ID = \"$domain\") LIMIT 1";
		($icon) = &DLookup($sql);
		$icon = ($icon . "." . $UserPrefs{"imageext"});
	    }
	    else
	    {
		$label = "Unknown";
		$icon = ("unknown." . $UserPrefs{"imageext"});
	    }
	}

	$hostdir = "h" . $host;
	mkdir($UserPrefs{"htmldir"} . $SystemPrefs{"contestsuffix"}[$Contest] . "/" . "$hostdir", 0755);
	open(OUTFILE, ">" . $UserPrefs{"htmldir"} . $SystemPrefs{"contestsuffix"}[$Contest] . "/$hostdir/" . $UserPrefs{"htmlindvhost"}) or die "Personal Proxy Statistics : Cannot open " . $UserPrefs{"htmldir"} . $SystemPrefs{"contestsuffix"}[$Contest] . "/$hostdir/" . $UserPrefs{"htmlindvhost"} . " ($!)\n";
	&write_header($Contest);
	print OUTFILE "     <TABLE BORDER=0 CELLPADDING=0 CELLSPACING=0 WIDTH=\"100%\">\n";
	print OUTFILE "      <TR ALIGN=CENTER>\n",
	              "       <TD CLASS=title NOWRAP>\n",
                      "        <STRONG>Hostname $hostname</STRONG><BR>\n",
                      "        Executive Hostname Summary\n",
                      "       </TD>\n",
                      "      </TR>\n";

	print OUTFILE "      <TR>\n",
                      "       <TD>\n",
                      "        &nbsp;\n",
                      "       </TD>\n",
                      "      </TR>\n";

	print OUTFILE "      <TR ALIGN=CENTER>\n",
                      "       <TD VALIGN=TOP WIDTH=\"100%\">\n",
                      "        <TABLE BORDER=0 CELLPADDING=2 CELLSPACING=0>\n";

	print OUTFILE "         <TR>\n",
                      "          <TD CLASS=subtitle NOWRAP COLSPAN=2 ALIGN=CENTER>\n",
                      "           Hostname Summary for $hostname\n",
                      "          </TD>\n",
                      "         </TR>\n";

	print OUTFILE "         <TR>\n",
                      "          <TD NOWRAP>\n",
                      "           Current Team Rank\n",
                      "          </TD>\n",
                      "          <TD NOWRAP ALIGN=RIGHT>\n",
#                      "           $count$msg\n",
                      "           $count\n",
                      "          </TD>\n",
                      "         </TR>\n";

	print OUTFILE "         <TR>\n",
                      "          <TD NOWRAP>\n",
                      "           Percent of Team\n",
                      "          </TD>\n",
                      "          <TD NOWRAP ALIGN=RIGHT>\n",
                      "           $percentofteam%\n",
                      "          </TD>\n",
                      "         </TR>\n";

	if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
	{
	    print OUTFILE "         <TR>\n",
                      "          <TD NOWRAP>\n",
                      "           Total " . $SystemPrefs{"contestheaderblocktype"}[$Contest] . "s to Search\n",
                      "          </TD>\n",
                      "          <TD NOWRAP ALIGN=RIGHT>\n",
                      "           $contestblocks " . $SystemPrefs{"contestblocktype"}[$Contest] . "s\n",
                      "          </TD>\n",
                      "         </TR>\n";
	}

	print OUTFILE "         <TR>\n",
                      "          <TD NOWRAP>\n",
                      "           Total " . $SystemPrefs{"contestheaderblocktype"}[$Contest] . "s Checked\n",
                      "          </TD>\n",
                      "          <TD NOWRAP ALIGN=RIGHT>\n";
	if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
	{
    	    print OUTFILE "           $blocks[$z] " . $SystemPrefs{"contestblocktype"}[$Contest] . "s\n",
	}
	else
	{
    	    print OUTFILE "           " . &ratef($blocks[$z] * $SystemPrefs{"contestfactor"}[$Contest]) . $SystemPrefs{"contestblocktype"}[$Contest] . "s\n",
	}
	print OUTFILE "          </TD>\n",
                      "         </TR>\n";

	if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
	{
	    print OUTFILE "         <TR>\n",
                      "          <TD NOWRAP>\n",
                      "           Keyspace Exhausted\n",
                      "          </TD>\n",
                      "          <TD NOWRAP ALIGN=RIGHT>\n",
                      "           $percentcomplete%\n",
                      "          </TD>\n",
                      "         </TR>\n";

	print OUTFILE "         <TR>\n",
                      "          <TD NOWRAP>\n",
                      "           Total " . $SystemPrefs{"contestheaderblockmeasure"}[$Contest] . "s Checked\n",
                      "          </TD>\n",
                      "          <TD NOWRAP ALIGN=RIGHT>\n",
                      "           $keyschecked\n",
                      "          </TD>\n",
                      "         </TR>\n";
	}

#	print OUTFILE "         <TR>\n",
#                      "          <TD NOWRAP>\n",
#                      "           Total Unique IPs\n",
#                      "          </TD>\n",
#                      "          <TD NOWRAP ALIGN=RIGHT>\n",
#                      "           $hosts_ips{$host}\n",
#                      "          </TD>\n",
#                      "         </TR>\n";

	print OUTFILE "         <TR>\n",
                      "          <TD NOWRAP>\n",
                      "           Time Working\n",
                      "          </TD>\n",
                      "          <TD NOWRAP ALIGN=RIGHT>\n",
                      "           $daysworking days\n",
                      "          </TD>\n",
                      "         </TR>\n";

	print OUTFILE "         <TR>\n",
                      "          <TD NOWRAP>\n",
                      "           Last Seen\n",
                      "          </TD>\n",
                      "          <TD CLASS=$lastseencolor NOWRAP ALIGN=RIGHT>\n",
                      "           $lastseen\n",
                      "          </TD>\n",
                      "         </TR>\n";

	print OUTFILE "         <TR>\n",
                      "          <TD NOWRAP>\n",
                      "           First " . $SystemPrefs{"contestheaderblocktype"}[$Contest] . " Submitted\n",
                      "          </TD>\n",
                      "          <TD NOWRAP ALIGN=RIGHT>\n",
                      "           $firstblockdate\n",
                      "          </TD>\n",
                      "         </TR>\n";

	print OUTFILE "         <TR>\n",
                      "          <TD NOWRAP>\n",
                      "           Last " . $SystemPrefs{"contestheaderblocktype"}[$Contest] . " Submitted\n",
                      "          </TD>\n",
                      "          <TD NOWRAP ALIGN=RIGHT>\n",
                      "           $lastblockdate\n",
                      "          </TD>\n",
                      "         </TR>\n";

	print OUTFILE "         <TR>\n",
                      "          <TD NOWRAP>\n",
                      "           Overall Rate\n",
                      "          </TD>\n",
                      "          <TD NOWRAP ALIGN=RIGHT>\n",
                      "           $averatef" . $SystemPrefs{"contestblockmeasure"}[$Contest] . "/sec\n",
                      "          </TD>\n",
                      "         </TR>\n";

	print OUTFILE "         <TR>\n",
                      "          <TD NOWRAP>\n",
                      "           Average " . $SystemPrefs{"contestheaderblocktype"}[$Contest] . "s\n",
                      "          </TD>\n",
                      "          <TD NOWRAP ALIGN=RIGHT>\n";

	if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
	{
    	    print OUTFILE "           $aveblocks per day\n";
	}
	else
	{
    	    print OUTFILE "           " . &ratef($aveblocks * $SystemPrefs{"contestfactor"}[$Contest]) . $SystemPrefs{"contestblocktype"}[$Contest] . "s per day\n";
	}
        print OUTFILE "          </TD>\n",
                      "         </TR>\n";


        $sql = "SELECT VERSION FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE ((SUBMITTED_DATE = \"$LastSubmittedDate\") AND (SUBMITTED_TIME = \"$LastSubmittedTime\")AND (HOST_ID = \"$host\") ) LIMIT 1";
	my ($lastversion) = &DLookup($sql);
	print OUTFILE "         <TR>\n",
                      "          <TD NOWRAP>\n",
                      "           Last Version Reported\n",
                      "          </TD>\n",
                      "          <TD NOWRAP ALIGN=RIGHT>\n",
                      "           $lastversion\n",
                      "          </TD>\n",
                      "         </TR>\n";

	print OUTFILE "         <TR>\n",
                      "          <TD NOWRAP>\n",
                      "           Hostname Domain\n",
                      "          </TD>\n",
                      "          <TD NOWRAP ALIGN=RIGHT>\n",
                      "           $label&nbsp;<IMG SRC=\"" . $UserPrefs{"htmlroot"} . $UserPrefs{"icondir"} . "flags/$icon\" ALIGN=BOTTOM BORDER=0 WIDTH=14 HEIGHT=14 ALT=\"$label\">\n",
                      "          </TD>\n",
                      "         </TR>\n";

	print OUTFILE "        </TABLE>\n",
                      "       </TD>\n",
                      "      </TR>\n";

	print OUTFILE "      <TR>\n",
                      "       <TD>\n",
                      "        &nbsp;\n",
                      "       </TD>\n",
                      "      </TR>\n";

	if ($blocksyesterday > 0) 
	{
	    $percentcomplete = &percentf($blocksyesterday, 8, $Contest);
	    print OUTFILE "      <TR>\n",
	                  "       <TD NOWRAP ALIGN=CENTER>\n";

	    if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
	    {
        	print OUTFILE "        $blocksyesterday " . $SystemPrefs{"contestblocktype"}[$Contest] . "s were completed yesterday ($percentcomplete% of the keyspace)<BR>at a sustained rate of $rateyesterday" . $SystemPrefs{"contestblockmeasure"}[$Contest] . "/sec!\n",
	    }
	    else
	    {
    		print OUTFILE "           " . &ratef($blocksyesterday * $SystemPrefs{"contestfactor"}[$Contest]) . $SystemPrefs{"contestblocktype"}[$Contest] . "s were completed yesterday <BR>at a sustained rate of $rateyesterday" . $SystemPrefs{"contestblockmeasure"}[$Contest] . "/sec!\n";
	    }
            print OUTFILE "       </TD>\n",
                          "      </TR>\n";
	} 
	else 
	{
	    print OUTFILE "      <TR>\n",
                          "       <TD NOWRAP ALIGN=CENTER>\n",
                          "        This host didn't submit any " . $SystemPrefs{"contestblocktype"}[$Contest] . "s yesterday, someone get that cpu cracking!\n",
                          "       </TD>\n",
                          "      </TR>\n";
	}

	print OUTFILE "      <TR>\n",
                      "       <TD>\n",
                      "        &nbsp;\n",
                      "       </TD>\n",
                      "      </TR>\n";

	print OUTFILE "      <TR>\n",
                      "       <TD NOWRAP ALIGN=CENTER>\n",
                      "        Click here to goto the <A HREF=\"" . $UserPrefs{"htmlroot"} . $SystemPrefs{"contestsuffix"}[$Contest]. "/" . "$hostdir/" . $UserPrefs{"htmlhosthist"} . "\">Graphical Hostname History</A> page.\n",
                      "       </TD>\n",
                      "      </TR>\n";

	print OUTFILE "      <TR>\n",
                      "       <TD>\n",
                      "        &nbsp;\n",
                      "       </TD>\n",
                      "      </TR>\n";

	print OUTFILE "      <TR>\n",
                      "       <TD NOWRAP ALIGN=CENTER>\n",
                      "        Click here to goto the <A HREF=\"" . $UserPrefs{"htmlroot"} . $SystemPrefs{"contestsuffix"}[$Contest]. "/" . "$hostdir/" . $UserPrefs{"htmlemailsbyhost"} . "\">Participants working for this Hostname</A> page.\n",
                      "       </TD>\n",
                      "      </TR>\n";

	print OUTFILE "      <TR>\n",
                      "       <TD>\n",
                      "        &nbsp;\n",
                      "       </TD>\n",
                      "      </TR>\n";

	print OUTFILE "      <TR>\n",
                      "       <TD NOWRAP ALIGN=CENTER>\n",
                      "        Hostname History for the Last " . $UserPrefs{"blockhistory"} . " Days\n",
                      "       </TD>\n",
                      "      </TR>\n";

	print OUTFILE "      <TR>\n",
                      "       <TD>\n",
                      "        &nbsp;\n",
                      "       </TD>\n",
                      "      </TR>\n";

	print OUTFILE "      <TR ALIGN=CENTER>\n",
                      "       <TD VALIGN=TOP WIDTH=\"100%\">\n",
                      "        <TABLE BORDER=0 CELLPADDING=2 CELLSPACING=0>\n";

	print OUTFILE "         <TR ALIGN=CENTER>\n",
                      "          <TD CLASS=section NOWRAP>\n",
                      "           Day History\n",
                      "          </TD>\n",
                      "          <TD CLASS=section NOWRAP>\n",
                      "           " . $SystemPrefs{"contestheaderblocktype"}[$Contest] . "s Submited\n",
                      "          </TD>\n",
                      "          <TD CLASS=section NOWRAP>\n",
                      "           Percent\n",
                      "          </TD>\n";
	if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
	{
    	    print OUTFILE "          <TD CLASS=section NOWRAP>\n",
                      "           " . $SystemPrefs{"contestheaderblockmeasure"}[$Contest] . "s Checked\n",
                      "          </TD>\n";
	}
        print OUTFILE "          <TD CLASS=section NOWRAP>\n",
                      "           " . $SystemPrefs{"contestheaderblockmeasure"}[$Contest] . " Rate\n",
                      "          </TD>\n";
	if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
	{
    	    print OUTFILE "          <TD CLASS=section NOWRAP>\n",
                      "           Keyspace\n",
                      "          </TD>\n";
	}
        print OUTFILE "         </TR>\n";

	for ($y = 0; $y <= $UserPrefs{"blockhistory"}; $y++) 
	{
	    ($year, $mon, $mday, $hour, $min, $sec, $wday) = &nice_gmtime($SystemPrefs{"timebegin"} - ($y * (24*60*60)));
	    my $blockdate = "$mday $Months[$mon-1] $year GMT";

    	    $sql = "SELECT IFNULL(SUM(WU_SIZE),0) FROM " . $SystemPrefs{"contestwutable"}[$Contest] . 
		" WHERE ((SUBMITTED_DATE = \"$year-$mon-$mday\") AND (HOST_ID = \"$host\"))";
    	    my ($blockshost) = &DLookup($sql);

	    if ($y > 0)
	    { 
		$rate = &ratef(&rate_day($blockshost, $SystemPrefs{"contestfactor"}[$Contest], $Contest));
	    }
	    else
	    {
		$rate = &ratef(&rate_day($blockshost / ($SystemPrefs{"secstoday"} / (60*60*24)), $SystemPrefs{"contestfactor"}[$Contest], $Contest));
	    }
	    $keyschecked = &keysf($blockshost, $Contest);
	    $percentcomplete = &percentf($blockshost, 8, $Contest);
	    $sql = "SELECT IFNULL(SUM(WU_SIZE),0) FROM " . $SystemPrefs{"contestwutable"}[$Contest] .
	                " WHERE (SUBMITTED_DATE = \"$year-$mon-$mday\")";
	    my ($blocksthatday) =&DLookup($sql);
	    if ($blocksthatday > 0)
	    {
		$percentofteam = sprintf("%.2f", 100 * $blockshost / $blocksthatday);
	    }
	    else
	    {
		$percentofteam = sprintf("%.2f", 0);
	    }
	    
	    if ($y % 2 == 0) 
	    {
		print OUTFILE "         <TR CLASS=even>\n";
	    } 
	    else 
	    {
		print OUTFILE "         <TR CLASS=odd>\n";
	    }

	    print OUTFILE "          <TD NOWRAP ALIGN=LEFT>\n",
	              "           $blockdate\n",
                      "          </TD>\n",
                      "          <TD NOWRAP ALIGN=RIGHT>\n";
	    if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
            {
        	print OUTFILE "           $blockshost " . $SystemPrefs{"contestblocktype"}[$Contest] . "s\n";
            }
	    else
	    {
		print OUTFILE "           ". &ratef($blockshost * $SystemPrefs{"contestfactor"}[$Contest]) . $SystemPrefs{"contestblockmeasure"}[$Contest] . "s/sec\n";
	    }
	    print OUTFILE "          </TD>\n",
                      "          <TD NOWRAP ALIGN=RIGHT>\n",
                      "           $percentofteam%\n",
		      "          </TD>\n";
	    if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
	    {
        	print OUTFILE "          <TD NOWRAP ALIGN=RIGHT>\n",
                      "           $keyschecked\n",
                      "          </TD>\n";
	    }
            print OUTFILE "          <TD NOWRAP ALIGN=RIGHT>\n",
	              "           $rate" . $SystemPrefs{"contestblockmeasure"}[$Contest] . "s/sec\n",
                      "          </TD>\n";
	    if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
	    {
        	print OUTFILE "          <TD NOWRAP ALIGN=RIGHT>\n",
                      "           $percentcomplete%\n",
                      "          </TD>\n";
	    }
            print OUTFILE "         </TR>\n";
	}

	print OUTFILE "        </TABLE>\n",
                      "       </TD>\n",
                      "      </TR>\n";

	print OUTFILE "     </TABLE>\n";
	&write_footer();
	close OUTFILE;

	&write_hostchart($host, $hostname, $hostdir, $Contest);
	&write_emailsbyhost($host,$hostname , $hostdir, $Contest);

	&Debug(".");
    }

    &Debugln("") if ($count > 0);
}  # write_indvhost


#--- Method Prolog -----------------------------------------------------------
#  Name: write_hostchart
#
#  Parameters: 
#      Contest - The contest to collect data for. 1=RC5, 2=DES, etc.
#
#  Return Value: None.
#
#  Description:
#    Routine to create the Graphical Hostname History plot, write the GNUplot 
#    config file and plot a chart for this Hostname.
#
#--- End Method Prolog -------------------------------------------------------
sub write_hostchart 
{
    my $host = shift @_;
    my $hostname = shift @_;
    my $hostdir = shift @_;
    my $Contest = shift @_;
    my ($year, $month, $mday, $hour, $min, $sec, $wday);
    my $numhosts;
    my ($sql, $y, $z);
    my @rate;
    my $colsp;
    my $daysworking;

    open(OUTFILE, ">" . $UserPrefs{"htmldir"} . $SystemPrefs{"contestsuffix"}[$Contest] . "/$hostdir/" . $UserPrefs{"htmlhosthist"}) or die "Personal Proxy Statistics : Cannot open " . $UserPrefs{"htmldir"} . $SystemPrefs{"contestsuffix"}[$Contest] . "/$hostdir/" . $UserPrefs{"htmlhosthist"} . " ($!)\n";
    &write_header($Contest);
    print OUTFILE "     <TABLE BORDER=0 CELLPADDING=0 CELLSPACING=0 WIDTH=\"100%\">\n";
    print OUTFILE "      <TR ALIGN=CENTER>\n";
    for ($y = $UserPrefs{"dayshistory"}; $y > 1; $y--) 
    {
	($year, $month, $mday, $hour, $min, $sec, $wday) = &nice_gmtime($SystemPrefs{"timebegin"} - $y * (24*60*60));
	print OUTFILE "       <TD CLASS=section>" . $SystemPrefs{"contestheaderblocktype"}[$Contest] . "s<BR>$Days[$wday]</TD>\n";
    }
    if ($UserPrefs{"dayshistory"} > 0) 
    {
	print OUTFILE "       <TD CLASS=section>" . $SystemPrefs{"contestheaderblocktype"}[$Contest] . "s<BR>Yesterday</TD>\n",
    }
    if ($UserPrefs{"dayshistory"} >= 0) 
    {
	print OUTFILE "       <TD CLASS=section>" . $SystemPrefs{"contestheaderblocktype"}[$Contest] . "s<BR>Today</TD>\n";
    }
    print OUTFILE "       <TD CLASS=section>" . $SystemPrefs{"contestheaderblocktype"}[$Contest] . "s<BR>Total</TD>\n",
                  "      </TR>\n";
    
    print OUTFILE "      <TR ALIGN=CENTER>\n";



    for ($y = $UserPrefs{"dayshistory"}; $y >= 0; $y--) 
    {
	($year, $month, $mday, $hour, $min, $sec, $wday) = &nice_gmtime($SystemPrefs{"timebegin"} - ($y * (24*60*60)));
#SELECT SUM(WU_SIZE) FROM `TBL_WU_CSC` WHERE DATE_FORMAT(SUBMITTED,"%Y-%m-%d") = "1999-07-11"
        $sql = "SELECT IFNULL(SUM(WU_SIZE),\"-\") FROM " . $SystemPrefs{"contestwutable"}[$Contest] . 
		" WHERE ((SUBMITTED_DATE = \"$year-$month-$mday\") AND (HOST_ID = \"$host\"))";
        my ($blocksthatday) = &DLookup($sql);
	if ($y > 0)
	{ 
	 $rate[$y] = &ratef(&rate_day($blocksthatday, $SystemPrefs{"contestfactor"}[$Contest], $Contest));
	}
	else
	{
	 $rate[$y] = &ratef(&rate_day($blocksthatday / ($SystemPrefs{"secstoday"} / (60*60*24)), $SystemPrefs{"contestfactor"}[$Contest], $Contest));
	}
	if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
	{
	    print OUTFILE "       <TD>$blocksthatday</TD>\n";
	}
	else
	{
	    if ($blocksthatday ne "-")
	    {
		print OUTFILE "       <TD>" . &ratef($blocksthatday * $SystemPrefs{"contestfactor"}[$Contest]) . "</TD>\n";
	    }
	    else
	    {
		print OUTFILE "       <TD>$blocksthatday</TD>\n";
	    }
	}
    }

    ($year, $month, $mday, $hour, $min, $sec, $wday) = &nice_gmtime($SystemPrefs{"timebegin"} - $UserPrefs{"dayshistory"} * (24*60*60));
#SELECT SUM(WU_SIZE) FROM TBL_WU_RC564 
    $sql = "SELECT IFNULL(SUM(WU_SIZE),\"-\") FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE (HOST_ID = \"$host\")";
    my ($blocksoverall) = &DLookup($sql);
    
    if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
    {
	print OUTFILE "       <TD>$blocksoverall</TD>\n";
    }
    else
    {
	if ($blocksoverall > "0")
	{
	    print OUTFILE "       <TD>" . &ratef($blocksoverall * $SystemPrefs{"contestfactor"}[$Contest]) . "</TD>\n";
	}
	else
	{
	    print OUTFILE "       <TD>$blocksoverall</TD>\n";
	}
    }
    print OUTFILE "      </TR>\n";
    print OUTFILE "      <TR ALIGN=CENTER>\n";

    # Print out day history list for all days before today
    for ($y = $UserPrefs{"dayshistory"}; $y >= 0; $y--)
    {
	print OUTFILE "       <TD NOWRAP>$rate[$y]" . $SystemPrefs{"contestblockmeasure"}[$Contest] . "/sec</TD>\n";
    }


    $sql = "SELECT UNIX_TIMESTAMP(MAX(SUBMITTED_DATE)) FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE (HOST_ID = \"$host\")";
    my ($LastSubmitted) = &DLookup($sql);
    $sql = "SELECT UNIX_TIMESTAMP(MIN(SUBMITTED_DATE)) FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE (HOST_ID = \"$host\")";
    my ($FirstSubmitted) =&DLookup($sql);
    
    my $totaldays = int((($LastSubmitted - $FirstSubmitted) / (60*60*24)) +1);
    my $aveblocks = int($blocksoverall / $totaldays);
    my $averate = &rate_day($blocksoverall / $totaldays, $SystemPrefs{"contestfactor"}[$Contest], $Contest);
    my $averatef = &ratef($averate);

    print OUTFILE "       <TD NOWRAP>$averatef" . $SystemPrefs{"contestblockmeasure"}[$Contest] . "/sec</TD>\n";
    print OUTFILE "      </TR>\n";


    $colsp = 2 + $UserPrefs{"dayshistory"};
    print OUTFILE "      <TR>\n",
                  "       <TD COLSPAN=$colsp>\n",
                  "        &nbsp;\n",
                  "       </TD>\n",
                  "      </TR>\n";

    print OUTFILE "      <TR ALIGN=CENTER>\n",
	          "       <TD CLASS=title NOWRAP COLSPAN=$colsp>\n",
                  "        <STRONG>Hostname $hostname</STRONG><BR>\n",
                  "        Graphical Hostname History\n",
                  "       </TD>\n",
                  "      </TR>\n";

    print OUTFILE "      <TR><TD NOWRAP COLSPAN=$colsp ALIGN=CENTER>\n",
                  "       <BR>Total Days: $totaldays days - ";
    if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
    {
	print OUTFILE "Average " . $SystemPrefs{"contestheaderblocktype"}[$Contest] . "s: $aveblocks " .
	          $SystemPrefs{"contestblocktype"}[$Contest] . "s - ";
    }
    else
    {
	print OUTFILE "Average " . $SystemPrefs{"contestheaderblocktype"}[$Contest] . "s: " .
	          &ratef($aveblocks * $SystemPrefs{"contestfactor"}[$Contest]) .
	          $SystemPrefs{"contestblocktype"}[$Contest] . "s - ";
    }
    print OUTFILE "Average Rate: $averatef" . $SystemPrefs{"contestblockmeasure"}[$Contest] . "s/sec\n",
                  "      </TD></TR>\n";

    print OUTFILE "      <TR>\n",
                  "       <TD COLSPAN=$colsp>\n",
                  "        &nbsp;\n",
                  "       </TD>\n",
                  "      </TR>\n";

    print OUTFILE "      <TR>\n",
	          "       <TD COLSPAN=$colsp ALIGN=CENTER>\n",
                  "        <IMG SRC=\"" . $UserPrefs{"htmlroot"} . $SystemPrefs{"contestsuffix"}[$Contest]. "/" . "$hostdir/" . $UserPrefs{"gnuplotoutput"} . "\" WIDTH=" . $UserPrefs{"chartwidth"} . " HEIGHT=" . $UserPrefs{"chartheight"} . " ALT=\"$host\">\n",
                  "       </TD>\n",
                  "      </TR>\n";

    print OUTFILE "      <TR>\n",
                  "       <TD COLSPAN=$colsp>\n",
                  "        &nbsp;\n",
                  "       </TD>\n",
                  "      </TR>\n";

    print OUTFILE "      <TR><TD CLASS=note NOWRAP COLSPAN=$colsp ALIGN=CENTER>\n",
                  "       For past days: " . $SystemPrefs{"contestblockmeasure"}[$Contest] . "s/sec = " . 
		  $SystemPrefs{"blockweight"}[$Contest] . " * " . $SystemPrefs{"contestblocktype"}[$Contest] . "s / " . $SystemPrefs{"contestdivide"}[$Contest] . "<BR>\n",
                  "       For today: " . $SystemPrefs{"contestblockmeasure"}[$Contest] . "s/sec = (" . 
		  $SystemPrefs{"blockweight"}[$Contest] . " * " . $SystemPrefs{"contestblocktype"}[$Contest] . "s / " . $SystemPrefs{"contestdivide"}[$Contest] . ") / (seconds today / (60*60*24))\n",
                  "      </TD></TR>\n";

    print OUTFILE "      <TR>\n",
                  "       <TD COLSPAN=$colsp>\n",
                  "        &nbsp;\n",
                  "       </TD>\n",
                  "      </TR>\n";

    print OUTFILE "      <TR>\n",
                  "       <TD NOWRAP COLSPAN=$colsp ALIGN=CENTER>\n",
                  "        Click here to goto the <A HREF=\"" . $UserPrefs{"htmlroot"} . $SystemPrefs{"contestsuffix"}[$Contest]. "/" . "$hostdir/" . $UserPrefs{"htmlindvhost"} . "\">Executive Hostname Summary</A> page.\n",
                  "       </TD>\n",
                  "      </TR>\n";

    print OUTFILE "      <TR>\n",
                  "       <TD COLSPAN=$colsp>\n",
                  "        &nbsp;\n",
                  "       </TD>\n",
                  "      </TR>\n";

    print OUTFILE "      <TR>\n",
                  "       <TD NOWRAP COLSPAN=$colsp ALIGN=CENTER>\n",
                  "        Click here to goto the <A HREF=\"" . $UserPrefs{"htmlroot"} . $SystemPrefs{"contestsuffix"}[$Contest]. "/" . "$hostdir/" . $UserPrefs{"htmlemailsbyhost"} . "\">Participants working for this Hostname</A> page.\n",
                  "       </TD>\n",
                  "      </TR>\n";

    print OUTFILE "     </TABLE>\n";
    &write_footer();
    close OUTFILE;

    open(GNUPLOTDATA, ">" . $UserPrefs{"htmldir"} . $SystemPrefs{"contestsuffix"}[$Contest] . "/$hostdir/" . $SystemPrefs{"gnuplotdata"}) or die "Personal Proxy Statistics : Cannot open " . $UserPrefs{"htmldir"} . "$hostdir/" . $SystemPrefs{"gnuplotdata"} . " ($!)\n";

    # Write out data file in format "Date, Rate".
##    &clear_nday_ave();

    my @row;
    $sql = "SELECT DATE_FORMAT(DATE_SUB(MIN(SUBMITTED_DATE), INTERVAL 1 DAY), \"%Y-%m-%d\") FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE (HOST_ID = \"$host\")";
    my $sth = &SelectSQL($sql);
    @row = $sth->fetchrow_array();
    my $Submitted = $row[0];
    my $WorkUnits = undef;
#SELECT SUBMITTED_DATE,SUM(WU_SIZE) FROM `TBL_WU_RC564` WHERE HOST_ID = "1" GROUP BY 1 ORDER BY 1 ASC 
    $sql = "SELECT SUBMITTED_DATE, SUM(WU_SIZE) FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE (HOST_ID = \"$host\") GROUP BY 1 ORDER BY 1 ASC";
    $sth = &SelectSQL($sql);
    my $line;
    my ($rowyear, $rowmonth, $rowday);
    my ($yeartemp, $monthtemp, $daytemp);
    my ($datetemp, $timetemp);
    my ($subyear, $submonth, $subday);
    my $averagerate;
    
    &clear_average;

    while (@row = $sth->fetchrow_array()) 
    {
	($subyear, $submonth, $subday) = split(/-/, $Submitted); 
	($rowyear, $rowmonth, $rowday) = split(/-/, $row[0]);
	
	if (defined($WorkUnits))
	{
	    $averagerate = &calculate_average($WorkUnits, 2, $Contest);
	    $line = "$subyear/$submonth/$subday " . sprintf("%.2f", &rate_day($WorkUnits, $SystemPrefs{"contestfactor"}[$Contest], $Contest)) . " $averagerate";
	    print GNUPLOTDATA "$line\n";
	}

    	$Submitted = $row[0];
	$WorkUnits = $row[1];
	# Check if days are missing between writen line and new started day
    	$sql = "SELECT DATE_FORMAT(DATE_ADD(\"$subyear-$submonth-$subday 00:00:00\", INTERVAL 1 DAY), \"%Y-%m-%d\")";
    	($datetemp) = &DLookup($sql);
	($yeartemp, $monthtemp, $daytemp) = split(/-/, $datetemp);
	while (($yeartemp != $rowyear) || ($monthtemp != $rowmonth) || ($daytemp != $rowday))
	{
	    $averagerate = &calculate_average(0, 2, $Contest);
	    $line = "$yeartemp/$monthtemp/$daytemp 0.00 $averagerate";
	    print GNUPLOTDATA "$line\n";
    	    $sql = "SELECT DATE_FORMAT(DATE_ADD(\"$yeartemp-$monthtemp-$daytemp 00:00:00\", INTERVAL 1 DAY), \"%Y-%m-%d\")";

    	    ($datetemp) = &DLookup($sql);
	    ($yeartemp, $monthtemp, $daytemp) = split(/-/, $datetemp);	     
	}

    }

    
    $averagerate = &calculate_average($WorkUnits, 2, $Contest);
    if ($Submitted != 0)
    {
     if ($Submitted eq "$GMTTS[0]-$GMTTS[1]-$GMTTS[2]")
     {
      $line = "$rowyear/$rowmonth/$rowday " . sprintf("%.2f", &rate_day($WorkUnits / ($SystemPrefs{"secstoday"} / (60*60*24)), $SystemPrefs{"contestfactor"}[$Contest], $Contest)) . " $averagerate";
     }
     else
     {
      $line = "$rowyear/$rowmonth/$rowday " . sprintf("%.2f", &rate_day($WorkUnits, $SystemPrefs{"contestfactor"}[$Contest], $Contest)) . " $averagerate";
     }
     print GNUPLOTDATA "$line\n";
    }



#    for ($x = $hosts_firstday{$host}; $x >= $hosts_lastday{$host}; $x--) {
#	($year, $mon, $mday, $hour, $min, $sec, $wday) = &nice_gmtime($time_begin - ($x - 1) * (24*60*60));
#	$hosts{$host}[$x] = defined($hosts{$host}[$x]) ? $hosts{$host}[$x] : 0;
#	if ($x == 1) {
#	    $rate = sprintf("%.2f", &rate_day($hosts{$host}[$x], $SystemPrefs{"contestfactor"}[$Contest], $Contest) / ($secs_today / (60*60*24)));
#	} else {
#	    $rate = sprintf("%.2f", &rate_day($hosts{$host}[$x], $SystemPrefs{"contestfactor"}[$Contest], $Contest));
#	}
#	$line = "$year/$mon/$mday $rate";
#	&push_nday_ave($rate);
#	$rate = &calc_nday_ave();
#	$line .= " $rate";
#	print GNUPLOTDATA "$line\n";
#    }

    close GNUPLOTDATA;
    
    my $range = "*";
    if ($blocksoverall == 0) { $range = 1; };
    $sql = "SELECT DATE_FORMAT(MIN(SUBMITTED_DATE), \"%Y/%m/%d\") FROM " . $SystemPrefs{"contestwutable"}[$Contest];
    ($FirstSubmitted) =&DLookup($sql);
    
    &plot_gui_graph($UserPrefs{"htmldir"} . $SystemPrefs{"contestsuffix"}[$Contest] . "/$hostdir/", "History Plot for " . $SystemPrefs{"contestname"}[$Contest] . "\\nHostname $hostname", $FirstSubmitted, "$GMTTS[0]/$GMTTS[1]/$GMTTS[2]", $range, $averate, $Contest);
}  # write_hostchart


#--- Method Prolog -----------------------------------------------------------
#  Name: write_emailsbyhost
#
#  Parameters: 
#      host - hostid in database
#      hostname - hostname (also from database)
#      hostdir - ("h".$host)
#      Contest - The contest to collect data for. 1=RC5, 2=DES, etc.
#
#  Return Value: None.
#
#  Description:
#    Routine to write all email adresses being used by a host.
#
#--- End Method Prolog -------------------------------------------------------
sub write_emailsbyhost
{
    my $host = shift @_;
    my $hostname = shift @_;
    my $hostdir = shift @_;
    my $Contest = shift @_;
    my $count = 0;
    my (@emails, @blocks, @takerow);
    my ($sql, $st);
    my $colsp;
    my $num;
    my $numhosts;
    my ($y, $z);
    my ($domain, $email, $nickname, $label, $icon);
    my $percentofhost;
    my $keyschecked;
    my $percentcomplete;
    my $emaildir;
    my ($head, $tail);
    

    open(OUTFILE, ">" . $UserPrefs{"htmldir"} . $SystemPrefs{"contestsuffix"}[$Contest] . "/$hostdir/" . $UserPrefs{"htmlemailsbyhost"}) or die "Personal Proxy Statistics : Cannot open " . $UserPrefs{"htmldir"} . "$hostdir/" . $UserPrefs{"htmlemailsbyhost"} . " ($!)\n";
    &write_header($Contest);
    print OUTFILE "     <TABLE BORDER=0 CELLPADDING=0 CELLSPACING=0 WIDTH=\"100%\">\n";
    print OUTFILE "      <TR ALIGN=CENTER>\n",
	          "       <TD CLASS=title NOWRAP>\n",
                  "        <STRONG>Hostname $hostname</STRONG><BR>\n",
                  "        Participants working for this Hostname\n",
                  "       </TD>\n",
                  "      </TR>\n";

    $sql = "SELECT UNIX_TIMESTAMP(MAX(SUBMITTED_DATE)) FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE (HOST_ID = \"$host\")";
    my ($LastSubmitted) = &DLookup($sql);
    $sql = "SELECT UNIX_TIMESTAMP(MIN(SUBMITTED_DATE)) FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE (HOST_ID = \"$host\")";
    my ($FirstSubmitted) =&DLookup($sql);
    $sql = "SELECT IFNULL(SUM(WU_SIZE),0) FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE (HOST_ID = \"$host\")";
    my ($blocksoverall) = &DLookup($sql);
    
    my $totaldays = int((($LastSubmitted - $FirstSubmitted) / (60*60*24)) + 1);
    my $aveblocks = int($blocksoverall / $totaldays);
    my $averate = &rate_day($blocksoverall / $totaldays, $SystemPrefs{"contestfactor"}[$Contest], $Contest);
    my $averatef = &ratef($averate);

    $colsp = 1 + $UserPrefs{"dayshistory"};

    print OUTFILE "      <TR><TD NOWRAP COLSPAN=$colsp ALIGN=CENTER>\n",
                  "       <BR>Total Days: $totaldays days - Average " . $SystemPrefs{"contestheaderblocktype"}[$Contest];
    if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
    {
	print OUTFILE ": $aveblocks " . $SystemPrefs{"contestblocktype"}[$Contest] . "s - ";
    }
    else
    {
	print OUTFILE ": " . &ratef($aveblocks * $SystemPrefs{"contestfactor"}[$Contest]) .$SystemPrefs{"contestblocktype"}[$Contest] . "s - ";
    }
    print OUTFILE "Average Rate: $averatef" . $SystemPrefs{"contestblockmeasure"}[$Contest] . "s/sec\n",
                  "      </TD></TR>\n";

    print OUTFILE "      <TR>\n",
                  "       <TD>\n",
                  "        &nbsp;\n",
                  "       </TD>\n",
                  "      </TR>\n";

    print OUTFILE "      <TR ALIGN=CENTER>\n",
                  "       <TD VALIGN=TOP WIDTH=\"100%\">\n",
                  "        <TABLE BORDER=0 CELLPADDING=2 CELLSPACING=0>\n";

    print OUTFILE "         <TR ALIGN=CENTER>\n",
                  "          <TD CLASS=section NOWRAP>\n",
                  "           Participant\n",
                  "          </TD>\n",
                  "          <TD CLASS=section NOWRAP>\n",
                  "           " . $SystemPrefs{"contestheaderblocktype"}[$Contest] . "s Submitted\n",
                  "          </TD>\n",
                  "          <TD CLASS=section NOWRAP>\n",
                  "           Percent\n",
                  "          </TD>\n";
    if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
    {
        print OUTFILE "          <TD CLASS=section NOWRAP>\n",
                  "           " . $SystemPrefs{"contestheaderblockmeasure"}[$Contest] . "s Checked\n",
                  "          </TD>\n",
	          "          <TD CLASS=section NOWRAP>\n",
                  "           Keyspace\n",
                  "          </TD>\n";
    }
    print OUTFILE "         </TR>\n";

    $sql = "SELECT PARTICIPANT_ID, IFNULL(SUM(WU_SIZE),0) FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE (HOST_ID = \"$host\") GROUP BY 1 ORDER BY 2 DESC";
    $st = &SelectSQL($sql);

    $numhosts = 0;
    $sql = "SELECT MAX(PARTICIPANT_ID) FROM TBL_PARTICIPANT";
    my ($maxemail) = &DLookup($sql);
    while (@takerow = $st->fetchrow_array()) 
    {
	$sql = "SELECT RETIRE_TO_ID FROM TBL_PARTICIPANT WHERE (PARTICIPANT_ID = \"" . $takerow[0] . "\") LIMIT 1";
	my ($retireto) = &DLookup($sql);
	if ($retireto > 0)
	{
	    $blocks[$retireto] += $takerow[1];
	}
	else
	{
	    $numhosts++;
	    $emails[$takerow[0]] = $numhosts;
	    $blocks[$takerow[0]] += $takerow[1];
	}
    }

    $sql = "SELECT SUM(WU_SIZE) FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE (HOST_ID = \"$host\")";
    my ($blocksthathost) = &DLookup($sql);

    $count = 0;

    for ($z = 1; $z <= $numhosts; $z++)
    {
	for ($y = 1; $y <=$maxemail; $y++)
	{
	    if ($emails[$y] == $z)
	    {
	     $email = $y;
	    } 
	}

	$count ++;

	if ($count % 2 == 0) 
	{
	    print OUTFILE "         <TR CLASS=even>\n";
	} 
	else 
	{
	    print OUTFILE "         <TR CLASS=odd>\n";
	}

	$sql = "SELECT IFNULL(NICKNAME,\"-\") FROM TBL_PARTICIPANT WHERE (PARTICIPANT_ID = \"$email\")";
	($nickname) = &DLookup($sql);

	$domain = undef;
#	if ($nickname eq "-")
#	{
	    $sql = "SELECT EMAIL FROM TBL_PARTICIPANT WHERE (PARTICIPANT_ID = \"$email\")";
	    my ($realemail) = &DLookup($sql);
	    ($head, $tail) = split(/@/, $realemail);
	    if ($realemail =~ /\.(\w+)$/) 
	    { 
		$domain = $1; 
	    }
#	}

        $emaildir = "e" . $email;

	if (defined($domain)) 
	{
	    $sql = "SELECT COUNTRY FROM TBL_COUNTRY WHERE (COUNTRY_ID = \"$domain\") LIMIT 1";
	    ($label) = &DLookup($sql);
	    if (defined($label))
	    {
	        $sql = "SELECT COUNTRY_ICON FROM TBL_COUNTRY WHERE (COUNTRY_ID = \"$domain\") LIMIT 1";
		($icon) = &DLookup($sql);
		$icon = ($icon . "." . $UserPrefs{"imageext"});
	    }
	    else
	    {
    		$label = "Unknown";
		$icon = ("unknown." . $UserPrefs{"imageext"});
	    }
	}
	else
	{
	    $label = "Unknown";
	    $icon = ("unknown." . $UserPrefs{"imageext"});
	}

	if ($blocksthathost > 0)
	{
	    $percentofhost = sprintf("%.2f", 100 * $blocks[$email] / $blocksthathost);
	}
	else
	{
	    $percentofhost = sprintf("%.2f", 0);
	}
	
	if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
	{
	    $keyschecked = &keysf($blocks[$email], $Contest);
	    $percentcomplete = &percentf($blocks[$email], 6, $Contest);
	}


	print OUTFILE "          <TD NOWRAP ALIGN=LEFT>\n";
	if ($nickname eq "-")
	{
            print OUTFILE "           <IMG SRC=\"" . $UserPrefs{"htmlroot"} . $UserPrefs{"icondir"} . "flags/$icon\" ALIGN=BOTTOM BORDER=0 WIDTH=14 HEIGHT=14 ALT=\"$label\">&nbsp;$head<A HREF=\"" . $UserPrefs{"htmlroot"} . $SystemPrefs{"contestsuffix"}[$Contest]. "/" . "$emaildir/" . $UserPrefs{"htmlindvemail"} . "\">\@</A>$tail\n";
	}
	else
	{
            print OUTFILE "           <IMG SRC=\"" . $UserPrefs{"htmlroot"} . $UserPrefs{"icondir"} . "flags/$icon\" ALIGN=BOTTOM BORDER=0 WIDTH=14 HEIGHT=14 ALT=\"$label\">&nbsp;<A HREF=\"" . $UserPrefs{"htmlroot"} . $SystemPrefs{"contestsuffix"}[$Contest]. "/" . "$emaildir/" . $UserPrefs{"htmlindvemail"} . "\">$nickname</A>\n";
	}
        print OUTFILE "          </TD>\n",
                      "          <TD NOWRAP ALIGN=RIGHT>\n";
	if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
	{
    	    print OUTFILE "           $blocks[$email]\n";
	}
	else
	{
    	    print OUTFILE "           " . &ratef($blocks[$email] * $SystemPrefs{"contestfactor"}[$Contest]) . "\n";
	}
	print OUTFILE "          </TD>\n",
                      "          <TD NOWRAP ALIGN=RIGHT>\n",
                      "           $percentofhost%\n",
                      "          </TD>\n";
	if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
	{
	    print OUTFILE "          <TD NOWRAP ALIGN=RIGHT>\n",
                      "           $keyschecked\n",
                      "          </TD>\n",
                      "          <TD NOWRAP ALIGN=RIGHT>\n",
                      "           $percentcomplete%\n",
                      "          </TD>\n";
	}
	print OUTFILE "         </TR>\n";
    }

    print OUTFILE "        </TABLE>\n",
                  "       </TD>\n",
                  "      </TR>\n";

    print OUTFILE "      <TR>\n",
                  "       <TD>\n",
                  "        &nbsp;\n",
                  "       </TD>\n",
                  "      </TR>\n";

    print OUTFILE "      <TR>\n",
                  "       <TD NOWRAP ALIGN=CENTER>\n",
                  "        Click here to goto the <A HREF=\"" . $UserPrefs{"htmlroot"} . $SystemPrefs{"contestsuffix"}[$Contest]. "/" . "$hostdir/" . $UserPrefs{"htmlindvhost"} . "\">Executive Hostname Summary</A> page.\n",
                  "       </TD>\n",
                  "      </TR>\n";

    print OUTFILE "      <TR>\n",
                  "       <TD>\n",
                  "        &nbsp;\n",
                  "       </TD>\n",
                  "      </TR>\n";

    print OUTFILE "      <TR>\n",
                  "       <TD NOWRAP ALIGN=CENTER>\n",
                  "        Click here to goto the <A HREF=\"" . $UserPrefs{"htmlroot"} . $SystemPrefs{"contestsuffix"}[$Contest]. "/" . "$hostdir/" . $UserPrefs{"htmlhosthist"} . "\">Graphical Hostname History</A> page.\n",
                  "       </TD>\n",
                  "      </TR>\n";

    print OUTFILE "     </TABLE>\n";
    &write_footer();
    close OUTFILE;
}  #  write_emailsbyhost


#--- Method Prolog -----------------------------------------------------------
#  Name: write_byemail
#
#  Parameters: 
#      Contest - The contest to collect data for. 1=RC5, 2=DES, etc.
#
#  Return Value: None.
#
#  Description:
#   Write stats ordered by email.
#
#--- End Method Prolog -------------------------------------------------------
sub write_byemail
{
    my $Contest = shift @_;
    &Debugln("Entering write_byemail($Contest) ...");

    my $count = 0;
    my $pagecount = 1;
    my $msg;
    my $displacement;
    my ($y, $z);
    my ($sql, $sql2, $sth);
    my (@row, @emails, @blocks);
    my ($email, $nickname, $numemails);
    my $lastseen;
    my $lastseentime;
    my $lastseencolor;
    my @pageblocks;
    my ($year, $month, $day, $hour, $min, $sec, $wday);
    my ($head, $tail);
    my ($emaildir, $realemail);
    my ($domain, $icon, $label);
    my ($blocksthatday, $percentofteam);
    my $blocksthatemail;
    my ($LastSubmittedDate, $LastSubmittedTime);
    my $LastTime;

    &write_section_header("", "Email Address", $UserPrefs{"htmlmail"}, $Contest);

    for ($y = $UserPrefs{"dayshistory"}+1; $y >= 0; $y--) 
    {
	$pageblocks[$y] = 0;
    }

    $sql = "SELECT SUM(WU_SIZE) FROM " . $SystemPrefs{"contestwutable"}[$Contest];
    my ($blocksoverall) = &DLookup($sql);
    $sql = "SELECT MAX(PARTICIPANT_ID) FROM TBL_PARTICIPANT";
    my ($maxemails) = &DLookup($sql);

    $numemails = 0;
    
#SELECT HOST_ID,SUM(WU_SIZE) FROM `TBL_WU_RC564` GROUP BY 1 ORDER BY 2 DESC
    $sql = "SELECT PARTICIPANT_ID,SUM(WU_SIZE) FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " GROUP BY 1 ORDER BY 2 DESC";
    $sth = &SelectSQL($sql);

    while (@row = $sth->fetchrow_array()) 
    {
	$sql = "SELECT RETIRE_TO_ID FROM TBL_PARTICIPANT WHERE (PARTICIPANT_ID = \"" . $row[0] . "\") LIMIT 1";
	my ($retireto) = &DLookup($sql);
	if ($retireto > 0)
	{
	    $blocks[$retireto] += $row[1];
	}
	else
	{
	    $numemails++;
	    $emails[$row[0]] = $numemails;
	    $blocks[$row[0]] += $row[1];
	}
    }

    for ($z = 1; $z <= $numemails; $z++)
    {
	for ($y = 1; $y <=$maxemails; $y++)
	{
	    if ($emails[$y] == $z)
	    {
	     $email = $y;
	    } 
	}
	$count += 1;

	# Start a new file if maxinum number of rows has been reached.
	if ($count > $UserPrefs{"maxrows"} * $pagecount) {
	    &write_section_footer($pagecount, $numemails, 0, $UserPrefs{"htmlmail"}, "Please click on the \"@\" symbol or the nickname for detailed participant information.", $Contest, 0);

	    for ($y = $UserPrefs{"dayshistory"}+1; $y >= 0; $y--) {
		$pageblocks[$y] = 0;
	    }

	    $pagecount += 1;
	    &write_section_header($pagecount, "Email Address", $UserPrefs{"htmlmail"}, $Contest);
	}




	if ($count % 2 == 0) {
	    print OUTFILE "      <TR CLASS=even ALIGN=RIGHT>\n";
	} else {
	    print OUTFILE "      <TR CLASS=odd ALIGN=RIGHT>\n";
	}

#	# Print out the Rank and Movement column.
#	$displacement = $emails_ry{$email} == 0 ? $numemail - $count : $emails_ry{$email} - $count;
#	$msg = &create_move_msg($displacement);

#        print OUTFILE "       <TD NOWRAP ALIGN=LEFT>&nbsp;$count$msg</TD>\n";
        print OUTFILE "       <TD NOWRAP ALIGN=LEFT>&nbsp;$count</TD>\n";



	$domain = undef;
        $label = undef;
	$icon = undef;
	$sql = "SELECT IFNULL(NICKNAME,\"-\") FROM TBL_PARTICIPANT WHERE (PARTICIPANT_ID = \"$email\") LIMIT 1";
	($nickname) = &DLookup($sql);

#	if ($nickname eq "-")
#	{
	    $sql = "SELECT EMAIL FROM TBL_PARTICIPANT WHERE (PARTICIPANT_ID = \"$email\") LIMIT 1";
	    my ($realemail) = &DLookup($sql);
	    if ($realemail =~ /\.(\w+)$/) 
    	    { 
    	        $domain = $1; 
	    }
	    ($head, $tail) = split(/\@/, $realemail);
#	}
	if (defined($domain)) 
	{
	    $sql = "SELECT COUNTRY FROM TBL_COUNTRY WHERE (COUNTRY_ID = \"$domain\") LIMIT 1";
	    ($label) = &DLookup($sql);
	    if (defined($label))
	    {
	        $sql = "SELECT COUNTRY_ICON FROM TBL_COUNTRY WHERE (COUNTRY_ID = \"$domain\") LIMIT 1";
		($icon) = &DLookup($sql);
		$icon = ($icon . "." . $UserPrefs{"imageext"});
	    }
	    else
	    {
		$label = "Unknown";
		$icon = ("unknown." . $UserPrefs{"imageext"});
	    }
	}

	$emaildir = "e" . $email;
	
	if ($nickname eq "-")
	{
	    print OUTFILE "       <TD NOWRAP ALIGN=LEFT><IMG SRC=\"" . $UserPrefs{"htmlroot"} . $UserPrefs{"icondir"} . "flags/$icon\" ALIGN=BOTTOM BORDER=0 WIDTH=14 HEIGHT=14 ALT=\"$label\">&nbsp;$head<A HREF=\"" . $UserPrefs{"htmlroot"} . $SystemPrefs{"contestsuffix"}[$Contest]. "/" . "$emaildir/" . $UserPrefs{"htmlindvemail"} . "\">\@</A>$tail</TD>\n";
	}
	else
	{
	    print OUTFILE "       <TD NOWRAP ALIGN=LEFT><IMG SRC=\"" . $UserPrefs{"htmlroot"} . $UserPrefs{"icondir"} . "flags/$icon\" ALIGN=BOTTOM BORDER=0 WIDTH=14 HEIGHT=14 ALT=\"$label\">&nbsp;<A HREF=\"" . $UserPrefs{"htmlroot"} . $SystemPrefs{"contestsuffix"}[$Contest]. "/" . "$emaildir/" . $UserPrefs{"htmlindvemail"} . "\">$nickname</A></TD>\n";
	}
	
	for ($y = $UserPrefs{"dayshistory"} - 3; $y >= 0; $y--) 
	{
	    ($year, $month, $day, $hour, $min, $sec, $wday) = &nice_gmtime($SystemPrefs{"timebegin"} - $y * (24*60*60));
	    $sql = "SELECT IFNULL(SUM(WU_SIZE),\"-\") FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE ((PARTICIPANT_ID = \"$email\") AND (SUBMITTED_DATE = \"$year-$month-$day\"))";
	    ($blocksthatday) = &DLookup($sql);
	    $sql = "SELECT PARTICIPANT_ID FROM TBL_PARTICIPANT WHERE (RETIRE_TO_ID = \"$email\")";
	    my @row;
	    my $sth = &SelectSQL($sql);
	    while (@row = $sth->fetchrow_array()) 
	    {
		$sql2 = "SELECT IFNULL(SUM(WU_SIZE),\"0\") FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE ((PARTICIPANT_ID = \"$row[0]\") AND (SUBMITTED_DATE = \"$year-$month-$day\"))";
    		my ($additionalblocksthatday) = &DLookup($sql2);
		$blocksthatday+= $additionalblocksthatday;
	    }



	    if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
	    {
		print OUTFILE "       <TD>$blocksthatday</TD>\n";
	    }
	    else
	    {
		if ($blocksthatday != "-")
		{
		    print OUTFILE "       <TD>" . &ratef($blocksthatday * $SystemPrefs{"contestfactor"}[$Contest]) . "</TD>\n";
		}
		else
		{
		    print OUTFILE "       <TD>$blocksthatday</TD>\n";
		}
	    }
	    if ($blocksthatday != "-")
	    {
	        $pageblocks[$y+1] += $blocksthatday;
	    }
	}

	$sql = "SELECT IFNULL(SUM(WU_SIZE),\"-\") FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE (PARTICIPANT_ID = \"$email\")";
	($blocksthatemail) = &DLookup($sql);

	$sql = "SELECT PARTICIPANT_ID FROM TBL_PARTICIPANT WHERE (RETIRE_TO_ID = \"$email\")";
	my @row;
	my $sth = &SelectSQL($sql);
	while (@row = $sth->fetchrow_array()) 
	{
	    $sql2 = "SELECT IFNULL(SUM(WU_SIZE),\"0\") FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE (PARTICIPANT_ID = \"$row[0]\")";
    	    my ($additionalblocksthatemail) = &DLookup($sql2);
	    $blocksthatemail+= $additionalblocksthatemail;
	}
	
	if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
	{
	    print OUTFILE "       <TD>$blocksthatemail</TD>\n";
	}
	else
	{
	    if ($blocksthatemail != "-")
	    {
		print OUTFILE "       <TD>" . &ratef($blocksthatemail * $SystemPrefs{"contestfactor"}[$Contest]) . "</TD>\n";
	    }
	    else
	    {
		print OUTFILE "       <TD>$blocksthatemail</TD>\n";
	    }
	}
	if ($blocksthatemail != "-")
	{
    	    $pageblocks[0] += $blocksthatemail;
	}
	
	# Print out the Last Seen column.
        $sql = "SELECT MAX(SUBMITTED_DATE) FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE (PARTICIPANT_ID = \"$email\")";
	($LastSubmittedDate) = &DLookup($sql);
        $sql = "SELECT MAX(SUBMITTED_TIME) FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE ((SUBMITTED_DATE = \"$LastSubmittedDate\") AND (PARTICIPANT_ID = \"$email\"))";
	($LastSubmittedTime) = &DLookup($sql);
	$sql = "SELECT UNIX_TIMESTAMP(\"$LastSubmittedDate $LastSubmittedTime\")";
	($LastTime) = &DLookup($sql);

	$sql = "SELECT PARTICIPANT_ID FROM TBL_PARTICIPANT WHERE (RETIRE_TO_ID = \"$email\")";
	my @row;
	my $sth = &SelectSQL($sql);
	while (@row = $sth->fetchrow_array()) 
	{
    	    $sql2 = "SELECT MAX(SUBMITTED_DATE) FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE (PARTICIPANT_ID = \"" . $row[0] . "\")";
	    ($LastSubmittedDate) = &DLookup($sql2);
    	    $sql2 = "SELECT MAX(SUBMITTED_TIME) FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE ((SUBMITTED_DATE = \"$LastSubmittedDate\") AND (PARTICIPANT_ID = \"" . $row[0] . "\"))";
	    ($LastSubmittedTime) = &DLookup($sql2);
	    $sql = "SELECT UNIX_TIMESTAMP(\"$LastSubmittedDate $LastSubmittedTime\")";
	    my ($LastTimeTemp) = &DLookup($sql);
	    if ($LastTimeTemp > $LastTime)
	    {
		$LastTime = $LastTimeTemp;
	    }
	}

	$lastseentime = ($SystemPrefs{"timebegin"}-($LastTime));
	$lastseen = &ls($lastseentime);
	$lastseencolor = &ls_color($lastseentime);

	print OUTFILE "       <TD CLASS=$lastseencolor NOWRAP>$lastseen</TD>\n";

	# Print out the Percent column.
	if ($blocksoverall > 0)
	{
	    $percentofteam = sprintf("%.2f", 100 * $blocks[$email] / $blocksoverall);
	}
	else
	{
	    $percentofteam = sprintf("%.2f", 0);
	}

	print OUTFILE "       <TD>$percentofteam%</TD>\n",
	              "      </TR>\n";
    }

    &write_section_footer($pagecount, $numemails, 0, $UserPrefs{"htmlmail"}, "Please click on the \"@\" symbol or the nickname for detailed participant information.", $Contest, 0);
   
    &write_indvemail($Contest);

    &Debugln("Exiting write_byemail($Contest) ...");
}  # write_byemail


#--- Method Prolog -----------------------------------------------------------
#  Name: write_indvemail
#
#  Parameters: 
#      Contest - The contest to collect data for. 1=RC5, 2=DES, etc.
#
#  Return Value: None.
#
#  Description:
#    Routine to write individual email info.
#
#--- End Method Prolog -------------------------------------------------------
sub write_indvemail
{
    my $Contest = shift @_;    
    my $count = 0;
    my $msg;
    my $displacement;
    my ($x, $y, $z);
    my ($firstblockdate, $lastblockdate);
    my ($lastseentime, $lastseen, $lastseencolor);
    my ($totaldays, $aveblocks, $averate, $averatef);    
    my ($year, $mon, $mday, $hour, $min, $sec, $wday);
    my ($email, $nickname, $emaildir, $sql, $sql2);
    my ($label, $icon, $domain);
    my ($contestblocks, $keyschecked, $percentcomplete, $percentofteam);
    my $sth;
    my @row;
    my @emails;
    my @blocks;
    my ($head, $tail);
    my ($blocksthatemail, $rate);
    my ($blocksyesterday, $rateyesterday);
    my $blocksthatday;
    my $colsp;

    $sql = "SELECT SUM(WU_SIZE) FROM " . $SystemPrefs{"contestwutable"}[$Contest];
    my ($blocksoverall) = &DLookup($sql);
    $sql = "SELECT PARTICIPANT_ID, SUM(WU_SIZE) FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " GROUP BY 1 ORDER BY 2 DESC";
    $sth = &SelectSQL($sql);

    my $numemails = 0;
    $sql = "SELECT MAX(PARTICIPANT_ID) FROM TBL_PARTICIPANT";
    my ($maxemail) = &DLookup($sql);
    while (@row = $sth->fetchrow_array()) 
    {
	$sql = "SELECT RETIRE_TO_ID FROM TBL_PARTICIPANT WHERE (PARTICIPANT_ID = \"" . $row[0] . "\") LIMIT 1";
	my ($retireto) = &DLookup($sql);
	if ($retireto > 0)
	{
	    $blocks[$retireto] += $row[1];
	}
	else
	{
	    $numemails++;
	    $emails[$row[0]] = $numemails;
	    $blocks[$row[0]] += $row[1];
	}
    }

    $count = 0;

    for ($z = 1; $z <= $numemails; $z++)
    {
	for ($y = 1; $y <=$maxemail; $y++)
	{
	    if ($emails[$y] == $z)
	    {
	     $email = $y;
	    } 
	}

	$count ++;

	# Print out the Rank and Movement column.
#	$displacement = $emails_ry{$email} == 0 ? $numemail - $count : $emails_ry{$email} - $count;
#	$msg = &create_move_msg($displacement);

	
	$sql = "SELECT UNIX_TIMESTAMP(MAX(SUBMITTED_DATE)) FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE (PARTICIPANT_ID = \"$email\")";
	my ($LastSubmitted) = &DLookup($sql);
	$sql = "SELECT UNIX_TIMESTAMP(MIN(SUBMITTED_DATE)) FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE (PARTICIPANT_ID = \"$email\")";
	my ($FirstSubmitted) =&DLookup($sql);
    
	$sql = "SELECT PARTICIPANT_ID FROM TBL_PARTICIPANT WHERE (RETIRE_TO_ID = \"$email\")";
	$sth = &SelectSQL($sql);
	while (@row = $sth->fetchrow_array()) 
	{
    	    $sql2 = "SELECT UNIX_TIMESTAMP(MAX(SUBMITTED_DATE)) FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE (PARTICIPANT_ID = \"" . $row[0] . "\")";
	    my ($LastSubmittedTemp) = &DLookup($sql2);
	    if ($LastSubmittedTemp > $LastSubmitted)
	    {
		$LastSubmitted = $LastSubmittedTemp;
	    }
    	    $sql2 = "SELECT IFNULL(UNIX_TIMESTAMP(MIN(SUBMITTED_DATE)),\"-\") FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE (PARTICIPANT_ID = \"" . $row[0] . "\")";
	    my ($FirstSubmittedTemp) = &DLookup($sql2);
	    if (($FirstSubmittedTemp < $FirstSubmitted) && ($FirstSubmittedTemp ne "-"))
	    {
		$FirstSubmitted = $FirstSubmittedTemp;
	    }
	}

	my $totaldays = int((($LastSubmitted - $FirstSubmitted) / (60*60*24)) + 1);
	my $aveblocks = int($blocks[$email] / $totaldays);
	my $averate = &rate_day($blocks[$email] / $totaldays, $SystemPrefs{"contestfactor"}[$Contest], $Contest);
	my $averatef = &ratef($averate);

        $colsp = 1 + $UserPrefs{"dayshistory"};

	($year, $mon, $mday, $hour, $min, $sec, $wday) = &nice_gmtime($FirstSubmitted);
	$firstblockdate = "$mday $Months[$mon-1] $year GMT";
	($year, $mon, $mday, $hour, $min, $sec, $wday) = &nice_gmtime($LastSubmitted);
	$lastblockdate = "$mday $Months[$mon-1] $year GMT";


	# Print out the Last Seen column.
        $sql = "SELECT MAX(SUBMITTED_DATE) FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE (PARTICIPANT_ID = \"$email\")";
	my ($LastSubmittedDate) = &DLookup($sql);
        $sql = "SELECT MAX(SUBMITTED_TIME) FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE ((SUBMITTED_DATE = \"$LastSubmittedDate\") AND (PARTICIPANT_ID = \"$email\"))";
	my ($LastSubmittedTime) = &DLookup($sql);
	$sql = "SELECT UNIX_TIMESTAMP(\"$LastSubmittedDate $LastSubmittedTime\")";
	my ($LastTime) = &DLookup($sql);

	$sql = "SELECT PARTICIPANT_ID FROM TBL_PARTICIPANT WHERE (RETIRE_TO_ID = \"$email\")";
	my @row;
	my $sth = &SelectSQL($sql);
	while (@row = $sth->fetchrow_array()) 
	{
    	    $sql2 = "SELECT MAX(SUBMITTED_DATE) FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE (PARTICIPANT_ID = \"" . $row[0] . "\")";
	    ($LastSubmittedDate) = &DLookup($sql2);
    	    $sql2 = "SELECT MAX(SUBMITTED_TIME) FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE ((SUBMITTED_DATE = \"$LastSubmittedDate\") AND (PARTICIPANT_ID = \"" . $row[0] . "\"))";
	    ($LastSubmittedTime) = &DLookup($sql2);
	    $sql2 = "SELECT UNIX_TIMESTAMP(\"$LastSubmittedDate $LastSubmittedTime\")";
	    my ($LastTimeTemp) = &DLookup($sql2);
	    if ($LastTimeTemp > $LastTime)
	    {
		$LastTime = $LastTimeTemp;
	    }
	}

	$lastseentime = ($SystemPrefs{"timebegin"}-($LastTime));
	$lastseen = &ls($lastseentime);
	$lastseencolor = &ls_color($lastseentime);


        ($year, $mon, $mday, $hour, $min, $sec, $wday) = &nice_gmtime($SystemPrefs{"timebegin"} - (1 * (24*60*60)));
#SELECT SUM(WU_SIZE) FROM `TBL_WU_CSC` WHERE DATE_FORMAT(SUBMITTED,"%Y-%m-%d") = "1999-07-11"
	$sql = "SELECT SUM(WU_SIZE) FROM " . $SystemPrefs{"contestwutable"}[$Contest] . 
	   " WHERE ((SUBMITTED_DATE = \"$year-$mon-$mday\") AND (PARTICIPANT_ID = \"$email\"))";
	($blocksyesterday) = &DLookup($sql);
    
        $sql = "SELECT PARTICIPANT_ID FROM TBL_PARTICIPANT WHERE RETIRE_TO_ID = \"$email\"";
	my @row;
	my $sth = &SelectSQL($sql);
	while (@row = $sth->fetchrow_array()) 
	{
	    $sql2 = "SELECT IFNULL(SUM(WU_SIZE),\"0\") FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE ((PARTICIPANT_ID = \"$row[0]\") AND (SUBMITTED_DATE = \"$year-$mon-$mday\"))";
    	    my ($additionalblocksyesterday) = &DLookup($sql2);
	    $blocksyesterday+= $additionalblocksyesterday;
	}

	$rateyesterday = &ratef(&rate_day($blocksyesterday, $SystemPrefs{"contestfactor"}[$Contest], $Contest));

	if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
        {
	    $contestblocks = &get_contest_blocks($Contest);
	    $keyschecked = &keysf($blocks[$email], $Contest);
	    $percentcomplete = &percentf($blocks[$email], 6, $Contest);
	}
	if ($blocksoverall > 0)
	{
	    $percentofteam = sprintf("%.2f", 100 * $blocks[$email] / $blocksoverall);
	}
	else
	{
	    $percentofteam = sprintf("%.2f", 0);
	}

	$domain = undef;
	$sql = "SELECT IFNULL(NICKNAME,\"-\") FROM TBL_PARTICIPANT WHERE (PARTICIPANT_ID = \"$email\") LIMIT 1";
	($nickname) = &DLookup($sql);

#	if ($nickname eq "-")
#	{
    	    $sql = "SELECT EMAIL FROM TBL_PARTICIPANT WHERE (PARTICIPANT_ID = \"$email\")";
	    my ($realemail) = &DLookup($sql);
	    ($head, $tail) = split(/@/, $realemail);
	    if ($realemail =~ /\.(\w+)$/) 
	    { 
		$domain = $1; 
	    }
#	}

	if (defined($domain)) 
	{
	    $sql = "SELECT COUNTRY FROM TBL_COUNTRY WHERE (COUNTRY_ID = \"$domain\") LIMIT 1";
	    ($label) = &DLookup($sql);
	    if (defined($label))
	    {
	        $sql = "SELECT COUNTRY_ICON FROM TBL_COUNTRY WHERE (COUNTRY_ID = \"$domain\") LIMIT 1";
		($icon) = &DLookup($sql);
		$icon = ($icon . "." . $UserPrefs{"imageext"});
	    }
	    else
	    {
    		$label = "Unknown";
		$icon = ("unknown." . $UserPrefs{"imageext"});
	    }
	}
	else
	{
	    $label = "Unknown";
	    $icon = ("unknown." . $UserPrefs{"imageext"});
	}

	$emaildir = "e" . $email;

	mkdir($UserPrefs{"htmldir"} . $SystemPrefs{"contestsuffix"}[$Contest] . "/" . "$emaildir", 0755);
	open(OUTFILE, ">" . $UserPrefs{"htmldir"} . $SystemPrefs{"contestsuffix"}[$Contest] . "/$emaildir/" . $UserPrefs{"htmlindvemail"}) or die "Personal Proxy Statistics : Cannot open " . $UserPrefs{"htmldir"} . $SystemPrefs{"contestsuffix"}[$Contest] . "/$emaildir/" . $UserPrefs{"htmlindvemail"} . " ($!)\n";
	&write_header($Contest);
	print OUTFILE "     <TABLE BORDER=0 CELLPADDING=0 CELLSPACING=0 WIDTH=\"100%\">\n";
	print OUTFILE "      <TR ALIGN=CENTER>\n",
	              "       <TD CLASS=title NOWRAP>\n";
	if ($nickname eq "-")
	{
    	    print OUTFILE "        <STRONG>Participant $realemail</STRONG><BR>\n";
	}
	else
	{
    	    print OUTFILE "        <STRONG>Participant $nickname</STRONG><BR>\n";
	}
        print OUTFILE "        Executive Participant Summary\n",
                      "       </TD>\n",
                      "      </TR>\n";

	print OUTFILE "      <TR>\n",
                      "       <TD>\n",
                      "        &nbsp;\n",
                      "       </TD>\n",
                      "      </TR>\n";

	print OUTFILE "      <TR ALIGN=CENTER>\n",
                      "       <TD VALIGN=TOP WIDTH=\"100%\">\n",
                      "        <TABLE BORDER=0 CELLPADDING=2 CELLSPACING=0>\n";

	print OUTFILE "         <TR>\n",
                      "          <TD CLASS=subtitle NOWRAP COLSPAN=2 ALIGN=CENTER>\n";
        if ($nickname eq"-")
	{
    	    print OUTFILE "           Participant Summary for $realemail\n";
	}
	else
	{
    	    print OUTFILE "           Participant Summary for $nickname\n";
	}
	print OUTFILE "          </TD>\n",
                      "         </TR>\n";

	print OUTFILE "         <TR>\n",
                      "          <TD NOWRAP>\n",
                      "           Current Team Rank\n",
                      "          </TD>\n",
                      "          <TD NOWRAP ALIGN=RIGHT>\n",
                      "           $count$msg\n",
                      "          </TD>\n",
                      "         </TR>\n";

	print OUTFILE "         <TR>\n",
                      "          <TD NOWRAP>\n",
                      "           Percent of Team\n",
                      "          </TD>\n",
                      "          <TD NOWRAP ALIGN=RIGHT>\n",
                      "           $percentofteam%\n",
                      "          </TD>\n",
                      "         </TR>\n";

	if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
        {
	    print OUTFILE "         <TR>\n",
                      "          <TD NOWRAP>\n",
                      "           Total " . $SystemPrefs{"contestheaderblocktype"}[$Contest] . "s to Search\n",
                      "          </TD>\n",
                      "          <TD NOWRAP ALIGN=RIGHT>\n",
                      "           $contestblocks " . $SystemPrefs{"contestblocktype"}[$Contest] . "s\n",
                      "          </TD>\n",
                      "         </TR>\n";
	}

	print OUTFILE "         <TR>\n",
                      "          <TD NOWRAP>\n",
                      "           Total " . $SystemPrefs{"contestheaderblocktype"}[$Contest] . "s Checked\n",
                      "          </TD>\n",
                      "          <TD NOWRAP ALIGN=RIGHT>\n";
	if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
        {
            print OUTFILE "           $blocks[$email] " . $SystemPrefs{"contestblocktype"}[$Contest] . "s\n";
	}
	else
	{
            print OUTFILE "           " . &ratef($blocks[$email] * $SystemPrefs{"contestfactor"}[$Contest]) . $SystemPrefs{"contestblocktype"}[$Contest] . "s\n";
	}
        print OUTFILE "          </TD>\n",
                      "         </TR>\n";

	if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
        {
	    print OUTFILE "         <TR>\n",
                      "          <TD NOWRAP>\n",
                      "           Keyspace Exhausted\n",
                      "          </TD>\n",
                      "          <TD NOWRAP ALIGN=RIGHT>\n",
                      "           $percentcomplete%\n",
                      "          </TD>\n",
                      "         </TR>\n";

	    print OUTFILE "         <TR>\n",
                      "          <TD NOWRAP>\n",
                      "           Total Keys Checked\n",
                      "          </TD>\n",
                      "          <TD NOWRAP ALIGN=RIGHT>\n",
                      "           $keyschecked\n",
                      "          </TD>\n",
                      "         </TR>\n";
	}

	print OUTFILE "         <TR>\n",
                      "          <TD NOWRAP>\n",
                      "           Time Working\n",
                      "          </TD>\n",
                      "          <TD NOWRAP ALIGN=RIGHT>\n",
                      "           $totaldays days\n",
                      "          </TD>\n",
                      "         </TR>\n";

	print OUTFILE "         <TR>\n",
                      "          <TD NOWRAP>\n",
                      "           Last Seen\n",
                      "          </TD>\n",
                      "          <TD CLASS=$lastseencolor NOWRAP ALIGN=RIGHT>\n",
                      "           $lastseen\n",
                      "          </TD>\n",
                      "         </TR>\n";

	print OUTFILE "         <TR>\n",
                      "          <TD NOWRAP>\n",
                      "           First " . $SystemPrefs{"contestheaderblocktype"}[$Contest] . " Submitted\n",
                      "          </TD>\n",
                      "          <TD NOWRAP ALIGN=RIGHT>\n",
                      "           $firstblockdate\n",
                      "          </TD>\n",
                      "         </TR>\n";

	print OUTFILE "         <TR>\n",
                      "          <TD NOWRAP>\n",
                      "           Last " . $SystemPrefs{"contestheaderblocktype"}[$Contest] . " Submitted\n",
                      "          </TD>\n",
                      "          <TD NOWRAP ALIGN=RIGHT>\n",
                      "           $lastblockdate\n",
                      "          </TD>\n",
                      "         </TR>\n";

	print OUTFILE "         <TR>\n",
                      "          <TD NOWRAP>\n",
                      "           Overall Rate\n",
                      "          </TD>\n",
                      "          <TD NOWRAP ALIGN=RIGHT>\n",
                      "           $averatef" . $SystemPrefs{"contestheaderblockmeasure"}[$Contest] . "s/sec\n",
                      "          </TD>\n",
                      "         </TR>\n";

	print OUTFILE "         <TR>\n",
                      "          <TD NOWRAP>\n",
                      "           Average " . $SystemPrefs{"contestheaderblocktype"}[$Contest] . "s\n",
                      "          </TD>\n",
                      "          <TD NOWRAP ALIGN=RIGHT>\n";
	if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
        {
            print OUTFILE "           $aveblocks per day\n";
	}
	else
	{
            print OUTFILE "           " . &ratef($aveblocks * $SystemPrefs{"contestfactor"}[$Contest]) . " per day\n";
	}
        print OUTFILE "          </TD>\n",
                      "         </TR>\n";

	print OUTFILE "         <TR>\n",
                      "          <TD NOWRAP>\n",
                      "           Distributed.Net Rank\n",
                      "          </TD>\n",
                      "          <TD NOWRAP ALIGN=RIGHT>\n";
	if ($nickname eq "-")
	{
	    print OUTFILE "           <A HREF=\"" . $SystemPrefs{"stats-server"} . "/participant/psearch.php" . $SystemPrefs{"conteststats"}[$Contest] . "&st=$realemail\">$realemail</A>\n",
	}
	else
	{
    	    if ($UserPrefs{"disableemailranksearch"})
    	    {
		print OUTFILE "           Disabled\n";
	    }
	    else
	    {
		print OUTFILE "           <A HREF=\"" . $SystemPrefs{"stats-server"} . "/participant/psearch.php" . $SystemPrefs{"conteststats"}[$Contest] . "&st=$realemail\">$nickname</A>\n",
	    }
	}
	print OUTFILE "          </TD>\n",
                      "         </TR>\n";

	print OUTFILE "         <TR>\n",
                      "          <TD NOWRAP>\n",
                      "           Participant Domain\n",
                      "          </TD>\n",
                      "          <TD NOWRAP ALIGN=RIGHT>\n",
                      "           $label&nbsp;<IMG SRC=\"" . $UserPrefs{"htmlroot"} . $UserPrefs{"icondir"} . "flags/$icon\" ALIGN=BOTTOM BORDER=0 WIDTH=14 HEIGHT=14 ALT=\"$label\">\n",
                      "          </TD>\n",
                      "         </TR>\n";

	print OUTFILE "        </TABLE>\n",
                      "       </TD>\n",
                      "      </TR>\n";

	print OUTFILE "      <TR>\n",
                      "       <TD>\n",
                      "        &nbsp;\n",
                      "       </TD>\n",
                      "      </TR>\n";

	if ($blocksyesterday == 0) {
	    print OUTFILE "      <TR>\n",
                          "       <TD NOWRAP ALIGN=CENTER>\n",
                          "        This participant didn't submit any " . $SystemPrefs{"contestheaderblocktype"}[$Contest] . "s yesterday, someone tell them to get cracking!\n",
                          "       </TD>\n",
                          "      </TR>\n";
	} 
	else 
	{
	    $percentcomplete = &percentf($blocksyesterday, 8, $Contest);
	    print OUTFILE "      <TR>\n",
	                  "       <TD NOWRAP ALIGN=CENTER>\n";
	    if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
    	    {
                print OUTFILE "        $blocksyesterday " . $SystemPrefs{"contestheaderblocktype"}[$Contest] . "s were completed yesterday ($percentcomplete% of the keyspace)\n";
	    }
	    else
	    {
                print OUTFILE "        " . &ratef($blocksyesterday * $SystemPrefs{"contestfactor"}[$Contest]) . $SystemPrefs{"contestheaderblocktype"}[$Contest] . "s were completed yesterday\n";
	    }
	    print OUTFILE "        <BR>at a sustained rate of $rateyesterday" . $SystemPrefs{"contestheaderblockmeasure"}[$Contest] . "s/sec!\n",
                          "       </TD>\n",
                          "      </TR>\n";
	}

	print OUTFILE "      <TR>\n",
                      "       <TD>\n",
                      "        &nbsp;\n",
                      "       </TD>\n",
                      "      </TR>\n";

	print OUTFILE "      <TR>\n",
                      "       <TD NOWRAP ALIGN=CENTER>\n",
                      "        Click here to goto the <A HREF=\"" . $UserPrefs{"htmlroot"} . $SystemPrefs{"contestsuffix"}[$Contest] . "/" . "$emaildir/" . $UserPrefs{"htmlemailhist"} . "\">Graphical Participant History</A> page.\n",
                      "       </TD>\n",
                      "      </TR>\n";

	print OUTFILE "      <TR>\n",
                      "       <TD>\n",
                      "        &nbsp;\n",
                      "       </TD>\n",
                      "      </TR>\n";

	if ($UserPrefs{"showbyhost"}) {
	    print OUTFILE "      <TR>\n",
                          "       <TD NOWRAP ALIGN=CENTER>\n",
                          "        Click here to goto the <A HREF=\"" . $UserPrefs{"htmlroot"} . $SystemPrefs{"contestsuffix"}[$Contest] . "/" . "$emaildir/" . $UserPrefs{"htmlhostsbyemail"} . "\">Hostnames working for this Participant</A> page.\n",
                          "       </TD>\n",
                          "      </TR>\n";

            print OUTFILE "      <TR>\n",
                          "       <TD>\n",
                          "        &nbsp;\n",
                          "       </TD>\n",
                          "      </TR>\n";
	}

	print OUTFILE "      <TR>\n",
                      "       <TD NOWRAP ALIGN=CENTER>\n",
                      "        Participant History for the Last " . $UserPrefs{"blockhistory"} . " Days\n",
                      "       </TD>\n",
                      "      </TR>\n";

	print OUTFILE "      <TR>\n",
                      "       <TD>\n",
                      "        &nbsp;\n",
                      "       </TD>\n",
                      "      </TR>\n";

	print OUTFILE "      <TR ALIGN=CENTER>\n",
                      "       <TD VALIGN=TOP WIDTH=\"100%\">\n",
                      "        <TABLE BORDER=0 CELLPADDING=2 CELLSPACING=0>\n";

	print OUTFILE "         <TR ALIGN=CENTER>\n",
                      "          <TD CLASS=section NOWRAP>\n",
                      "           Day History\n",
                      "          </TD>\n",
                      "          <TD CLASS=section NOWRAP>\n",
                      "           " . $SystemPrefs{"contestheaderblocktype"}[$Contest] . "s Submited\n",
                      "          </TD>\n",
                      "          <TD CLASS=section NOWRAP>\n",
                      "           Percent\n",
                      "          </TD>\n";
	if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
    	{
	    print OUTFILE "          <TD CLASS=section NOWRAP>\n",
                      "           " . $SystemPrefs{"contestheaderblockmeasure"}[$Contest] . "s Checked\n",
                      "          </TD>\n",
                      "          <TD CLASS=section NOWRAP>\n",
                      "           " . $SystemPrefs{"contestheaderblockmeasure"}[$Contest] . " Rate\n",
                      "          </TD>\n",
                      "          <TD CLASS=section NOWRAP>\n",
                      "           Keyspace\n",
                      "          </TD>\n";
	}
        print OUTFILE "         </TR>\n";
	
	for ($y = 0; $y <= $UserPrefs{"blockhistory"}; $y++) 
	{
	    ($year, $mon, $mday, $hour, $min, $sec, $wday) = &nice_gmtime($SystemPrefs{"timebegin"} - ($y * (24*60*60)));
	    my $blockdate = "$mday $Months[$mon-1] $year GMT";

	    my $blockshost = 0;
    	    $sql = "SELECT IFNULL(SUM(WU_SIZE),0) FROM " . $SystemPrefs{"contestwutable"}[$Contest] . 
		" WHERE ((SUBMITTED_DATE = \"$year-$mon-$mday\") AND (PARTICIPANT_ID = \"$email\"))";
    	    ($blockshost) = &DLookup($sql);
	    $sql = "SELECT PARTICIPANT_ID FROM TBL_PARTICIPANT WHERE (RETIRE_TO_ID = \"$email\")";
	    ($sth) = &SelectSQL($sql);
	    while (@row = $sth->fetchrow_array()) 
	    {
        	$sql = "SELECT IFNULL(SUM(WU_SIZE),0) FROM " . $SystemPrefs{"contestwutable"}[$Contest] . 
		" WHERE ((SUBMITTED_DATE = \"$year-$mon-$mday\") AND (PARTICIPANT_ID = \"" . $row[0] . "\"))";
        	my ($otherblocks) = &DLookup($sql);
		if ($otherblocks > 0)
		{
		    $blockshost += $otherblocks;
		}
	    }
	    $sql = "SELECT IFNULL(SUM(WU_SIZE),0) FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE (SUBMITTED_DATE = \"$year-$mon-$mday\")";
	    ($blocksthatday) =&DLookup($sql);

	    if ($y > 1)
	    { 
		$rate = &ratef(&rate_day($blockshost, $SystemPrefs{"contestfactor"}[$Contest], $Contest));
	    }
	    else
	    {
		$rate = &ratef(&rate_day($blockshost / ($SystemPrefs{"secstoday"} / (60*60*24)), $SystemPrefs{"contestfactor"}[$Contest], $Contest));
	    }
	    
	    if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
    	    {
		$keyschecked = &keysf($blockshost, $Contest);
		$percentcomplete = &percentf($blockshost, 8, $Contest);
	    }
	    
	    if ($blocksthatday > 0)
	    {
		$percentofteam = sprintf("%.2f", 100 * $blockshost / $blocksthatday);
	    }
	    else
	    {
		$percentofteam = sprintf("%.2f", 0);
	    }

	    if ($y % 2 == 0) 
	    {
		print OUTFILE "         <TR CLASS=even>\n";
	    } 
	    else 
	    {
		print OUTFILE "         <TR CLASS=odd>\n";
	    }
	    print OUTFILE "          <TD NOWRAP ALIGN=LEFT>\n",
	              "           $blockdate\n",
                      "          </TD>\n",
                      "          <TD NOWRAP ALIGN=RIGHT>\n";
            if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
    	    {
	        print OUTFILE "           $blockshost " . $SystemPrefs{"contestblocktype"}[$Contest] . "s\n";
            }
	    else
    	    {
	        print OUTFILE "           " . &ratef($blockshost * $SystemPrefs{"contestfactor"}[$Contest]) . $SystemPrefs{"contestblocktype"}[$Contest] . "s\n";
            }
	    print OUTFILE "          </TD>\n",
                      "          <TD NOWRAP ALIGN=RIGHT>\n",
                      "           $percentofteam%\n",
		      "          </TD>\n";
	    if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
    	    {
                print OUTFILE "          <TD NOWRAP ALIGN=RIGHT>\n",
                      "           $keyschecked\n",
                      "          </TD>\n",
                      "          <TD NOWRAP ALIGN=RIGHT>\n",
                      "           $rate" . $SystemPrefs{"contestblockmeasure"}[$Contest] . "s/sec\n",
                      "          </TD>\n",
                      "          <TD NOWRAP ALIGN=RIGHT>\n",
                      "           $percentcomplete%\n",
                      "          </TD>\n";
	    }
	    print OUTFILE "         </TR>\n";
	}

	print OUTFILE "        </TABLE>\n",
                      "       </TD>\n",
                      "      </TR>\n";

	print OUTFILE "     </TABLE>\n";
	&write_footer();
	close OUTFILE;

#	&write_emailchart($email, $realemail, $emaildir, $blocks[$email], $haveretired, $Contest);
	if ($nickname eq "-")
	{
	    &write_emailchart($email, $realemail, $emaildir, $Contest);
	    &write_hostsbyemail($email, $realemail, $emaildir, $blocks[$email], $Contest) if ($UserPrefs{"showbyhost"});
	}
	else
	{
	    &write_emailchart($email, $nickname, $emaildir, $Contest);
	    &write_hostsbyemail($email, $nickname, $emaildir, $blocks[$email], $Contest) if ($UserPrefs{"showbyhost"});
	}
	
	&Debug(".");
    }

    &Debugln("") if ($count > 0);
}  # write_indvemail

#--- Method Prolog -----------------------------------------------------------
#  Name: write_emailchart
#
#  Parameters: 
#      email - emailid from database
#      realemail - emailaddress (string) from database
#      emaildir - ("e".$email)
###DISABLED      blocksthatemail - blocks/workunits overall for this emailaddress 
#      haveretired - this emailaddress is one where others have retired TO 
#      Contest - The contest to collect data for. 1=RC5, 2=DES, etc.
#
#  Return Value: None.
#
#  Description:
#    Routine to create the Graphical Participant History plot, write the GNUplot 
#    config file and plot a chart for this Participant.
#
#--- End Method Prolog -------------------------------------------------------
sub write_emailchart 
{
    my $email = shift @_;
    my $realemail = shift @_;
    my $emaildir = shift @_;
    my $Contest = shift @_;
    my ($x, $y, $z);
    my ($year, $month, $mday, $hour, $min, $sec, $wday);
    my (@rate, @row, @row2);
    my ($sql, $sql2, $sth, $sth2);
    my ($blocks, $tempblocks);
    my ($FirstSubmitted, $LastSubmitted);
    my $FirstSubmittedRetire;
    my @retired = undef;
    my $colsp;

    $sql = "SELECT IFNULL(SUM(WU_SIZE),0) FROM " . $SystemPrefs{"contestwutable"}[$Contest] . 
    " WHERE (PARTICIPANT_ID = \"$email\")";
    ($blocks) = &DLookup($sql);

    $sql = "SELECT PARTICIPANT_ID FROM TBL_PARTICIPANT WHERE RETIRE_TO_ID = \"$email\"";
    $sth = &SelectSQL($sql);
    while (@row = $sth->fetchrow_array()) 
    {
	$sql2 = "SELECT IFNULL(SUM(WU_SIZE),\"0\") FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE (PARTICIPANT_ID = \"$row[0]\")";
	my ($additionalblocks) = &DLookup($sql2);
	$blocks+= $additionalblocks;
    }

    open(OUTFILE, ">" . $UserPrefs{"htmldir"} . $SystemPrefs{"contestsuffix"}[$Contest] . "/$emaildir/" . $UserPrefs{"htmlemailhist"}) or die "Personal Proxy Statistics : Cannot open " . $UserPrefs{"htmldir"} . $SystemPrefs{"contestsuffix"}[$Contest] . "/$emaildir/" . $UserPrefs{"htmlemailhist"} ." ($!)\n";
    &write_header($Contest);
    print OUTFILE "     <TABLE BORDER=0 CELLPADDING=0 CELLSPACING=0 WIDTH=\"100%\">\n";
    print OUTFILE "      <TR ALIGN=CENTER>\n";

    for ($y = $UserPrefs{"dayshistory"}; $y > 1; $y--) 
    {
	($year, $month, $mday, $hour, $min, $sec, $wday) = &nice_gmtime($SystemPrefs{"timebegin"} - $y * (24*60*60));
	print OUTFILE "       <TD CLASS=section>" . $SystemPrefs{"contestheaderblocktype"}[$Contest] . "s<BR>$Days[$wday]</TD>\n";
    }
    if ($UserPrefs{"dayshistory"} > 0) 
    {
	print OUTFILE "       <TD CLASS=section>" . $SystemPrefs{"contestheaderblocktype"}[$Contest] . "s<BR>Yesterday</TD>\n",
    }
    if ($UserPrefs{"dayshistory"} >= 0) 
    {
	print OUTFILE "       <TD CLASS=section>" . $SystemPrefs{"contestheaderblocktype"}[$Contest] . "s<BR>Today</TD>\n";
    }
    print OUTFILE "       <TD CLASS=section>" . $SystemPrefs{"contestheaderblocktype"}[$Contest] . "s<BR>Total</TD>\n",
                  "      </TR>\n";
    
    print OUTFILE "      <TR ALIGN=CENTER>\n";

    for ($y = $UserPrefs{"dayshistory"}; $y >= 0; $y--) 
    {
	($year, $month, $mday, $hour, $min, $sec, $wday) = &nice_gmtime($SystemPrefs{"timebegin"} - ($y * (24*60*60)));
#SELECT SUM(WU_SIZE) FROM `TBL_WU_CSC` WHERE DATE_FORMAT(SUBMITTED,"%Y-%m-%d") = "1999-07-11"
        $sql = "SELECT IFNULL(SUM(WU_SIZE),0) FROM " . $SystemPrefs{"contestwutable"}[$Contest] . 
		" WHERE ((SUBMITTED_DATE = \"$year-$month-$mday\") AND (PARTICIPANT_ID = \"$email\"))";
        my ($blocksthatday) = &DLookup($sql);

	$sql = "SELECT PARTICIPANT_ID FROM TBL_PARTICIPANT WHERE RETIRE_TO_ID = \"$email\"";
	$sth = &SelectSQL($sql);
	while (@row = $sth->fetchrow_array()) 
	{
	    $sql2 = "SELECT IFNULL(SUM(WU_SIZE),\"0\") FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE ((PARTICIPANT_ID = \"$row[0]\") AND (SUBMITTED_DATE = \"$year-$month-$mday\"))";
	    my ($additionalblocksthatday) = &DLookup($sql2);
	    $blocksthatday+= $additionalblocksthatday;
	}
	if ($y > 0)
	{ 
	 $rate[$y] = &ratef(&rate_day($blocksthatday, $SystemPrefs{"contestfactor"}[$Contest], $Contest));
	}
	else
	{
	 $rate[$y] = &ratef(&rate_day($blocksthatday / ($SystemPrefs{"secstoday"} / (60*60*24)), $SystemPrefs{"contestfactor"}[$Contest], $Contest));
	}
	if ($blocksthatday ==0) 
	{
	    $blocksthatday = "-";
	}
	if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
    	{
	    print OUTFILE "       <TD>$blocksthatday</TD>\n";
	}
	else
	{
	    if ($blocksthatday > 0)
	    {
		print OUTFILE "       <TD>" . &ratef($blocksthatday * $SystemPrefs{"contestfactor"}[$Contest]) . "</TD>\n";
	    }
	    else
	    {
		print OUTFILE "       <TD>$blocksthatday</TD>\n";
	    }
	}
    }
    if ($blocks > 0) 
    {
	$tempblocks = $blocks;
    }
    else
    {
	$tempblocks = "-";
    }
    if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
    {
	print OUTFILE "       <TD>$tempblocks</TD>\n";
    }
    else
    {
	print OUTFILE "       <TD>" . &ratef($tempblocks * $SystemPrefs{"contestfactor"}[$Contest]) . "</TD>\n";
    }
    print OUTFILE "      </TR>\n";
    print OUTFILE "      <TR ALIGN=CENTER>\n";

    # Print out day history list for all days before today
    for ($y = $UserPrefs{"dayshistory"}; $y >= 0; $y--)
    {
	print OUTFILE "       <TD NOWRAP>$rate[$y]" . $SystemPrefs{"contestblockmeasure"}[$Contest] . "/sec</TD>\n";
    }

    $sql = "SELECT UNIX_TIMESTAMP(MAX(SUBMITTED_DATE)) FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE (PARTICIPANT_ID = \"$email\")";
    ($LastSubmitted) = &DLookup($sql);
    $sql = "SELECT UNIX_TIMESTAMP(MIN(SUBMITTED_DATE)) FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE (PARTICIPANT_ID = \"$email\")";
    ($FirstSubmitted) =&DLookup($sql);    

    $sql = "SELECT PARTICIPANT_ID FROM TBL_PARTICIPANT WHERE (RETIRE_TO_ID = \"$email\")";
    $sth = &SelectSQL($sql);
    while (@row = $sth->fetchrow_array()) 
    {
    	$sql2 = "SELECT UNIX_TIMESTAMP(MAX(SUBMITTED_DATE)) FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE (PARTICIPANT_ID = \"" . $row[0] . "\")";
	my ($LastSubmittedTemp) = &DLookup($sql2);
	if ($LastSubmittedTemp > $LastSubmitted)
	{
	    $LastSubmitted = $LastSubmittedTemp;
	}
    	$sql2 = "SELECT IFNULL(UNIX_TIMESTAMP(MIN(SUBMITTED_DATE)),\"-\") FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE (PARTICIPANT_ID = \"" . $row[0] . "\")";
	my ($FirstSubmittedTemp) = &DLookup($sql2);
	if (($FirstSubmittedTemp < $FirstSubmitted) && ($FirstSubmittedTemp ne "-"))
	{
	    $FirstSubmitted = $FirstSubmittedTemp;
        }
    }

    my $totaldays = int((($LastSubmitted - $FirstSubmitted) / (60*60*24)) + 1);
    my $aveblocks = int($blocks / $totaldays);
    my $averate = &rate_day($blocks / $totaldays, $SystemPrefs{"contestfactor"}[$Contest], $Contest);
    my $averatef = &ratef($averate);

    # Print out total for day history list
    print OUTFILE "       <TD NOWRAP>$averatef" . $SystemPrefs{"contestblockmeasure"}[$Contest] . "/sec</TD>\n";
    print OUTFILE "      </TR>\n";


    $colsp = 2 + $UserPrefs{"dayshistory"};

    print OUTFILE "      <TR>\n",
                  "       <TD COLSPAN=$colsp>\n",
                  "        &nbsp;\n",
                  "       </TD>\n",
                  "      </TR>\n";

    print OUTFILE "      <TR ALIGN=CENTER>\n",
	          "       <TD CLASS=title NOWRAP COLSPAN=$colsp>\n",
                  "        <STRONG>Participant $realemail</STRONG><BR>\n",
                  "        Graphical Participant History\n",
                  "       </TD>\n",
                  "      </TR>\n";

    print OUTFILE "      <TR><TD NOWRAP COLSPAN=$colsp ALIGN=CENTER>\n",
                  "       <BR>Total Days: $totaldays days - ";
    if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
    {
	print OUTFILE "Average " . $SystemPrefs{"contestheaderblocktype"}[$Contest] . "s: $aveblocks " . $SystemPrefs{"contestblocktype"}[$Contest] . "s - ";
    }
    else
    {
	print OUTFILE "Average " . $SystemPrefs{"contestheaderblocktype"}[$Contest] . "s: " .
	              &ratef($aveblocks * $SystemPrefs{"contestfactor"}[$Contest]) . 
		      $SystemPrefs{"contestblocktype"}[$Contest] . "s - ";
    }
    print OUTFILE "Average Rate: $averatef" . $SystemPrefs{"contestblockmeasure"}[$Contest] . "s/sec\n",
                  "      </TD></TR>\n";

    print OUTFILE "      <TR>\n",
                  "       <TD COLSPAN=$colsp>\n",
                  "        &nbsp;\n",
                  "       </TD>\n",
                  "      </TR>\n";

    print OUTFILE "      <TR>\n",
	          "       <TD COLSPAN=$colsp ALIGN=CENTER>\n",
                  "        <IMG SRC=\"" . $UserPrefs{"htmlroot"} . $SystemPrefs{"contestsuffix"}[$Contest] . "/" . "$emaildir/" . $UserPrefs{"gnuplotoutput"} . "\" WIDTH=" . $UserPrefs{"chartwidth"} . " HEIGHT=" . $UserPrefs{"chartheight"} . " ALT=\"$realemail\">\n",
                  "       </TD>\n",
                  "      </TR>\n";

    print OUTFILE "      <TR>\n",
                  "       <TD COLSPAN=$colsp>\n",
                  "        &nbsp;\n",
                  "       </TD>\n",
                  "      </TR>\n";

    print OUTFILE "      <TR><TD CLASS=note NOWRAP COLSPAN=$colsp ALIGN=CENTER>\n",
                  "       For past days: " . $SystemPrefs{"contestblockmeasure"}[$Contest] . "s/sec = " . 
		  $SystemPrefs{"contestblockweight"}[$Contest] . " * " . 
		  $SystemPrefs{"contestblocktype"}[$Contest] . "s / " . $SystemPrefs{"contestdivide"}[$Contest] . "<BR>\n",
                  "       For today: " . $SystemPrefs{"contestblockmeasure"}[$Contest] . "s/sec = (" . 
		  $SystemPrefs{"contestblockweight"}[$Contest] . " * " . 
		  $SystemPrefs{"contestblocktype"}[$Contest] . "s / " . $SystemPrefs{"contestdivide"}[$Contest] . ") / (seconds today / (60*60*24))\n",
                  "      </TD></TR>\n";

    print OUTFILE "      <TR>\n",
                  "       <TD COLSPAN=$colsp>\n",
                  "        &nbsp;\n",
                  "       </TD>\n",
                  "      </TR>\n";

    print OUTFILE "      <TR>\n",
                  "       <TD NOWRAP COLSPAN=$colsp ALIGN=CENTER>\n",
                  "        Click here to goto the <A HREF=\"" . $UserPrefs{"htmlroot"} . $SystemPrefs{"contestsuffix"}[$Contest] . "/" . "$emaildir/" . $UserPrefs{"htmlindvemail"} . "\">Executive Participant Summary</A> page.\n",
                  "       </TD>\n",
                  "      </TR>\n";

    if ($UserPrefs{"showbyhost"}) {
	print OUTFILE "      <TR>\n",
                      "       <TD COLSPAN=$colsp>\n",
                      "        &nbsp;\n",
                      "       </TD>\n",
                      "      </TR>\n";

	print OUTFILE "      <TR>\n",
                      "       <TD NOWRAP COLSPAN=$colsp ALIGN=CENTER>\n",
                      "        Click here to goto the <A HREF=\"" . $UserPrefs{"htmlroot"} . $SystemPrefs{"contestsuffix"}[$Contest] . "/" . "$emaildir/" . $UserPrefs{"htmlhostsbyemail"} . "\">Hostnames working for this Participant</A> page.\n",
                      "       </TD>\n",
                      "      </TR>\n";
    }

    print OUTFILE "     </TABLE>\n";
    &write_footer();
    close OUTFILE;

    open(GNUPLOTDATA, ">" . $UserPrefs{"htmldir"} . $SystemPrefs{"contestsuffix"}[$Contest] . "/$emaildir/" . $SystemPrefs{"gnuplotdata"}) or die "Personal Proxy Statistics : Cannot open " . $UserPrefs{"htmldir"} . $SystemPrefs{"contestsuffix"}[$Contest] . "/$emaildir/" . $SystemPrefs{"gnuplotdata"} . " ($!)\n";

# Write out data file in format "Date, Rate".
##    &clear_nday_ave();

    $sql = "SELECT FROM_UNIXTIME($FirstSubmitted,\"%Y-%m-%d\")";
    my ($Submitted) = &DLookup($sql); 
    my $WorkUnits = 0;
    my $WorkUnitsTemp;
    my $line;
    my ($rowyear, $rowmonth, $rowday);
    my $subyear = undef;
    my $submonth = undef;
    my $subday = undef;
    my ($yeartemp, $monthtemp, $daytemp);
    my ($datetemp, $timetemp);
    my $averagerate;
    
    &clear_average;
    
#SELECT SUBMITTED_DATE,SUM(WU_SIZE) FROM `TBL_WU_RC564` WHERE (PARTICIPANT_ID = "1") GROUP BY 1 ORDER BY 1 ASC
    $sql = "SELECT SUBMITTED_DATE, SUM(WU_SIZE) FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE (PARTICIPANT_ID = \"$email\") GROUP BY 1 ORDER BY 1 ASC";
    $sth = &SelectSQL($sql);
    while (@row = $sth->fetchrow_array()) 
    {
        ($Submitted) = $row[0];
	($WorkUnits) = $row[1];
	($rowyear, $rowmonth, $rowday) = split(/-/, $row[0]);

	$sql = "SELECT PARTICIPANT_ID FROM TBL_PARTICIPANT WHERE RETIRE_TO_ID = \"$email\"";
	$sth2 = &SelectSQL($sql);
	while (@row2 = $sth2->fetchrow_array()) 
	{
	    $sql2 = "SELECT IFNULL(SUM(WU_SIZE),\"0\") FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE ((PARTICIPANT_ID = \"$row2[0]\") AND (SUBMITTED_DATE = \"$rowyear-$rowmonth-$rowday\"))";
    	    my ($additionalblocksthatday) = &DLookup($sql2);
	    $WorkUnits+= $additionalblocksthatday;
	}

	# Check if days are missing between writen line and new started day
	if (defined($subyear)) 
	{
	    $sql = "SELECT DATE_FORMAT(DATE_ADD(\"$subyear-$submonth-$subday 00:00:00\", INTERVAL 1 DAY), \"%Y-%m-%d\")";
    	    ($datetemp) = &DLookup($sql);
	    ($yeartemp, $monthtemp, $daytemp) = split(/-/, $datetemp);
	}
	else
	{
#	    $sql2 = "SELECT COUNT(*) FROM TBL_PARTICIPANT WHERE RETIRE_TO_ID = \"$email\"";
#	    my $retired = &SelectSQL($sql2);
#	    if ($retired > 0)
#	    {
#		$sql="SELECT UNIX_TIMESTAMP(\"$Submitted 00:00:00\")";
#    		($FirstSubmittedTest) = &DLookup($sql);
#	        if ($FirstSubmitted != $FirstSubmittedTest)
#		{
#		    $sql="SELECT DATE_FORMAT(FROM_UNIXTIME(\"$FirstSubmitted\"), \"%Y-%m-%d\")";
#		    ($FirstSubmittedTest) = &DLookup($sql);
#		    ($yeartemp, $monthtemp, $daytemp) = split(/-/, $FirstSubmittedTest);
#		}
#		else
#		{
#		    my ($hour, $min, $sec, $wday);
#		    ($yeartemp, $monthtemp, $daytemp, $hour, $min, $sec, $wday) = &nice_gmtime($FirstSubmitted);		
#		}
#		($subyear, $submonth, $subday) = split(/-/, $row[0]);
#	    }
#	    else
#	    {
		($subyear, $submonth, $subday) = split(/-/, $row[0]);
		($yeartemp, $monthtemp, $daytemp) = split(/-/, $row[0]);
#	    }
	}
	while (($yeartemp != $rowyear) || ($monthtemp != $rowmonth) || ($daytemp != $rowday))
	{
	    $WorkUnitsTemp = 0;

	    $sql = "SELECT PARTICIPANT_ID FROM TBL_PARTICIPANT WHERE RETIRE_TO_ID = \"$email\"";
	    $sth2 = &SelectSQL($sql);
	    while (@row2 = $sth2->fetchrow_array()) 
	    {
		$sql2 = "SELECT IFNULL(SUM(WU_SIZE),\"0\") FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE ((PARTICIPANT_ID = \"$row2[0]\") AND (SUBMITTED_DATE = \"$yeartemp-$monthtemp-$daytemp\"))";
    		my ($additionalblocksthatday) = &DLookup($sql2);
		$WorkUnitsTemp+= $additionalblocksthatday;
	    }

	    $averagerate = &calculate_average($WorkUnits, 2, $Contest);
	    $line = "$yeartemp/$monthtemp/$daytemp " . sprintf("%.2f", &rate_day($WorkUnitsTemp, $SystemPrefs{"contestfactor"}[$Contest], $Contest)) . " $averagerate";
	    print GNUPLOTDATA "$line\n";
    	    $sql = "SELECT DATE_FORMAT(DATE_ADD(\"$yeartemp-$monthtemp-$daytemp 00:00:00\", INTERVAL 1 DAY), \"%Y-%m-%d\")";
    	    ($datetemp) = &DLookup($sql);
	    ($yeartemp, $monthtemp, $daytemp) = split(/-/, $datetemp);	     
	}
	$averagerate = &calculate_average($WorkUnits, 2, $Contest);
        if ($Submitted eq "$GMTTS[0]-$GMTTS[1]-$GMTTS[2]")
	{
    	    $line = "$rowyear/$rowmonth/$rowday " . sprintf("%.2f", &rate_day($WorkUnits  / ($SystemPrefs{"secstoday"} / (60*60*24)), $SystemPrefs{"contestfactor"}[$Contest], $Contest)) . " $averagerate";
	}
	else
	{
	    $line = "$rowyear/$rowmonth/$rowday " . sprintf("%.2f", &rate_day($WorkUnits, $SystemPrefs{"contestfactor"}[$Contest], $Contest)) . " $averagerate";
	}
	print GNUPLOTDATA "$line\n";
	$subyear = $rowyear;
	$submonth = $rowmonth;
	$subday = $rowday;
    }

    if ($Submitted ne "$GMTTS[0]-$GMTTS[1]-$GMTTS[2]")
    {
	$rowyear = $GMTTS[0];
	$rowmonth = $GMTTS[1];
	$rowday = $GMTTS[2];
	while (($yeartemp != $rowyear) || ($monthtemp != $rowmonth) || ($daytemp != $rowday))
	{
    	    $sql = "SELECT DATE_FORMAT(DATE_ADD(\"$yeartemp-$monthtemp-$daytemp 00:00:00\", INTERVAL 1 DAY), \"%Y-%m-%d\")";
    	    ($datetemp) = &DLookup($sql);
	    ($yeartemp, $monthtemp, $daytemp) = split(/-/, $datetemp);	     
	    $WorkUnitsTemp = 0;

	    $sql = "SELECT PARTICIPANT_ID FROM TBL_PARTICIPANT WHERE RETIRE_TO_ID = \"$email\"";
	    $sth2 = &SelectSQL($sql);
	    while (@row2 = $sth2->fetchrow_array()) 
	    {
		$sql2 = "SELECT IFNULL(SUM(WU_SIZE),\"0\") FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE ((PARTICIPANT_ID = \"$row2[0]\") AND (SUBMITTED_DATE = \"$yeartemp-$monthtemp-$daytemp\"))";
    		my ($additionalblocksthatday) = &DLookup($sql2);
		$WorkUnitsTemp+= $additionalblocksthatday;
	    }

	    $averagerate = &calculate_average($WorkUnits, 2, $Contest);
	    $line = "$yeartemp/$monthtemp/$daytemp " . sprintf("%.2f", &rate_day($WorkUnitsTemp, $SystemPrefs{"contestfactor"}[$Contest], $Contest)) . " $averagerate";
	    print GNUPLOTDATA "$line\n";
	}
    }

    close GNUPLOTDATA;
    
    my $range = "*";
#    if ($blocksoverall == 0) { $range = 1; };
    $sql = "SELECT DATE_FORMAT(MIN(SUBMITTED_DATE), \"%Y/%m/%d\") FROM " . $SystemPrefs{"contestwutable"}[$Contest];
    ($FirstSubmitted) =&DLookup($sql);
    
    &plot_gui_graph($UserPrefs{"htmldir"} . $SystemPrefs{"contestsuffix"}[$Contest] . "/$emaildir/", "History Plot for " . $SystemPrefs{"contestname"}[$Contest] . "\\nParticipant $realemail", $FirstSubmitted, "$GMTTS[0]/$GMTTS[1]/$GMTTS[2]", $range, $averate, $Contest);

}  # write_emailchart



#--- Method Prolog -----------------------------------------------------------
#  Name: write_hostsbyemail
#
#  Parameters: 
#      email - emailid from database
#      realemail - emailaddress (string) from database
#      emaildir - ("e".$email)
#      blocksthatemail - blocks/workunits overall for this emailaddress 
#      haveretired - this emailaddress is one where others have retired TO 
#      Contest - The contest to collect data for. 1=RC5, 2=DES, etc.
#
#  Return Value: None.
#
#  Description:
#    Routine to write all hostnames being used by a give email address.
#
#--- End Method Prolog -------------------------------------------------------
sub write_hostsbyemail
{
    my $email = shift @_;
    my $realemail = shift @_;
    my $emaildir = shift @_;
    my $blocksthatemail = shift @_;
    my $Contest = shift @_;
    my $count = 0;
    my $tempblocks;
    my ($sql, $sql2, $sql3, $sth, $sth2);
    my ($LastSubmitted, $FirstSubmitted);
    my ($x, $y);
    my $i;
    my @hostblocks;
    my @hostid;
    my @retired;
    my (@row, @row2);
    my ($percentofemail, $keyschecked, $percentcomplete);

#    if ($haveretired > 0)
#    {	
#	$x=0;
#	$sql = "SELECT PARTICIPANT_ID FROM TBL_PARTICIPANT WHERE (RETIRE_TO_ID = \"$email\")";
#	($sth) = &SelectSQL($sql);
#	while (@row = $sth->fetchrow_array()) 
#	{
#	    $x++;
#	    $retired[$haveretired] = $row[0]
#	}
#    }

    open(OUTFILE, ">" . $UserPrefs{"htmldir"} . $SystemPrefs{"contestsuffix"}[$Contest] . "/$emaildir/" . $UserPrefs{"htmlhostsbyemail"}) or die "Personal Proxy Statistics : Cannot open " . $UserPrefs{"htmldir"} . $SystemPrefs{"contestsuffix"}[$Contest] . "/$emaildir/" . $UserPrefs{"htmlhostsbyemail "} ." ($!)\n";
    &write_header($Contest);
    print OUTFILE "     <TABLE BORDER=0 CELLPADDING=0 CELLSPACING=0 WIDTH=\"100%\">\n";
    print OUTFILE "      <TR ALIGN=CENTER>\n",
	          "       <TD CLASS=title NOWRAP>\n",
                  "        <STRONG>Participant $realemail</STRONG><BR>\n",
                  "        Hostnames working for this Participant\n",
                  "       </TD>\n",
                  "      </TR>\n";

    $sql = "SELECT UNIX_TIMESTAMP(MAX(SUBMITTED_DATE)) FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE (PARTICIPANT_ID = \"$email\")";
    ($LastSubmitted) = &DLookup($sql);
    $sql = "SELECT UNIX_TIMESTAMP(MIN(SUBMITTED_DATE)) FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE (PARTICIPANT_ID = \"$email\")";
    ($FirstSubmitted) =&DLookup($sql);    

    $sql = "SELECT PARTICIPANT_ID FROM TBL_PARTICIPANT WHERE (RETIRE_TO_ID = \"$email\")";
    $sth = &SelectSQL($sql);
    while (@row = $sth->fetchrow_array()) 
    {
        $sql2 = "SELECT UNIX_TIMESTAMP(MAX(SUBMITTED_DATE)) FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE (PARTICIPANT_ID = \"" . $row[0] . "\")";
	my ($LastSubmittedTemp) = &DLookup($sql2);
	if ($LastSubmittedTemp > $LastSubmitted)
	{
	    $LastSubmitted = $LastSubmittedTemp;
	}
    	$sql2 = "SELECT IFNULL(UNIX_TIMESTAMP(MIN(SUBMITTED_DATE)),\"-\") FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE (PARTICIPANT_ID = \"" . $row[0] . "\")";
	my ($FirstSubmittedTemp) = &DLookup($sql2);
	if (($FirstSubmittedTemp < $FirstSubmitted) && ($FirstSubmittedTemp ne "-"))
	{
	    $FirstSubmitted = $FirstSubmittedTemp;
	}
    }
    	
    my $totaldays = int((($LastSubmitted - $FirstSubmitted) / (60*60*24)) + 1);
    my $aveblocks = int($blocksthatemail / $totaldays);
    my $averate = &rate_day($blocksthatemail / $totaldays, $SystemPrefs{"contestfactor"}[$Contest], $Contest);
    my $averatef = &ratef($averate);

    print OUTFILE "      <TR><TD NOWRAP ALIGN=CENTER>\n",
                  "       <BR>Total Days: $totaldays days - ";
    if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
    {
	print OUTFILE "Average " . $SystemPrefs{"contestheaderblocktype"}[$Contest] . "s: $aveblocks " . $SystemPrefs{"contestblocktype"}[$Contest] . "s - ";
    }
    else
    {
	print OUTFILE "Average " . $SystemPrefs{"contestheaderblocktype"}[$Contest] . "s: " .
	              &ratef($aveblocks * $SystemPrefs{"contestfactor"}[$Contest]) . 
		      $SystemPrefs{"contestblocktype"}[$Contest] . "s - ";
    }
    print OUTFILE "Average Rate: $averatef" . $SystemPrefs{"contestheaderblockmeasure"}[$Contest] . "s/sec\n",
                  "      </TD></TR>\n";

    print OUTFILE "      <TR>\n",
                  "       <TD>\n",
                  "        &nbsp;\n",
                  "       </TD>\n",
                  "      </TR>\n";

    print OUTFILE "      <TR ALIGN=CENTER>\n",
                  "       <TD VALIGN=TOP WIDTH=\"100%\">\n",
                  "        <TABLE BORDER=0 CELLPADDING=2 CELLSPACING=0>\n";

    print OUTFILE "         <TR ALIGN=CENTER>\n",
                  "          <TD CLASS=section NOWRAP>\n",
                  "           Hostname\n",
                  "          </TD>\n",
                  "          <TD CLASS=section NOWRAP>\n",
                  "           " . $SystemPrefs{"contestheaderblocktype"}[$Contest] . "s Submited\n",
                  "          </TD>\n",
                  "          <TD CLASS=section NOWRAP>\n",
                  "           Percent\n",
                  "          </TD>\n";
    if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
    {
        print OUTFILE "          <TD CLASS=section NOWRAP>\n",
                  "           " . $SystemPrefs{"contestheaderblockmeasure"}[$Contest] . "s Checked\n",
                  "          </TD>\n",
                  "          <TD CLASS=section NOWRAP>\n",
                  "           Keyspace\n",
                  "          </TD>\n";
    }
    print OUTFILE "         </TR>\n";

    $sql = "SELECT HOST_ID FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE (PARTICIPANT_ID = \"$email\") GROUP BY 1 ORDER BY 1 DESC";
    $sth = &SelectSQL($sql);
    $count = 0;
    while (@row = $sth->fetchrow_array()) 
    {
	$count++;
	my $sql2 = "SELECT IFNULL(SUM(WU_SIZE),0) FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE ((PARTICIPANT_ID = \"" . $email . "\") AND (HOST_ID = \"" . $row[0] ."\"))";
	($tempblocks) = &DLookup($sql2);
        $hostblocks[$count] += $tempblocks;
	$hostid[$count] = $row[0];
    }

    $sql = "SELECT PARTICIPANT_ID FROM TBL_PARTICIPANT WHERE RETIRE_TO_ID = \"$email\"";
    $sth = &SelectSQL($sql);
    while (@row = $sth->fetchrow_array()) 
    {
        $sql2 = "SELECT HOST_ID FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE (PARTICIPANT_ID = \"" . $row[0] . "\") GROUP BY 1 ORDER BY 1 DESC";
        $sth2 = &SelectSQL($sql2);
        while (@row2 = $sth2->fetchrow_array()) 
        {	
    	    $sql3 = "SELECT IFNULL(SUM(WU_SIZE),0) FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE ((PARTICIPANT_ID = \"" . $row[0] . "\") AND (HOST_ID = \"" . $row2[0] ."\"))";
	    ($tempblocks) = &DLookup($sql3);

	    my $done = 0;
	    for ($i = 1; $i <= $count ; $i++) 
	    {
		if ($row2[0] == $hostid[$i])
		{
    		    $hostblocks[$i] += $tempblocks;
		    $done = 1;
		}
	    }
	    if ($done != 1)
	    {
		$count++;
		$hostblocks[$count] += $tempblocks;
		$hostid[$count] = $row2[0];
	    }
	}
    }

# Need to sort hosts (@hostsblocks @hostsid) by workunits
    
    for ($i = 1; $i <= $count ; $i++) 
    {
	if ($i % 2 == 0) {
	    print OUTFILE "         <TR CLASS=even>\n";
	} else {
	    print OUTFILE "         <TR CLASS=odd>\n";
	}

	my $domain = undef;
	$sql = "SELECT HOSTNAME FROM TBL_HOST WHERE (HOST_ID = \"" . $hostid[$i] . "\")";
	my ($hostname) = &DLookup($sql);
	my ($head, $tail) = split /\./, $hostname, 2;
	my $hostdir = "h" . $hostid[$i];
	if ($hostname =~ /\.(\w+)$/) 
	{ 
	    $domain = $1; 
	}
	my $icon = undef;
	my $label = undef;
	if (defined($domain)) 
	{
	    $sql = "SELECT COUNTRY FROM TBL_COUNTRY WHERE (COUNTRY_ID = \"$domain\") LIMIT 1";
	    ($label) = &DLookup($sql);
	    if (defined($label))
	    {
	        $sql = "SELECT COUNTRY_ICON FROM TBL_COUNTRY WHERE (COUNTRY_ID = \"$domain\") LIMIT 1";
		($icon) = &DLookup($sql);
		$icon = ($icon . "." . $UserPrefs{"imageext"});
	    }
	    else
	    {
    		$label = "Unknown";
		$icon = ("unknown." . $UserPrefs{"imageext"});
	    }
	}
	else
	{
	    $label = "Unknown";
	    $icon = ("unknown." . $UserPrefs{"imageext"});
	}

	if ($blocksthatemail > 0)
	{
	    $percentofemail = sprintf("%.2f", 100 * $hostblocks[$i] / $blocksthatemail);
	}
	else
	{
	    $percentofemail = sprintf("%.2f", 0);
	}
	
	if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
	{
	    $keyschecked = &keysf($hostblocks[$i], $Contest);
	    $percentcomplete = &percentf($hostblocks[$i], 6, $Contest);
	}
	
#	# Lookup host position
#	$pos = 1;
#	foreach $lookup (sort { $hosts{$b}[0] <=> $hosts{$a}[0] } keys %hosts) {
#	    if ($lookup eq $host) { last; }
#	    $pos += 1;
#	}

	my $hostlink;
	if (defined($tail)) { 
	    $hostlink = "<A HREF=\"" . $UserPrefs{"htmlroot"} . $SystemPrefs{"contestsuffix"}[$Contest] . "/" . "$hostdir/" . $UserPrefs{"htmlindvhost"} . "\">$head</A>.$tail";
	} else {
	    $hostlink = "<A HREF=\"" . $UserPrefs{"htmlroot"} . $SystemPrefs{"contestsuffix"}[$Contest] . "/" . "$hostdir/" . $UserPrefs{"htmlindvhost"} . "\">$head</A>";
	}

	print OUTFILE "          <TD NOWRAP ALIGN=LEFT>\n",
                      "           <IMG SRC=\"" . $UserPrefs{"htmlroot"} . $UserPrefs{"icondir"} . "flags/$icon\" ALIGN=BOTTOM BORDER=0 WIDTH=14 HEIGHT=14 ALT=\"$label\">&nbsp;$hostlink\n",
                      "          </TD>\n",
                      "          <TD NOWRAP ALIGN=RIGHT>\n";
	if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
	{
            print OUTFILE "           " . $hostblocks[$i] . "\n";
	}
	else
	{
            print OUTFILE "           " . &ratef($hostblocks[$i] * $SystemPrefs{"contestfactor"}[$Contest]) . "\n";
	}
        print OUTFILE "          </TD>\n",
                      "          <TD NOWRAP ALIGN=RIGHT>\n",
                      "           $percentofemail%\n",
                      "          </TD>\n";
	if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
	{
            print OUTFILE "          <TD NOWRAP ALIGN=RIGHT>\n",
                      "           $keyschecked\n",
                      "          </TD>\n",
                      "          <TD NOWRAP ALIGN=RIGHT>\n",
                      "           $percentcomplete%\n",
                      "          </TD>\n";
	}
	print OUTFILE "         </TR>\n";
    }

    print OUTFILE "        </TABLE>\n",
                  "       </TD>\n",
                  "      </TR>\n";


    print OUTFILE "      <TR>\n",
                  "       <TD>\n",
                  "        &nbsp;\n",
                  "       </TD>\n",
                  "      </TR>\n";

    print OUTFILE "      <TR>\n",
                  "       <TD NOWRAP ALIGN=CENTER>\n",
                  "        Click here to goto the <A HREF=\"" . $UserPrefs{"htmlroot"} . $SystemPrefs{"contestsuffix"}[$Contest] . "/" . "$emaildir/" . $UserPrefs{"htmlindvemail"} . "\">Executive Participant Summary</A> page.\n",
                  "       </TD>\n",
                  "      </TR>\n";

    print OUTFILE "      <TR>\n",
                  "       <TD>\n",
                  "        &nbsp;\n",
                  "       </TD>\n",
                  "      </TR>\n";

    print OUTFILE "      <TR>\n",
                  "       <TD NOWRAP ALIGN=CENTER>\n",
                  "        Click here to goto the <A HREF=\"" . $UserPrefs{"htmlroot"} . $SystemPrefs{"contestsuffix"}[$Contest] . "/" . "$emaildir/" . $UserPrefs{"htmlemailhist"} . "\">Graphical Participant History</A> page.\n",
                  "       </TD>\n",
                  "      </TR>\n";

    print OUTFILE "     </TABLE>\n";
    &write_footer();
    close OUTFILE;
}  #  write_hostsbyemail



#--- Method Prolog -----------------------------------------------------------
#  Name: write_byos
#
#  Parameters: 
#      Contest - The contest to collect data for. 1=RC5, 2=DES, etc.
#
#  Return Value: None.
#
#  Description:
#    Write stats ordered by OS.
#
#--- End Method Prolog -------------------------------------------------------
sub write_byos
{
    my $Contest = shift @_;
    &Debugln("Entering write_byos($Contest) ...");
    my $count = 0;
    my $pagecount = 1;
    my ($y, $z);
    my $lastseen;
    my $lastseentime;
    my $lastseencolor;
    my (@os, @blocks, @row);
    my ($sql, $sth, $numos);
    

    &write_section_header("", "Operating System", $UserPrefs{"htmlos"}, $Contest);

    for ($y = $UserPrefs{"dayshistory"}+1; $y >= 0; $y--) {
	$pageblocks[$y] = 0;
    }

    $sql = "SELECT SUM(WU_SIZE) FROM " . $SystemPrefs{"contestwutable"}[$Contest];
    my ($blocksoverall) = &DLookup($sql);

    $numos = 0;
#SELECT OS_ID,SUM(WU_SIZE) FROM `TBL_WU_RC564` GROUP BY 1 ORDER BY 2 DESC
    $sql = "SELECT OS_ID,IFNULL(SUM(WU_SIZE),\"-\") FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " GROUP BY 1 ORDER BY 2 DESC";
    $sth = &SelectSQL($sql);

    while (@row = $sth->fetchrow_array()) 
    {
	$numos++;
	$os[$numos] = $row[0];
	$blocks[$numos] = $row[1];
    }


    for ($z = 1; $z <= $numos; $z++)
    {
	my $osnow = $os[$z];
	$count += 1;

	# Start a new file if maxinum number of rows has been reached.
	if ($count > $UserPrefs{"maxrows"} * $pagecount) 
	{
	    &write_section_footer($pagecount, $numos, 0, $UserPrefs{"htmlos"}, "", $Contest, 0);

	    for ($y = $UserPrefs{"dayshistory"}+1; $y >= 0; $y--) {
		$pageblocks[$y] = 0;
	    }

	    $pagecount += 1;
	    &write_section_header($pagecount, "Operating System", $UserPrefs{"htmlos"}, $Contest);
	}

	if ($count % 2 == 0) {
	    print OUTFILE "      <TR CLASS=even ALIGN=RIGHT>\n";
	} else {
	    print OUTFILE "      <TR CLASS=odd ALIGN=RIGHT>\n";
	}


#	# Print out the Rank and Movement column.
#	$displacement = $oss_ry{$os} == 0 ? $numos - $count : $oss_ry{$os} - $count;
#	$msg = &create_move_msg($displacement);
#        print OUTFILE "       <TD NOWRAP ALIGN=LEFT>&nbsp;$count$msg</TD>\n";
	print OUTFILE "       <TD NOWRAP ALIGN=LEFT>&nbsp;$count</TD>\n";
	
        my $label = undef;
	my $icon = undef;
	$sql = "SELECT OS FROM TBL_OS WHERE (OS_ID = \"$osnow\") LIMIT 1";
	($label) = &DLookup($sql);
	if (defined($label))
	{
	    $sql = "SELECT OS_ICON FROM TBL_OS WHERE (OS_ID = \"$osnow\") LIMIT 1";
	    ($icon) = &DLookup($sql);
	    $icon = ($icon . "." . $UserPrefs{"imageext"});
	}
	else
	{
	    $label = "Unknown";
	    $icon = ("unknown." . $UserPrefs{"imageext"});
	}

	print OUTFILE "       <TD NOWRAP ALIGN=LEFT><IMG SRC=\"" . $UserPrefs{"htmlroot"} . $UserPrefs{"icondir"} . "os/$icon\" ALIGN=BOTTOM BORDER=0 WIDTH=14 HEIGHT=14 ALT=\"$label\">&nbsp;$label</TD>\n";

	for ($y = $UserPrefs{"dayshistory"} - 3; $y >= 0; $y--) 
	{
	    my ($year, $month, $day, $hour, $min, $sec, $wday) = &nice_gmtime($SystemPrefs{"timebegin"} - ($y * (24*60*60)));
	    $sql = "SELECT IFNULL(SUM(WU_SIZE),\"-\") FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE ((OS_ID = \"$osnow\") AND (SUBMITTED_DATE = \"$year-$month-$day\"))";
	    my ($blocksthatday) = &DLookup($sql);
	    
	    if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
	    {
		print OUTFILE "       <TD>$blocksthatday</TD>\n";
	    }
	    else
	    {
		if ($blocksthatday ne "-")
		{
		    print OUTFILE "       <TD>" . &ratef($blocksthatday * $SystemPrefs{"contestfactor"}[$Contest]) . "</TD>\n";
		}
		else
		{
		print OUTFILE "       <TD>$blocksthatday</TD>\n";
		}
	    }
	    
	    if ($blocksthatday ne "-")
	    {
	    	$pageblocks[$y+1] += $blocksthatday;
	    }
	}

	
        if ($blocks[$z] ne "-")
	{
	    $pageblocks[0] += $blocks[$z];
	}
	
	if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
	{
	    print OUTFILE "       <TD>" . $blocks[$z] . "</TD>\n";
	}
	else
	{
	    print OUTFILE "       <TD>" . &ratef($blocks[$z] * $SystemPrefs{"contestfactor"}[$Contest]) . "</TD>\n";
	}


# Print out the Last Seen column.
        $sql = "SELECT MAX(SUBMITTED_DATE) FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE (OS_ID = \"$osnow\")";
	my ($LastSubmittedDate) = &DLookup($sql);
        $sql = "SELECT MAX(SUBMITTED_TIME) FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE ((SUBMITTED_DATE = \"$LastSubmittedDate\") AND (OS_ID = \"$osnow\"))";
	my ($LastSubmittedTime) = &DLookup($sql);
	$sql = "SELECT UNIX_TIMESTAMP(\"$LastSubmittedDate $LastSubmittedTime\")";
	my ($LastTime) = &DLookup($sql);

	$lastseentime = ($SystemPrefs{"timebegin"}-($LastTime));
	$lastseen = &ls($lastseentime);
	$lastseencolor = &ls_color($lastseentime);

	print OUTFILE "       <TD CLASS=$lastseencolor NOWRAP>$lastseen</TD>\n";

# Print out the Percent column.

	my $percentofteam;
	if ($blocks[$z] > 0)
	{
	    $percentofteam = sprintf("%.2f", 100 * $blocks[$z] / $blocksoverall);
	}
	else
	{
	    $percentofteam = sprintf("%.2f", 0);
	}

	print OUTFILE "       <TD>$percentofteam%</TD>\n",
	              "      </TR>\n";

    }

    &write_section_footer($pagecount, $numos, 0, $UserPrefs{"htmlos"}, "", $Contest, 0);
 
    &Debugln("Exiting write_byos($Contest) ...");
}  # write_byos




#--- Method Prolog -----------------------------------------------------------
#  Name: write_bycpu
#
#  Parameters: 
#      Contest - The contest to collect data for. 1=RC5, 2=DES, etc.
#
#  Return Value: None.
#
#  Description:
#    Write stats ordered by CPU.
#
#--- End Method Prolog -------------------------------------------------------
sub write_bycpu
{
    my $Contest = shift @_;
    &Debugln("Entering write_bycpu($Contest) ...");
    my $count = 0;
    my $pagecount = 1;
    my ($y, $z);
    my $lastseen;
    my $lastseentime;
    my $lastseencolor;
    my (@cpu, @blocks, @row);
    my ($sql, $sth, $numos);
    

    &write_section_header("", "CPU Type", $UserPrefs{"htmlcpu"}, $Contest);

    for ($y = $UserPrefs{"dayshistory"}+1; $y >= 0; $y--) {
	$pageblocks[$y] = 0;
    }

    $sql = "SELECT IFNULL(SUM(WU_SIZE),\"-\") FROM " . $SystemPrefs{"contestwutable"}[$Contest];
    my ($blocksoverall) = &DLookup($sql);

    my $numcpu = 0;
#SELECT CPU_ID,SUM(WU_SIZE) FROM `TBL_WU_RC564` GROUP BY 1 ORDER BY 2 DESC
    $sql = "SELECT CPU_ID,IFNULL(SUM(WU_SIZE),\"-\") FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " GROUP BY 1 ORDER BY 2 DESC";
    $sth = &SelectSQL($sql);

    while (@row = $sth->fetchrow_array()) 
    {
	$numcpu++;
	$cpu[$numcpu] = $row[0];
	$blocks[$numcpu] = $row[1];
    }


    for ($z = 1; $z <= $numcpu; $z++)
    {
	my $cpunow = $cpu[$z];
	$count += 1;

	# Start a new file if maxinum number of rows has been reached.
	if ($count > $UserPrefs{"maxrows"} * $pagecount) 
	{
	    &write_section_footer($pagecount, $numcpu, 0, $UserPrefs{"htmlcpu"}, "", $Contest, 0);

	    for ($y = $UserPrefs{"dayshistory"}+1; $y >= 0; $y--) {
		$pageblocks[$y] = 0;
	    }

	    $pagecount += 1;
	    &write_section_header($pagecount, "CPU Type", $UserPrefs{"htmlcpu"}, $Contest);
	}

	if ($count % 2 == 0) {
	    print OUTFILE "      <TR CLASS=even ALIGN=RIGHT>\n";
	} else {
	    print OUTFILE "      <TR CLASS=odd ALIGN=RIGHT>\n";
	}


#	# Print out the Rank and Movement column.
#	$displacement = $oss_ry{$os} == 0 ? $numos - $count : $oss_ry{$os} - $count;
#	$msg = &create_move_msg($displacement);
#        print OUTFILE "       <TD NOWRAP ALIGN=LEFT>&nbsp;$count$msg</TD>\n";
	print OUTFILE "       <TD NOWRAP ALIGN=LEFT>&nbsp;$count</TD>\n";
	
        my $label = undef;
	my $icon = undef;
	$sql = "SELECT CPU FROM TBL_CPU WHERE (CPU_ID = \"$cpunow\") LIMIT 1";
	($label) = &DLookup($sql);
	if (defined($label))
	{
	    $sql = "SELECT CPU_ICON FROM TBL_CPU WHERE (CPU_ID = \"$cpunow\") LIMIT 1";
	    ($icon) = &DLookup($sql);
	    $icon = ($icon . "." . $UserPrefs{"imageext"});
	}
	else
	{
	    $label = "Unknown";
	    $icon = ("unknown." . $UserPrefs{"imageext"});
	}

	print OUTFILE "       <TD NOWRAP ALIGN=LEFT><IMG SRC=\"" . $UserPrefs{"htmlroot"} . $UserPrefs{"icondir"} . "cpu/$icon\" ALIGN=BOTTOM BORDER=0 WIDTH=14 HEIGHT=14 ALT=\"$label\">&nbsp;$label</TD>\n";

	for ($y = $UserPrefs{"dayshistory"} - 3; $y >= 0; $y--) 
	{
	    my ($year, $month, $day, $hour, $min, $sec, $wday) = &nice_gmtime($SystemPrefs{"timebegin"} - $y * (24*60*60));
	    $sql = "SELECT IFNULL(SUM(WU_SIZE),\"-\") FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE ((CPU_ID = \"$cpunow\") AND (SUBMITTED_DATE = \"$year-$month-$day\"))";
	    my ($blocksthatday) = &DLookup($sql);
	    
	    if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
	    {
		print OUTFILE "       <TD>$blocksthatday</TD>\n";
	    }
	    else
	    {
		if ($blocksthatday ne "-")
		{
		    print OUTFILE "       <TD>" . &ratef($blocksthatday * $SystemPrefs{"contestfactor"}[$Contest]) . "</TD>\n";
		}
		else
		{
		print OUTFILE "       <TD>$blocksthatday</TD>\n";
		}
	    }
	    
	    if ($blocksthatday ne "-")
	    {
	    	$pageblocks[$y+1] += $blocksthatday;
	    }
	}

	
	if ($blocks[$z] ne "-")
	{
	    $pageblocks[0] += $blocks[$z];
	}

	if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
	{
	    print OUTFILE "       <TD>" . $blocks[$z] . "</TD>\n";
	}
	else
	{
	    print OUTFILE "       <TD>" . &ratef($blocks[$z] * $SystemPrefs{"contestfactor"}[$Contest]) . "</TD>\n";
	}


# Print out the Last Seen column.
        $sql = "SELECT MAX(SUBMITTED_DATE) FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE (CPU_ID = \"$cpunow\")";
	my ($LastSubmittedDate) = &DLookup($sql);
        $sql = "SELECT MAX(SUBMITTED_TIME) FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE ((SUBMITTED_DATE = \"$LastSubmittedDate\") AND (CPU_ID = \"$cpunow\"))";
	my ($LastSubmittedTime) = &DLookup($sql);
	$sql = "SELECT UNIX_TIMESTAMP(\"$LastSubmittedDate $LastSubmittedTime\")";
	my ($LastTime) = &DLookup($sql);

	$lastseentime = ($SystemPrefs{"timebegin"}-($LastTime));
	$lastseen = &ls($lastseentime);
	$lastseencolor = &ls_color($lastseentime);

	print OUTFILE "       <TD CLASS=$lastseencolor NOWRAP>$lastseen</TD>\n";

# Print out the Percent column.

	my $percentofteam;
	if (($blocks[$z] ne "-") && ($blocksoverall ne "-"))
	{
	    $percentofteam = sprintf("%.2f", 100 * $blocks[$z] / $blocksoverall);
	}
	else
	{
	    $percentofteam = sprintf("%.2f", 0);
	}

	print OUTFILE "       <TD>$percentofteam%</TD>\n",
	              "      </TR>\n";

    }

    &write_section_footer($pagecount, $numcpu, 0, $UserPrefs{"htmlcpu"}, "", $Contest, 0);
 
    &Debugln("Exiting write_bycpu($Contest) ...");
}  # write_bycpu




#--- Method Prolog -----------------------------------------------------------
#  Name: write_byversion
#
#  Parameters: 
#      Contest - The contest to collect data for. 1=RC5, 2=DES, etc.
#
#  Return Value: None.
#
#  Description:
#    Write stats ordered by Clientversion.
#
#--- End Method Prolog -------------------------------------------------------
sub write_byversion
{
    my $Contest = shift @_;
    &Debugln("Entering write_byversion($Contest) ...");
    my $count = 0;
    my $pagecount = 1;
    my ($y, $z);
    my $lastseen;
    my $lastseentime;
    my $lastseencolor;
    my (@ver, @blocks, @row);
    my ($sql, $sth, $numos);
    

    &write_section_header("", "Client Version", $UserPrefs{"htmlver"}, $Contest);

    for ($y = $UserPrefs{"dayshistory"}+1; $y >= 0; $y--) {
	$pageblocks[$y] = 0;
    }

    $sql = "SELECT IFNULL(SUM(WU_SIZE),\"-\") FROM " . $SystemPrefs{"contestwutable"}[$Contest];
    my ($blocksoverall) = &DLookup($sql);

    my $numver = 0;
#SELECT VERSION,SUM(WU_SIZE) FROM `TBL_WU_RC564` GROUP BY 1 ORDER BY 2 DESC
    $sql = "SELECT VERSION,IFNULL(SUM(WU_SIZE),\"-\") FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " GROUP BY 1 ORDER BY 2 DESC";
    $sth = &SelectSQL($sql);

    while (@row = $sth->fetchrow_array()) 
    {
	$numver++;
	$ver[$numver] = $row[0];
	$blocks[$numver] = $row[1];
    }


    for ($z = 1; $z <= $numver; $z++)
    {
	my $version = $ver[$z];
	$count += 1;

	# Start a new file if maxinum number of rows has been reached.
	if ($count > $UserPrefs{"maxrows"} * $pagecount) 
	{
	    &write_section_footer($pagecount, $numver, 0, $UserPrefs{"htmlver"}, "", $Contest, 0);

	    for ($y = $UserPrefs{"dayshistory"}+1; $y >= 0; $y--) {
		$pageblocks[$y] = 0;
	    }

	    $pagecount += 1;
	    &write_section_header($pagecount, "Client Version", $UserPrefs{"htmlver"}, $Contest);
	}

	if ($count % 2 == 0) {
	    print OUTFILE "      <TR CLASS=even ALIGN=RIGHT>\n";
	} else {
	    print OUTFILE "      <TR CLASS=odd ALIGN=RIGHT>\n";
	}


#	# Print out the Rank and Movement column.
#	$displacement = $oss_ry{$os} == 0 ? $numos - $count : $oss_ry{$os} - $count;
#	$msg = &create_move_msg($displacement);
#        print OUTFILE "       <TD NOWRAP ALIGN=LEFT>&nbsp;$count$msg</TD>\n";
	print OUTFILE "       <TD NOWRAP ALIGN=LEFT>&nbsp;$count</TD>\n";
	
	print OUTFILE "       <TD NOWRAP ALIGN=LEFT>&nbsp;$version</TD>\n";

	for ($y = $UserPrefs{"dayshistory"} - 3; $y >= 0; $y--) 
	{
	    my ($year, $month, $day, $hour, $min, $sec, $wday) = &nice_gmtime($SystemPrefs{"timebegin"} - $y * (24*60*60));
	    $sql = "SELECT IFNULL(SUM(WU_SIZE),\"-\") FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE ((VERSION = \"$version\") AND (SUBMITTED_DATE = \"$year-$month-$day\"))";
	    my ($blocksthatday) = &DLookup($sql);
	    
	    if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
	    {
		print OUTFILE "       <TD>$blocksthatday</TD>\n";
	    }
	    else
	    {
		if ($blocksthatday ne "-")
		{
		    print OUTFILE "       <TD>" . &ratef($blocksthatday * $SystemPrefs{"contestfactor"}[$Contest]) . "</TD>\n";
		}
		else
		{
		print OUTFILE "       <TD>$blocksthatday</TD>\n";
		}
	    }
	    
	    if ($blocksthatday ne "-")
	    {
	    	$pageblocks[$y+1] += $blocksthatday;
	    }
	}

	
	if ($blocks[$z] ne "-")
	{
	    $pageblocks[0] += $blocks[$z];
	}

	if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
	{
	    print OUTFILE "       <TD>" . $blocks[$z] . "</TD>\n";
	}
	else
	{
	    print OUTFILE "       <TD>" . &ratef($blocks[$z] * $SystemPrefs{"contestfactor"}[$Contest]) . "</TD>\n";
	}


# Print out the Last Seen column.
        $sql = "SELECT MAX(SUBMITTED_DATE) FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE (VERSION = \"$version\")";
	my ($LastSubmittedDate) = &DLookup($sql);
        $sql = "SELECT MAX(SUBMITTED_TIME) FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE ((SUBMITTED_DATE = \"$LastSubmittedDate\") AND (VERSION = \"$version\"))";
	my ($LastSubmittedTime) = &DLookup($sql);
	$sql = "SELECT UNIX_TIMESTAMP(\"$LastSubmittedDate $LastSubmittedTime\")";
	my ($LastTime) = &DLookup($sql);

	$lastseentime = ($SystemPrefs{"timebegin"}-($LastTime));
	$lastseen = &ls($lastseentime);
	$lastseencolor = &ls_color($lastseentime);

	print OUTFILE "       <TD CLASS=$lastseencolor NOWRAP>$lastseen</TD>\n";

# Print out the Percent column.

	my $percentofteam;
	if (($blocks[$z] ne "-") && ($blocksoverall ne "-"))
	{
	    $percentofteam = sprintf("%.2f", 100 * $blocks[$z] / $blocksoverall);
	}
	else
	{
	    $percentofteam = sprintf("%.2f", 0);
	}

	print OUTFILE "       <TD>$percentofteam%</TD>\n",
	              "      </TR>\n";

    }

    &write_section_footer($pagecount, $numver, 0, $UserPrefs{"htmlver"}, "", $Contest, 0);
 
    &Debugln("Exiting write_byversion($Contest) ...");
}  # write_byversion






#--- Method Prolog -----------------------------------------------------------
#  Name: write_byfull
#
#  Parameters: 
#      Contest - The contest to collect data for. 1=RC5, 2=DES, etc.
#
#  Return Value: None.
#
#  Description:
#    Write stats ordered by Full detail. (OS/CPU/Version)
#
#--- End Method Prolog -------------------------------------------------------
sub write_byfull
{
    my $Contest = shift @_;
    &Debugln("Entering write_byfull($Contest) ...");
    my $count = 0;
    my $pagecount = 1;
    my ($y, $z);
    my $lastseen;
    my $lastseentime;
    my $lastseencolor;
    my (@ver, @blocks, @row);
    my (@cpu_id, @os_id);
    my ($sql, $sth, $numos);
    

    &write_section_header("", "Full Detail", $UserPrefs{"htmlfull"}, $Contest);

    for ($y = $UserPrefs{"dayshistory"}+1; $y >= 0; $y--) {
	$pageblocks[$y] = 0;
    }

    $sql = "SELECT SUM(WU_SIZE) FROM " . $SystemPrefs{"contestwutable"}[$Contest];
    my ($blocksoverall) = &DLookup($sql);

    my $num = 0;
#SELECT OS_ID,CPU_ID,VERSION,SUM(WU_SIZE) FROM `TBL_WU_RC564` GROUP BY 1,2,3 ORDER BY 4 DESC
    $sql = "SELECT OS_ID, CPU_ID, VERSION, IFNULL(SUM(WU_SIZE),\"-\") FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " GROUP BY 1,2,3 ORDER BY 4 DESC";
    $sth = &SelectSQL($sql);

    while (@row = $sth->fetchrow_array()) 
    {
	$num++;
	$os_id[$num] = $row[0];
	$cpu_id[$num] = $row[1];
	$ver[$num] = $row[2];
	$blocks[$num] = $row[3];
    }


    for ($z = 1; $z <= $num; $z++)
    {
	$count += 1;

	# Start a new file if maxinum number of rows has been reached.
	if ($count > $UserPrefs{"maxrows"} * $pagecount) 
	{
	    &write_section_footer($pagecount, $num, 0, $UserPrefs{"htmlfull"}, "", $Contest, 0);

	    for ($y = $UserPrefs{"dayshistory"}+1; $y >= 0; $y--) {
		$pageblocks[$y] = 0;
	    }

	    $pagecount += 1;
	    &write_section_header($pagecount, "Full Detail", $UserPrefs{"htmlfull"}, $Contest);
	}

	if ($count % 2 == 0) {
	    print OUTFILE "      <TR CLASS=even ALIGN=RIGHT>\n";
	} else {
	    print OUTFILE "      <TR CLASS=odd ALIGN=RIGHT>\n";
	}


#	# Print out the Rank and Movement column.
#	$displacement = $oss_ry{$os} == 0 ? $numos - $count : $oss_ry{$os} - $count;
#	$msg = &create_move_msg($displacement);
#        print OUTFILE "       <TD NOWRAP ALIGN=LEFT>&nbsp;$count$msg</TD>\n";
	print OUTFILE "       <TD NOWRAP ALIGN=LEFT>&nbsp;$count</TD>\n";

        my $oslabel = undef;
	my $osicon = undef;
	$sql = "SELECT OS FROM TBL_OS WHERE (OS_ID = \"" . $os_id[$z] . "\") LIMIT 1";
	($oslabel) = &DLookup($sql);
	if (defined($oslabel))
	{
	    $sql = "SELECT OS_ICON FROM TBL_OS WHERE (OS_ID = \"" . $os_id[$z] . "\") LIMIT 1";
	    ($osicon) = &DLookup($sql);
	    $osicon = ($osicon . "." . $UserPrefs{"imageext"});
	}
	else
	{
	    $oslabel = "Unknown";
	    $osicon = ("unknown." . $UserPrefs{"imageext"});
	}

        my $cpulabel = undef;
	my $cpuicon = undef;
	$sql = "SELECT CPU FROM TBL_CPU WHERE (CPU_ID = \"" . $cpu_id[$z] . "\") LIMIT 1";
	($cpulabel) = &DLookup($sql);
	if (defined($cpulabel))
	{
	    $sql = "SELECT CPU_ICON FROM TBL_CPU WHERE (CPU_ID = \"" . $cpu_id[$z] . "\") LIMIT 1";
	    ($cpuicon) = &DLookup($sql);
	    $cpuicon = ($cpuicon . "." . $UserPrefs{"imageext"});
	}
	else
	{
	    $cpulabel = "Unknown";
	    $cpuicon = ("unknown." . $UserPrefs{"imageext"});
	}



	print OUTFILE "       <TD NOWRAP ALIGN=LEFT><IMG SRC=\"" . $UserPrefs{"htmlroot"} . $UserPrefs{"icondir"} . "os/$osicon\" ALIGN=BOTTOM BORDER=0 WIDTH=14 HEIGHT=14 ALT=\"$oslabel\">";
	print OUTFILE "       <IMG SRC=\"" . $UserPrefs{"htmlroot"} . $UserPrefs{"icondir"} . "cpu/$cpuicon\" ALIGN=BOTTOM BORDER=0 WIDTH=14 HEIGHT=14 ALT=\"$cpulabel\">";
	
	print OUTFILE "&nbsp;$oslabel, $cpulabel, " . $ver[$z] . "</TD>\n";

	for ($y = $UserPrefs{"dayshistory"} - 3; $y >= 0; $y--) 
	{
	    my ($year, $month, $day, $hour, $min, $sec, $wday) = &nice_gmtime($SystemPrefs{"timebegin"} - $y * (24*60*60));
	    $sql = "SELECT IFNULL(SUM(WU_SIZE),\"-\") FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE " . 
		"( (OS_ID = \"" . $os_id[$z] . "\") " . 
		"AND (CPU_ID = \"" . $cpu_id[$z] . "\") " .
		"AND (VERSION = \"" . $ver[$z] . "\") " .
		"AND (SUBMITTED_DATE = \"$year-$month-$day\"))";
	    my ($blocksthatday) = &DLookup($sql);
	    
	    if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
	    {
		print OUTFILE "       <TD>$blocksthatday</TD>\n";
	    }
	    else
	    {
		if ($blocksthatday ne "-")
		{
		    print OUTFILE "       <TD>" . &ratef($blocksthatday * $SystemPrefs{"contestfactor"}[$Contest]) . "</TD>\n";
		}
		else
		{
		print OUTFILE "       <TD>$blocksthatday</TD>\n";
		}
	    }
	    
	    if ($blocksthatday ne "-")
	    {
	    	$pageblocks[$y+1] += $blocksthatday;
	    }
	}

	
	if ($blocks[$z] ne "-")
	{
	    $pageblocks[0] += $blocks[$z];
	}

	if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
	{
	    print OUTFILE "       <TD>" . $blocks[$z] . "</TD>\n";
	}
	else
	{
	    print OUTFILE "       <TD>" . &ratef($blocks[$z] * $SystemPrefs{"contestfactor"}[$Contest]) . "</TD>\n";
	}


# Print out the Last Seen column.
        $sql = "SELECT MAX(SUBMITTED_DATE) FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE " . 
	    "( (OS_ID = \"" . $os_id[$z] . "\") " . 
	    "AND (CPU_ID = \"" . $cpu_id[$z] . "\") " .
	    "AND (VERSION = \"" . $ver[$z] . "\"))";
	my ($LastSubmittedDate) = &DLookup($sql);
        $sql = "SELECT MAX(SUBMITTED_TIME) FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE " .
	    "((SUBMITTED_DATE = \"$LastSubmittedDate\") " . 
	    "AND (OS_ID = \"" . $os_id[$z] . "\") " . 
	    "AND (CPU_ID = \"" . $cpu_id[$z] . "\") " .
	    "AND (VERSION = \"" . $ver[$z] . "\"))";
	my ($LastSubmittedTime) = &DLookup($sql);
	$sql = "SELECT UNIX_TIMESTAMP(\"$LastSubmittedDate $LastSubmittedTime\")";
	my ($LastTime) = &DLookup($sql);

	$lastseentime = ($SystemPrefs{"timebegin"}-($LastTime));
	$lastseen = &ls($lastseentime);
	$lastseencolor = &ls_color($lastseentime);

	print OUTFILE "       <TD CLASS=$lastseencolor NOWRAP>$lastseen</TD>\n";

# Print out the Percent column.

	my $percentofteam;
	if (($blocks[$z] ne "-") && ($blocksoverall ne "-"))
	{
	    $percentofteam = sprintf("%.2f", 100 * $blocks[$z] / $blocksoverall);
	}
	else
	{
	    $percentofteam = sprintf("%.2f", 0);
	}

	print OUTFILE "       <TD>$percentofteam%</TD>\n",
	              "      </TR>\n";

    }

    &write_section_footer($pagecount, $num, 0, $UserPrefs{"htmlver"}, "", $Contest, 0);
 
    &Debugln("Exiting write_byfull($Contest) ...");
}  # write_byversion




#--- Method Prolog -----------------------------------------------------------
#  Name: write_bydomain_header
#
#  Parameters: 
#      Contest - The contest to collect data for. 1=RC5, 2=DES, etc.
#
#  Return Value: None.
#
#  Description:
#    Write (table-) header of HTML-files for "By Domain" section.
#
#--- End Method Prolog -------------------------------------------------------
sub write_bydomain_header
{
    my $page = shift @_;
    my $Contest = shift @_;
    my $y;
    my ($year, $month, $mday, $hour, $min, $sec, $wday);

    open(OUTFILE, ">" . $UserPrefs{"htmldir"} . $SystemPrefs{"contestsuffix"}[$Contest] . "/" . $page . $UserPrefs{"htmldomain"}) or die "Personal Proxy Statistics : Cannot open " . $UserPrefs{"htmldir"} . $SystemPrefs{"contestsuffix"}[$Contest] . "/" . $page . $UserPrefs{"htmldomain"} . " ($!)\n";
    &write_header($Contest);
    print OUTFILE "     <TABLE BORDER=0 CELLPADDING=0 CELLSPACING=0 WIDTH=\"100%\">\n";
    print OUTFILE "      <TR ALIGN=CENTER>\n",
                  "       <TD CLASS=section>Rank</TD>\n",
                  "       <TD CLASS=section NOWRAP>Email Address Domain</TD>\n";

    for ($y = $UserPrefs{"dayshistory"} - 4; $y >= 1; $y--) {
	($year, $month, $mday, $hour, $min, $sec, $wday) = &nice_gmtime($SystemPrefs{"timebegin"} - $y * (24*60*60));
	print OUTFILE "       <TD CLASS=section>" . $SystemPrefs{"contestheaderblocktype"}[$Contest] . "s<BR>$Days[$wday]</TD>\n";
    }
    if ($UserPrefs{"dayshistory"} - 3 >= 1) {
	print OUTFILE "       <TD CLASS=section>" . $SystemPrefs{"contestheaderblocktype"}[$Contest] . "s<BR>Yesterday</TD>\n";
    }
    if ($UserPrefs{"dayshistory"} - 3 >= 0) {
	print OUTFILE "       <TD CLASS=section>" . $SystemPrefs{"contestheaderblocktype"}[$Contest] . "s<BR>Today</TD>\n";
    }
    print OUTFILE "       <TD CLASS=section>" . $SystemPrefs{"contestheaderblocktype"}[$Contest] . "s<BR>Total</TD>\n",
                  "       <TD CLASS=section>Email<BR>Count</TD>\n",
                  "       <TD CLASS=section>Last<BR>Seen</TD>\n",
                  "       <TD CLASS=section>Percent</TD>\n",
                  "      </TR>\n";
}  # write_bydomain_header


#--- Method Prolog -----------------------------------------------------------
#  Name: write_bydomain
#
#  Parameters: 
#      Contest - The contest to collect data for. 1=RC5, 2=DES, etc.
#
#  Return Value: None.
#
#  Description:
#    Write stats ordered by Email Address Domain.
#
#--- End Method Prolog -------------------------------------------------------
sub write_bydomain
{
    my $Contest = shift @_;
    &Debugln("Entering write_bydomain($Contest) ...");

    my $count = 0;
    my $pagecount = 1;
    my ($x, $y, $z);
    my $lastseen;
    my $lastseentime;
    my $lastseencolor;
    my $numdomain;
    my (@domain, @emails);
    my $sql;
    my ($label, $icon);
    my $blocksthatday;
    my ($year, $month, $day, $hour, $min, $sec, $wday);
    my $blocksthatdomain;
    my $blocks;
    my ($LastSubmittedDate, $LastSubmittedTime, $LastTime);

    &write_bydomain_header("", $Contest);

    for ($y = $UserPrefs{"dayshistory"}+1; $y >= 0; $y--) 
    {
	$pageblocks[$y] = 0;
    }

    $sql = "SELECT SUM(WU_SIZE) FROM " . $SystemPrefs{"contestwutable"}[$Contest];
    my ($blocksoverall) = &DLookup($sql);

    $sql = "SELECT MAX(COUNTRY_ID) FROM TBL_PARTICIPANT";
    my ($lastdomain) =&DLookup($sql);
    $numdomain = 0;
    $y = 0;
    while ($y ne $lastdomain) 
    {
	$numdomain++;
	$sql = "SELECT MIN(COUNTRY_ID) FROM TBL_PARTICIPANT WHERE (COUNTRY_ID > \"$y\")";
    	($y) =&DLookup($sql);	
	$domain[$numdomain] = $y;
	$sql = "SELECT COUNT(PARTICIPANT_ID) FROM TBL_PARTICIPANT WHERE (COUNTRY_ID = \"" . $domain[$numdomain] . "\")";
    	($emails[$numdomain]) =&DLookup($sql);	
    }
    for ($z = 1; $z <= $numdomain; $z++)
    {
	$count += 1;

	# Start a new file if maxinum number of rows has been reached.
	if ($count > $UserPrefs{"maxrows"} * $pagecount) {
	    &write_section_footer($pagecount, $numdomain, 0, $UserPrefs{"htmldomain"}, "", $Contest, 1);

	    for ($y = $UserPrefs{"dayshistory"}+1; $y >= 0; $y--) {
		$pageblocks[$y] = 0;
	    }

	    $pagecount += 1;
	    &write_bydomain_header($pagecount, $Contest);
	}

#	for ($y = $UserPrefs{"dayshistory"} - 4; $y > 0; $y--) {
#	    $domains{$domain}[$y] = defined($domains{$domain}[$y]) ? $domains{$domain}[$y] : 0;
#	}

#	for ($y = $UserPrefs{"dayshistory"} - 4; $y > 0; $y--) {
#	    $pageblocks[$y] += defined($domains{$domain}[$y]) ? $domains{$domain}[$y] : 0;
#	}
#	$pageblocks[0] += defined($domains{$domain}[0]) ? $domains{$domain}[0] : 0;

	
	if ($count % 2 == 0) {
	    print OUTFILE "      <TR CLASS=even ALIGN=RIGHT>\n";
	} else {
	    print OUTFILE "      <TR CLASS=odd ALIGN=RIGHT>\n";
	}

#	# Print out the Rank and Movement column.
#	$displacement = $domains_ry{$domain} == 0 ? $numdomain - $count : $domains_ry{$domain} - $count;
#	$msg = &create_move_msg($displacement);
#       print OUTFILE "       <TD NOWRAP ALIGN=LEFT>&nbsp;$count$msg</TD>\n";
       print OUTFILE "       <TD NOWRAP ALIGN=LEFT>&nbsp;$count</TD>\n";

	if (defined($domain[$z])) 
	{
	    $sql = "SELECT COUNTRY FROM TBL_COUNTRY WHERE (COUNTRY_ID = \"$domain[$z]\") LIMIT 1";
	    ($label) = &DLookup($sql);
	    if (defined($label))
	    {
	        $sql = "SELECT COUNTRY_ICON FROM TBL_COUNTRY WHERE (COUNTRY_ID = \"$domain[$z]\") LIMIT 1";
		($icon) = &DLookup($sql);
		$icon = ($icon . "." . $UserPrefs{"imageext"});
	    }
	    else
	    {
		$label = "Unknown";
		$icon = ("unknown." . $UserPrefs{"imageext"});
	    }
	}

	print OUTFILE "       <TD NOWRAP ALIGN=LEFT><IMG SRC=\"" . $UserPrefs{"htmlroot"} . $UserPrefs{"icondir"} . "flags/$icon\" ALIGN=BOTTOM BORDER=0 WIDTH=14 HEIGHT=14 ALT=\"$label\">&nbsp;$label</TD>\n";

	for ($y = $UserPrefs{"dayshistory"} - 3; $y >= 0; $y--) 
	{
	    $blocksthatday = 0;
	    for ($x = 1; $x <= $emails[$z]; $x++)
	    {
		$sql = "SELECT PARTICIPANT_ID FROM TBL_PARTICIPANT WHERE (COUNTRY_ID = \"" . $domain[$z] . "\") LIMIT " . ($x-1) . ",1"; 
		my ($email) = &DLookup($sql); 
		($year, $month, $day, $hour, $min, $sec, $wday) = &nice_gmtime($SystemPrefs{"timebegin"} - $y * (24*60*60));
		$sql = "SELECT IFNULL(SUM(WU_SIZE),0) FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE ((SUBMITTED_DATE = \"$year-$month-$day\") AND (PARTICIPANT_ID = $email))";
		($blocks) = &DLookup($sql);
		if ($blocks > 0)
		{
		    $blocksthatday += $blocks;
		}
	    }
	    if ($blocksthatday > 0)
	    {
	    	$pageblocks[$y+1] += $blocksthatday;
	    }
	    else
	    {
		$blocksthatday = "-";
	    }
	    
	    if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
	    {
		print OUTFILE "       <TD>$blocksthatday</TD>\n";
	    }
	    else
	    {
		if ($blocksthatday ne "-")
		{
		    print OUTFILE "       <TD>" . &ratef($blocksthatday * $SystemPrefs{"contestfactor"}[$Contest]) . "</TD>\n";
		}
		else
		{
		print OUTFILE "       <TD>$blocksthatday</TD>\n";
		}
	    }
	}
	$blocksthatdomain = 0;
	for ($x = 1; $x <= $emails[$z]; $x++)
	{
	    $sql = "SELECT PARTICIPANT_ID FROM TBL_PARTICIPANT WHERE (COUNTRY_ID = \"" . $domain[$z] . "\") LIMIT " . ($x-1) . ",1"; 
	    my ($email) = &DLookup($sql); 
	    ($year, $month, $day, $hour, $min, $sec, $wday) = &nice_gmtime($SystemPrefs{"timebegin"} - $y * (24*60*60));
	    $sql = "SELECT IFNULL(SUM(WU_SIZE),\"-\") FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE (PARTICIPANT_ID = $email)";
	    ($blocks) = &DLookup($sql);
	    if ($blocks ne "-")
	    {
		$blocksthatdomain += $blocks;
	    }
	    
    	    $sql = "SELECT MAX(SUBMITTED_DATE) FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE (PARTICIPANT_ID = \"$email\")";
	    ($LastSubmittedDate) = &DLookup($sql);
    	    $sql = "SELECT MAX(SUBMITTED_TIME) FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE ((SUBMITTED_DATE = \"$LastSubmittedDate\") AND (PARTICIPANT_ID = \"$email\"))";
    	    ($LastSubmittedTime) = &DLookup($sql);
	    $sql = "SELECT UNIX_TIMESTAMP(\"$LastSubmittedDate $LastSubmittedTime\")";
	    ($LastTime) = &DLookup($sql);
	}
	if ($blocksthatdomain == 0)
	{
	    $blocksthatdomain = "-";
	}

	if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
	{
	    print OUTFILE "       <TD>$blocksthatdomain</TD>\n";
	}
	else
	{
	    if ($blocksthatdomain ne "-")
	    {
		print OUTFILE "       <TD>" . &ratef($blocksthatdomain * $SystemPrefs{"contestfactor"}[$Contest]) . "</TD>\n";
	    }
	    else
	    {
		print OUTFILE "       <TD>$blocksthatdomain</TD>\n";
	    }
	}
	
        if ($blocksthatdomain ne "-")
        {
	    $pageblocks[0] += $blocksthatdomain;
	}
	
	print OUTFILE "       <TD>$emails[$z]</TD>\n";

#	# Print out the Last Seen column.
	$lastseentime = ($SystemPrefs{"timebegin"}-($LastTime));
	$lastseen = &ls($lastseentime);
	$lastseencolor = &ls_color($lastseentime);
	print OUTFILE "       <TD CLASS=$lastseencolor NOWRAP>$lastseen</TD>\n";

#	# Print out the Percent column.
	my $percentofteam;
	if ($blocksoverall > 0)
	{ 
	    $percentofteam = sprintf("%.2f", 100 * $blocksthatdomain / $blocksoverall);
	}
	else
	{
	    $percentofteam = sprintf("%.2f", 0);
	}
	print OUTFILE "       <TD>$percentofteam%</TD>\n",
		      "      </TR>\n";

    }
    
    &write_section_footer($pagecount, $numdomain, 1, $UserPrefs{"htmldomain"}, "", $Contest, 1);

    &Debugln("Exiting write_bydomain($Contest) ...");
}  # write_bydomain


#--- Method Prolog -----------------------------------------------------------
#  Name: write_byhour
#
#  Parameters: 
#      Contest - The contest to collect data for. 1=RC5, 2=DES, etc.
#
#  Return Value: None.
#
#  Description:
#    Write main team stats by hour of day.
#
#--- End Method Prolog -------------------------------------------------------
sub write_byhour
{
    my $Contest = shift @_;
    &Debugln("Entering write_byhour($Contest) ...");
    my $sql;
    my @hourofday;
    my $best;
    my ($line, $clock, $ampm);
    my ($x, $y);
    my ($width, $antiwidth);
    my $percentofteam;
    my $blocksoverall;
    my $totaldays;
    my $rate;
    

    open(OUTFILE, ">" . $UserPrefs{"htmldir"} . $SystemPrefs{"contestsuffix"}[$Contest] . "/" . $UserPrefs{"htmlhour"}) or die "Personal Proxy Statistics : Cannot open " . $UserPrefs{"htmldir"} . $SystemPrefs{"contestsuffix"}[$Contest] . "/" . $UserPrefs{"htmlhour"} . " ($!)\n";
    &write_header($Contest);
    print OUTFILE "     <TABLE BORDER=0 CELLPADDING=0 CELLSPACING=0 WIDTH=\"100%\">\n",
                  "      <TR ALIGN=CENTER>\n",
                  "       <TD CLASS=section NOWRAP>Hour of Day GMT</TD>\n",
                  "       <TD CLASS=section WIDTH=3 NOWRAP>&nbsp;</TD>\n",
                  "       <TD CLASS=section NOWRAP>" . $SystemPrefs{"contestheaderblocktype"}[$Contest] . "s</TD>\n",
                  "       <TD CLASS=section WIDTH=3 NOWRAP>&nbsp;</TD>\n",
                  "       <TD CLASS=section WIDTH=\"100%\" NOWRAP>Graphical Rate in " . $SystemPrefs{"contestheaderblocktype"}[$Contest] . "s</TD>\n",
                  "       <TD CLASS=section WIDTH=3 NOWRAP>&nbsp;</TD>\n",
                  "       <TD CLASS=section NOWRAP>" . $SystemPrefs{"contestheaderblockmeasure"}[$Contest] . " Rate</TD>\n",
                  "       <TD CLASS=section WIDTH=3 NOWRAP>&nbsp;</TD>\n",
                  "       <TD CLASS=section NOWRAP>Percent</TD>\n",
                  "      </TR>\n";

    $sql = "SELECT IFNULL(SUM(WU_SIZE),0) FROM " . $SystemPrefs{"contestwutable"}[$Contest];
    ($blocksoverall) = &DLookup($sql);

    # Calculate best day of week
    $best = 0;
    for ($y = 0; $y < 24; $y++) {
	$sql = "SELECT IFNULL(SUM(WU_SIZE),0) FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " WHERE (HOUR(SUBMITTED_TIME) = $y)";
	($hourofday[$y]) = &DLookup($sql); 
	if ($hourofday[$y] > $best) 
	{
	    $best = $hourofday[$y];
	}
    }
    
    $sql = "SELECT UNIX_TIMESTAMP(MAX(SUBMITTED_DATE)) FROM " . $SystemPrefs{"contestwutable"}[$Contest];
    my ($LastSubmitted) = &DLookup($sql);
    $sql = "SELECT UNIX_TIMESTAMP(MIN(SUBMITTED_DATE)) FROM " . $SystemPrefs{"contestwutable"}[$Contest];
    my ($FirstSubmitted) =&DLookup($sql);
    
    $totaldays = int((($LastSubmitted - $FirstSubmitted) / (60*60*24)) + 1);
    
    
    # Calculate past rates and display rates.
    if ($totaldays > 1)
    {
	for ($y = 0; $y < 24; $y++) 
	{
	    $x = (($y - 1) % 12) + 1;
	    $clock = ($y % 12) + 1;
	    if ($y < 12) 
	    {
		$ampm = "am";
	    } 
	    else 
	    {
		$ampm = "pm";
	    }
	    if ($best == 0) 
	    {
		$width = 0;
	    } 
	    else 
	    {
		$width = int($hourofday[$y] * 100 / $best);
	    }
	    $antiwidth = 100 - $width;
	    if ($hourofday[$y] == 0) 
	    {
		$rate = &ratef(0);
	    } 
	    else 
	    {
		$rate = &ratef(&rate_day((24 * $hourofday[$y]) / ($totaldays - 1), $SystemPrefs{"contestfactor"}[$Contest], $Contest));
	    }

	    # Print out the Percent column.
	    if ($blocksoverall > 0)
	    {
		$percentofteam = sprintf("%.2f", 100 * $hourofday[$y] / $blocksoverall);
	    }
	    else
	    {
		$percentofteam = sprintf("%.2f", 0);
	    }
	    $line = "      <TR>\n";
	    $line .= "       <TD NOWRAP ALIGN=LEFT><IMG SRC=\"" . $UserPrefs{"htmlroot"} . $UserPrefs{"icondir"} . "clock/hr$clock." . $UserPrefs{"imageext"} . "\" ALIGN=BOTTOM BORDER=0 WIDTH=14 HEIGHT=14 ALT=\"$x:00$ampm-$x:59$ampm\">&nbsp;$x:00$ampm-$x:59$ampm</TD>\n";
	    $line .= "       <TD WIDTH=3 NOWRAP>&nbsp;</TD>\n";
	    if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
	    {
		$line .= "       <TD NOWRAP ALIGN=RIGHT>$hourofday[$y]</TD>\n";
	    }
	    else
	    {
		if ($hourofday[$y] > 0)
		{
		    $line .= "       <TD NOWRAP ALIGN=RIGHT>" . &ratef($hourofday[$y] * $SystemPrefs{"contestfactor"}[$Contest]) . "</TD>\n";
		}
		else
		{
		    $line .= "       <TD NOWRAP ALIGN=RIGHT>$hourofday[$y]</TD>\n";
		}
	    }
	    $line .= "       <TD WIDTH=3 NOWRAP>&nbsp;</TD>\n";
	    $line .= "       <TD WIDTH=\"100%\" NOWRAP><TABLE WIDTH=\"100%\" BORDER=0 CELLSPACING=0 CELLPADDING=0><TR><TD CLASS=history WIDTH=\"$width%\"><IMG SRC=\"" . $UserPrefs{"htmlroot"} . $UserPrefs{"icondir"} . "clock/blank." . $UserPrefs{"imageext"} . "\" ALIGN=BOTTOM BORDER=0 WIDTH=3 HEIGHT=14>&nbsp;</TD><TD WIDTH=\"$antiwidth%\">&nbsp;</TD></TR></TABLE></TD>\n";
	    $line .= "       <TD WIDTH=3 NOWRAP>&nbsp;</TD>\n";
	    $line .= "       <TD NOWRAP ALIGN=RIGHT>$rate" . $SystemPrefs{"contestblockmeasure"}[$Contest] . "s/sec</TD>\n";
	    $line .= "       <TD WIDTH=3 NOWRAP>&nbsp;</TD>\n";
	    $line .= "       <TD NOWRAP ALIGN=RIGHT>$percentofteam%</TD>\n";
	    $line .= "      </TR>\n";
	    print OUTFILE "$line";
	}
    }

    print OUTFILE "     </TABLE>\n";
    &write_footer($Contest);
    close OUTFILE;

    &Debugln("Exiting write_byhour($Contest) ...");
}  # write_byhour


#--- Method Prolog -----------------------------------------------------------
#  Name: write_bydate
#
#  Parameters: 
#      Contest - The contest to collect data for. 1=RC5, 2=DES, etc.
#
#  Return Value: None.
#
#  Description:
#    Write main team stats by date.
#
#--- End Method Prolog -------------------------------------------------------
sub write_bydate
{
    my $Contest = shift @_;
    &Debugln("Entering write_bydate($Contest) ...");
    my ($sql, $sth);
    my (@row, @dates, @blocks);
    my $best;
    my ($x, $y, $z);
    my ($year, $mon, $mday, $hour, $min, $sec, $wday);
    my ($width, $antiwidth);
    my ($line, $rate);

    open(OUTFILE, ">" . $UserPrefs{"htmldir"} . $SystemPrefs{"contestsuffix"}[$Contest] . "/" . $UserPrefs{"htmldate"}) or die "Personal Proxy Statistics : Cannot open " . $UserPrefs{"htmldir"} . $SystemPrefs{"contestsuffix"}[$Contest] . "/" . $UserPrefs{"htmldate"} . " ($!)\n";
    &write_header($Contest);
    print OUTFILE "     <TABLE BORDER=0 CELLPADDING=0 CELLSPACING=0 WIDTH=\"100%\">\n",
                  "      <TR ALIGN=CENTER>\n",
                  "       <TD CLASS=section NOWRAP>Date</TD>\n",
                  "       <TD CLASS=section WIDTH=3 NOWRAP>&nbsp;</TD>\n",
                  "       <TD CLASS=section NOWRAP>" . $SystemPrefs{"contestheaderblocktype"}[$Contest] . "s</TD>\n",
                  "       <TD CLASS=section WIDTH=3 NOWRAP>&nbsp;</TD>\n",
                  "       <TD CLASS=section WIDTH=\"100%\" NOWRAP>Graphical Rate in " . $SystemPrefs{"contestheaderblocktype"}[$Contest] . "s</TD>\n",
                  "       <TD CLASS=section WIDTH=3 NOWRAP>&nbsp;</TD>\n",
                  "       <TD CLASS=section NOWRAP>" . $SystemPrefs{"contestheaderblockmeasure"}[$Contest] . " Rate</TD>\n",
                  "      </TR>\n";

    # Calculate best day
    $best = 0;
    $z = 0;
    
    $sql = "SELECT UNIX_TIMESTAMP(SUBMITTED_DATE), SUM(WU_SIZE) FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " GROUP BY 1 ORDER BY 1 DESC";
    $sth = &SelectSQL($sql);
    while (@row = $sth->fetchrow_array()) 
    {
	$z++;
	$dates[$z] = $row[0];
	$blocks[$z] = $row[1];
	if ($blocks[$z] > $best) 
	{
	    $best = $blocks[$z];
	}
    }

    # Calculate past rates
    my $prev = $SystemPrefs{"timebegin"};
    for ($y = 1; $y < $z; $y++) 
    {
	while ($prev > ($dates[$y] + 86400))
	{
	    $prev = $prev - 86400;
	    ($year, $mon, $mday, $hour, $min, $sec, $wday) = &nice_gmtime($prev);
	    $rate = &ratef(&rate_day(0, $SystemPrefs{"contestfactor"}[$Contest], $Contest));
	    $width = 0;
	    $antiwidth = 100;
	    $line = "      <TR>\n";
	    $line .= "       <TD NOWRAP>$mday $Months[$mon-1] $year GMT</TD>\n";
	    $line .= "       <TD WIDTH=3 NOWRAP>&nbsp;</TD>\n";
	    $line .= "       <TD NOWRAP ALIGN=RIGHT>0</TD>\n";
	    $line .= "       <TD WIDTH=3 NOWRAP>&nbsp;</TD>\n";
	    $line .= "       <TD WIDTH=\"100%\" NOWRAP><TABLE WIDTH=\"100%\" BORDER=0 CELLSPACING=0 CELLPADDING=0><TR><TD CLASS=history WIDTH=\"$width%\">&nbsp;</TD><TD WIDTH=\"$antiwidth%\">&nbsp;</TD></TR></TABLE></TD>\n";
	    $line .= "       <TD WIDTH=3 NOWRAP>&nbsp;</TD>\n";
	    $line .= "       <TD NOWRAP ALIGN=RIGHT>$rate" . $SystemPrefs{"contestblockmeasure"}[$Contest] . "s/sec</TD>\n";
	    $line .= "      </TR>\n";
	print OUTFILE "$line";
	}
	$prev = $dates[$y];

	($year, $mon, $mday, $hour, $min, $sec, $wday) = &nice_gmtime($dates[$y]);
	if ($best == 0) 
	{
	    $width = 0;
	} else {
	    $width = int($blocks[$y] * 100 / $best);
	}
	$antiwidth = 100 - $width;
	if ($y > 1)
	{ 
	 $rate = &ratef(&rate_day($blocks[$y], $SystemPrefs{"contestfactor"}[$Contest], $Contest));
	}
	else
	{
	 $rate = &ratef(&rate_day($blocks[$y] / ($SystemPrefs{"secstoday"} / (60*60*24)), $SystemPrefs{"contestfactor"}[$Contest], $Contest));
	}
	$line = "      <TR>\n";
	$line .= "       <TD NOWRAP>$mday $Months[$mon-1] $year GMT</TD>\n";
	$line .= "       <TD WIDTH=3 NOWRAP>&nbsp;</TD>\n";
	if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
	{
	    $line .= "       <TD NOWRAP ALIGN=RIGHT>$blocks[$y]</TD>\n";
	}
	else
	{
	    $line .= "       <TD NOWRAP ALIGN=RIGHT>" . &ratef($blocks[$y] * $SystemPrefs{"contestfactor"}[$Contest]) . "</TD>\n";
	}
	$line .= "       <TD WIDTH=3 NOWRAP>&nbsp;</TD>\n";
	$line .= "       <TD WIDTH=\"100%\" NOWRAP><TABLE WIDTH=\"100%\" BORDER=0 CELLSPACING=0 CELLPADDING=0><TR><TD CLASS=history WIDTH=\"$width%\">&nbsp;</TD><TD WIDTH=\"$antiwidth%\">&nbsp;</TD></TR></TABLE></TD>\n";
	$line .= "       <TD WIDTH=3 NOWRAP>&nbsp;</TD>\n";
	$line .= "       <TD NOWRAP ALIGN=RIGHT>$rate" . $SystemPrefs{"contestblockmeasure"}[$Contest] . "s/sec</TD>\n";
	$line .= "      </TR>\n";
	print OUTFILE "$line";
    }
    
    print OUTFILE "     </TABLE>\n";
    &write_footer($Contest);
    close OUTFILE;

    &Debugln("Exiting write_bydate($Contest) ...");
}  # write_bydate



#--- Method Prolog -----------------------------------------------------------
#  Name: write_byweek
#
#  Parameters: 
#      Contest - The contest to collect data for. 1=RC5, 2=DES, etc.
#
#  Return Value: None.
#
#  Description:
#    Write the main team stats by week
#
#--- End Method Prolog -------------------------------------------------------
sub write_byweek
{
    my $Contest = shift @_;
    &Debugln("Entering write_byweek($Contest) ...");
    my ($year, $mon, $mday, $hour, $min, $sec, $wday);
    my ($sql, $best, $numdays);
    my ($x, $y, $z);
    my $numblocks;
    my $sth;
    my $blocks;
    my $date;
    my ($prev, $curr);
    my ($rate, $width, $antiwidth);
    my $line;

    open(OUTFILE, ">" . $UserPrefs{"htmldir"} . $SystemPrefs{"contestsuffix"}[$Contest] . "/" . $UserPrefs{"htmlweek"}) or die "Personal Proxy Statistics : Cannot open " . $UserPrefs{"htmldir"} . $SystemPrefs{"contestsuffix"}[$Contest] . "/" . $UserPrefs{"htmlweek"} . " ($!)\n";
    &write_header($Contest);
    print OUTFILE "     <TABLE BORDER=0 CELLPADDING=0 CELLSPACING=0 WIDTH=\"100%\">\n",
                  "      <TR ALIGN=CENTER>\n",
                  "       <TD CLASS=section NOWRAP>Week</TD>\n",
                  "       <TD CLASS=section WIDTH=3 NOWRAP>&nbsp;</TD>\n",
                  "       <TD CLASS=section NOWRAP>" . $SystemPrefs{"contestheaderblocktype"}[$Contest] . "s</TD>\n",
                  "       <TD CLASS=section WIDTH=3 NOWRAP>&nbsp;</TD>\n",
                  "       <TD CLASS=section WIDTH=\"100%\" NOWRAP>Graphical Rate in " . $SystemPrefs{"contestheaderblocktype"}[$Contest] . "s</TD>\n",
                  "       <TD CLASS=section WIDTH=3 NOWRAP>&nbsp;</TD>\n",
                  "       <TD CLASS=section NOWRAP>" . $SystemPrefs{"contestheaderblockmeasure"}[$Contest] . " Rate</TD>\n",
                  "      </TR>\n";

    $sql = "SELECT UNIX_TIMESTAMP(MAX(SUBMITTED_DATE)) FROM " . $SystemPrefs{"contestwutable"}[$Contest];
    my ($LastSubmitted) = &DLookup($sql);
    $sql = "SELECT UNIX_TIMESTAMP(MIN(SUBMITTED_DATE)) FROM " . $SystemPrefs{"contestwutable"}[$Contest];
    my ($FirstSubmitted) =&DLookup($sql);
    my $totaldays = int((($LastSubmitted - $FirstSubmitted) / (60*60*24)) + 1);

    # Calculate best week
    $best = 0;
    $z = 0;
    $numblocks = undef;
    $prev = $SystemPrefs{"timebegin"};
    my (@row, @weeks, @dates);
    
    $sql = "SELECT UNIX_TIMESTAMP(SUBMITTED_DATE), SUM(WU_SIZE) FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " GROUP BY 1 ORDER BY 1 DESC ";
    $sth = &SelectSQL($sql);
    while (@row = $sth->fetchrow_array()) 
    {
	$date = $row[0];
	$blocks = $row[1];
	($year, $mon, $mday, $hour, $min, $sec, $wday) = &nice_gmtime($date);
	$numblocks += $blocks;
	if (($wday == 1) && (defined($numblocks)))
	{
	    if ($numblocks > $best) 
	    {
		$best = $numblocks;
	    }
	    $z++;
	    while ($prev > ($date + 604800))
	    {
	     $prev = $prev - 604800;
	     $dates[$z] = $prev;
	     $weeks[$z] =0;
	     $z++;
	    }
	    $dates[$z] = $date;
	    $prev = $date;
	    $weeks[$z] = $numblocks;
	    $numblocks = 0;
	}
    }

    $z++;
    if ($numblocks > 0)
    {
	($year, $mon, $mday, $hour, $min, $sec, $wday) = &nice_gmtime($date);
	$numdays = (7 - $wday);
	if ($wday == 0) {$wday = 7};
	$dates[$z] = $date - (($wday) * (24*60*60));
	$weeks[$z] = $numblocks;
    }
    else
    {
	$numdays = 0;
	$dates[$z] = 0;
	$weeks[$z] = 0;
    }

    # Calculate past rates
    if ($z >= 1) 
    {
	if ($best == 0) 
	{
	    $width = 100;
	} 
	else 
	{
	    $width = int($weeks[1] * 100 / $best);
	}
	$antiwidth = 100 - $width;
	($year, $mon, $mday, $hour, $min, $sec, $wday) = &nice_gmtime($SystemPrefs{"timebegin"} - 1 * (24*60*60));
	if ($wday == 0) {$wday = 7};
	$rate = &ratef(&rate_day($weeks[1], $SystemPrefs{"contestfactor"}[$Contest], $Contest) / $wday); 
	($year, $mon, $mday, $hour, $min, $sec, $wday) = &nice_gmtime($dates[1]);
	$curr = $mday . " " . $Months[$mon-1] . " " . $year;
	$line = "      <TR>\n";
	$line .= "       <TD NOWRAP ALIGN=RIGHT>$curr GMT</TD>\n";
	$line .= "       <TD WIDTH=3 NOWRAP>&nbsp;</TD>\n";
	if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
	{
	    $line .= "       <TD NOWRAP ALIGN=RIGHT>" . $weeks[1] . "</TD>\n";
	}
	else
	{
	    $line .= "       <TD NOWRAP ALIGN=RIGHT>" . &ratef($weeks[1] * $SystemPrefs{"contestfactor"}[$Contest]) . "</TD>\n";
	}
	$line .= "       <TD WIDTH=3 NOWRAP>&nbsp;</TD>\n";
	$line .= "       <TD WIDTH=\"100%\" NOWRAP><TABLE WIDTH=\"100%\" BORDER=0 CELLSPACING=0 CELLPADDING=0><TR><TD CLASS=history WIDTH=\"$width%\">&nbsp;</TD><TD WIDTH=\"$antiwidth%\">&nbsp;</TD></TR></TABLE></TD>\n";
	$line .= "       <TD WIDTH=3 NOWRAP>&nbsp;</TD>\n";
	$line .= "       <TD NOWRAP ALIGN=RIGHT>$rate" . $SystemPrefs{"contestblockmeasure"}[$Contest] . "s/sec</TD>\n";
	$line .= "      </TR>\n";
	print OUTFILE "$line";
    }
    if ($z > 2)
    {
	for ($y = 2; $y < $z; $y++) 
	{
	    ($year, $mon, $mday, $hour, $min, $sec, $wday) = &nice_gmtime($dates[$y]);
	    $curr = $mday . " " . $Months[$mon-1] . " " . $year;
	
	    if ($best > 0) 
	    {
		$width = int($weeks[$y] * 100 / $best);
	    }
	    else 
	    {
		$width = 0;
	    }
		$antiwidth = 100 - $width;
		$rate = &ratef(&rate_day($weeks[$y], $SystemPrefs{"contestfactor"}[$Contest], $Contest) / 7);
		$line = "      <TR>\n";
		$line .= "       <TD NOWRAP ALIGN=RIGHT>$curr GMT</TD>\n";
		$line .= "       <TD WIDTH=3 NOWRAP>&nbsp;</TD>\n";
		if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
		{
		    $line .= "       <TD NOWRAP ALIGN=RIGHT>" . $weeks[$y] . "</TD>\n";
		}
		else
		{
		    $line .= "       <TD NOWRAP ALIGN=RIGHT>" . &ratef($weeks[$y] * $SystemPrefs{"contestfactor"}[$Contest]) . "</TD>\n";
		}
	        $line .= "       <TD NOWRAP ALIGN=RIGHT>" . $weeks[$y] . "</TD>\n";
		$line .= "       <TD WIDTH=3 NOWRAP>&nbsp;</TD>\n";
	        $line .= "       <TD WIDTH=\"100%\" NOWRAP><TABLE WIDTH=\"100%\" BORDER=0 CELLSPACING=0 CELLPADDING=0><TR><TD CLASS=history WIDTH=\"$width%\">&nbsp;</TD><TD WIDTH=\"$antiwidth%\">&nbsp;</TD></TR></TABLE></TD>\n";
		$line .= "       <TD WIDTH=3 NOWRAP>&nbsp;</TD>\n";
		$line .= "       <TD NOWRAP ALIGN=RIGHT>$rate" . $SystemPrefs{"contestblockmeasure"}[$Contest] . "s/sec</TD>\n";
	        $line .= "      </TR>\n";
		print OUTFILE "$line";
	}
    }
    
    if ($numdays > 0) 
    {
	if ($best == 0) 
	{
	    $width = 100;
	} else {
	    $width = int($weeks[$z] * 100 / $best);
	}
	$antiwidth = 100 - $width;
	$rate = &ratef(&rate_day($weeks[$z], $SystemPrefs{"contestfactor"}[$Contest], $Contest) / $numdays); 
	($year, $mon, $mday, $hour, $min, $sec, $wday) = &nice_gmtime($dates[$z]);
	$curr = $mday . " " . $Months[$mon-1] . " " . $year;
	$line = "      <TR>\n";
	$line .= "       <TD NOWRAP ALIGN=RIGHT>$curr GMT</TD>\n";
	$line .= "       <TD WIDTH=3 NOWRAP>&nbsp;</TD>\n";
	if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
	{
	    $line .= "       <TD NOWRAP ALIGN=RIGHT>" . $weeks[$z] . "</TD>\n";
	}
	else
	{
	    $line .= "       <TD NOWRAP ALIGN=RIGHT>" . &ratef($weeks[$z] * $SystemPrefs{"contestfactor"}[$Contest]) . "</TD>\n";
	}
	$line .= "       <TD WIDTH=3 NOWRAP>&nbsp;</TD>\n";
	$line .= "       <TD WIDTH=\"100%\" NOWRAP><TABLE WIDTH=\"100%\" BORDER=0 CELLSPACING=0 CELLPADDING=0><TR><TD CLASS=history WIDTH=\"$width%\">&nbsp;</TD><TD WIDTH=\"$antiwidth%\">&nbsp;</TD></TR></TABLE></TD>\n";
	$line .= "       <TD WIDTH=3 NOWRAP>&nbsp;</TD>\n";
	$line .= "       <TD NOWRAP ALIGN=RIGHT>$rate" . $SystemPrefs{"contestblockmeasure"}[$Contest] . "s/sec</TD>\n";
	$line .= "      </TR>\n";
	print OUTFILE "$line";
    }

    print OUTFILE "     </TABLE>\n";
    &write_footer($Contest);
    close OUTFILE;

    &Debugln("Exiting write_byweek($Contest) ...");
}  # write_byweek



#--- Method Prolog -----------------------------------------------------------
#  Name: write_bymonth
#
#  Parameters: 
#      Contest - The contest to collect data for. 1=RC5, 2=DES, etc.
#
#  Return Value: None.
#
#  Description:
#    Write the main team stats by month
#
#--- End Method Prolog -------------------------------------------------------
sub write_bymonth
{
    my $Contest = shift @_;
    &Debugln("Entering write_bymonth($Contest) ...");
    my ($year, $mon, $mday, $hour, $min, $sec, $wday);
    my ($sql, $best, $numdays);
    my ($x, $y, $z);
    my $month;
    my $sth;
    my $blocks;
    my ($date_month, $date_year);
    my ($prev, $curr);
    my ($rate, $width, $antiwidth);
    my $line;

    open(OUTFILE, ">" . $UserPrefs{"htmldir"} . $SystemPrefs{"contestsuffix"}[$Contest] . "/" . $UserPrefs{"htmlmonth"}) or die "Personal Proxy Statistics : Cannot open " . $UserPrefs{"htmldir"} . $SystemPrefs{"contestsuffix"}[$Contest] . "/" . $UserPrefs{"htmlmonth"} . " ($!)\n";
    &write_header($Contest);
    print OUTFILE "     <TABLE BORDER=0 CELLPADDING=0 CELLSPACING=0 WIDTH=\"100%\">\n",
                  "      <TR ALIGN=CENTER>\n",
                  "       <TD CLASS=section NOWRAP>Month</TD>\n",
                  "       <TD CLASS=section WIDTH=3 NOWRAP>&nbsp;</TD>\n",
                  "       <TD CLASS=section NOWRAP>" . $SystemPrefs{"contestheaderblocktype"}[$Contest] . "s</TD>\n",
                  "       <TD CLASS=section WIDTH=3 NOWRAP>&nbsp;</TD>\n",
                  "       <TD CLASS=section WIDTH=\"100%\" NOWRAP>Graphical Rate in " . $SystemPrefs{"contestheaderblocktype"}[$Contest] . "s</TD>\n",
                  "       <TD CLASS=section WIDTH=3 NOWRAP>&nbsp;</TD>\n",
                  "       <TD CLASS=section NOWRAP>" . $SystemPrefs{"contestheaderblockmeasure"}[$Contest] . " Rate</TD>\n",
                  "      </TR>\n";

    $sql = "SELECT UNIX_TIMESTAMP(MAX(SUBMITTED_DATE)) FROM " . $SystemPrefs{"contestwutable"}[$Contest];
    my ($LastSubmitted) = &DLookup($sql);
    $sql = "SELECT UNIX_TIMESTAMP(MIN(SUBMITTED_DATE)) FROM " . $SystemPrefs{"contestwutable"}[$Contest];
    my ($FirstSubmitted) =&DLookup($sql);
    my $totaldays = int((($LastSubmitted - $FirstSubmitted) / (60*60*24)) + 1);

    # Calculate best month
    $best = 0;
    $z = 0;
    ($year, $mon, $mday, $hour, $min, $sec, $wday) = &nice_gmtime($SystemPrefs{"timebegin"});
    $prev = ($year . "-" . $mon . "-1");
    my (@row, @dates_month, @dates_year, @monthblocks);
    
    $sql = "SELECT MONTH(SUBMITTED_DATE), YEAR(SUBMITTED_DATE), SUM(WU_SIZE) FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " GROUP BY 2,1 ORDER BY 2 DESC,1 DESC";
    $sth = &SelectSQL($sql);
    while (@row = $sth->fetchrow_array()) 
    {
	$date_month = $row[0];
	$date_year = $row[1];
	$blocks = $row[2];
        if ($blocks > $best) 
        {
	    $best = $blocks;
	}
	$z++;
	$sql = "SELECT DATE_SUB(\"$prev\", INTERVAL 1 MONTH)";
	($prev) = &DLookup($sql);
        ($year, $month, $mday) = split(/-/, $prev);
	while ((int($year) != $date_year) && (int($month) != $date_month))
        {
	    $dates_month[$z] = $month;
    	    $dates_year[$z] = $year;
            $monthblocks[$z] =0;
            $z++;
	    $sql = "SELECT DATE_SUB(\"$prev\", INTERVAL 1 MONTH)";
	    ($prev) = &DLookup($sql); 
	    ($year, $month, $mday) = split(/-/, $prev);
        }
        $dates_month[$z] = $date_month;
        $dates_year[$z] = $date_year;
	$prev = ($date_year . "-" . $date_month . "-1");
	$monthblocks[$z] = $blocks;
    }

    # Calculate past rates
    if ($z > 1) 
    {
	if ($best == 0) 
	{
	    $width = 100;
	} 
	else 
	{
	    $width = int($monthblocks[1] * 100 / $best);
	}
	$antiwidth = 100 - $width;
	($year, $mon, $mday, $hour, $min, $sec, $wday) = &nice_gmtime($SystemPrefs{"timebegin"} - 1 * (24*60*60));
	$rate = &ratef(&rate_day($monthblocks[1], $SystemPrefs{"contestfactor"}[$Contest], $Contest) / $mday); 
	$curr =  $Months[$dates_month[1]-1] . " " . $dates_year[1];
	$line = "      <TR>\n";
	$line .= "       <TD NOWRAP ALIGN=RIGHT>$curr GMT</TD>\n";
	$line .= "       <TD WIDTH=3 NOWRAP>&nbsp;</TD>\n";
	if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
	{
	    $line .= "       <TD NOWRAP ALIGN=RIGHT>" . $monthblocks[1] . "</TD>\n";
	}
	else
	{
	    $line .= "       <TD NOWRAP ALIGN=RIGHT>" . &ratef($monthblocks[1] * $SystemPrefs{"contestfactor"}[$Contest]) . "</TD>\n";
	}
	$line .= "       <TD WIDTH=3 NOWRAP>&nbsp;</TD>\n";
	$line .= "       <TD WIDTH=\"100%\" NOWRAP><TABLE WIDTH=\"100%\" BORDER=0 CELLSPACING=0 CELLPADDING=0><TR><TD CLASS=history WIDTH=\"$width%\">&nbsp;</TD><TD WIDTH=\"$antiwidth%\">&nbsp;</TD></TR></TABLE></TD>\n";
	$line .= "       <TD WIDTH=3 NOWRAP>&nbsp;</TD>\n";
	$line .= "       <TD NOWRAP ALIGN=RIGHT>$rate" . $SystemPrefs{"contestblockmeasure"}[$Contest] . "s/sec</TD>\n";
	$line .= "      </TR>\n";
	print OUTFILE "$line";
    }
    if ($z > 2)
    {
	for ($y = 2; $y < $z; $y++) 
	{
	    $curr =  $Months[$dates_month[$y]-1] . " " . $dates_year[$y];
	
	    if ($best > 0) 
	    {
		$width = int($monthblocks[$y] * 100 / $best);
	    }
	    else 
	    {
		$width = 0;
	    }
	    $antiwidth = 100 - $width;
	    $sql="SELECT DAYOFMONTH(DATE_SUB(DATE_ADD(\"" . $dates_year[$y] . "-" . $dates_month[$y] ."\-1\", INTERVAL 1 MONTH), INTERVAL 1 DAY))";
	    ($numdays)=&DLookup($sql);
	    $rate = &ratef(&rate_day($monthblocks[$y], $SystemPrefs{"contestfactor"}[$Contest], $Contest) / $numdays);
	    $line = "      <TR>\n";
	    $line .= "       <TD NOWRAP ALIGN=RIGHT>$curr GMT</TD>\n";
	    $line .= "       <TD WIDTH=3 NOWRAP>&nbsp;</TD>\n";
	    if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
	    {
		$line .= "       <TD NOWRAP ALIGN=RIGHT>" . $monthblocks[$y] . "</TD>\n";
	    }
	    else
	    {
		$line .= "       <TD NOWRAP ALIGN=RIGHT>" . &ratef($monthblocks[$y] * $SystemPrefs{"contestfactor"}[$Contest]) . "</TD>\n";
	    }
	    $line .= "       <TD WIDTH=3 NOWRAP>&nbsp;</TD>\n";
	    $line .= "       <TD WIDTH=\"100%\" NOWRAP><TABLE WIDTH=\"100%\" BORDER=0 CELLSPACING=0 CELLPADDING=0><TR><TD CLASS=history WIDTH=\"$width%\">&nbsp;</TD><TD WIDTH=\"$antiwidth%\">&nbsp;</TD></TR></TABLE></TD>\n";
	    $line .= "       <TD WIDTH=3 NOWRAP>&nbsp;</TD>\n";
	    $line .= "       <TD NOWRAP ALIGN=RIGHT>$rate" . $SystemPrefs{"contestblockmeasure"}[$Contest] . "s/sec</TD>\n";
	    $line .= "      </TR>\n";
	    print OUTFILE "$line";
	}
    }

    if ($z >= 1)
    {
	$sql="SELECT UNIX_TIMESTAMP(DATE_SUB(DATE_ADD(\"" . $dates_year[$z] . "-" . $dates_month[$z] ."-1\", INTERVAL 1 MONTH), INTERVAL 1 DAY))";
	my ($endofmonth) = &DLookup($sql);
	if ($SystemPrefs{"timebegin"} < $endofmonth)
	{
	    $endofmonth = $SystemPrefs{"timebegin"};	
	}
	$sql="SELECT DAYOFMONTH(FROM_UNIXTIME(\"$endofmonth\"))";
	($numdays) = &DLookup($sql);
	($year, $mon, $mday, $hour, $min, $sec, $wday) = &nice_gmtime($FirstSubmitted);
    	$numdays = $numdays - $mday +1 ;
	if ($numdays < 1) 
	{
	    $numdays = 1;
	}
	if ($best == 0) 
	{
	    $width = 100;
	} else {
	    $width = int($monthblocks[$z] * 100 / $best);
	}
	$antiwidth = 100 - $width;
	$rate = &ratef(&rate_day($monthblocks[$z], $SystemPrefs{"contestfactor"}[$Contest], $Contest) / $numdays); 
	$curr =  $Months[$dates_month[$z]-1] . " " . $dates_year[$z];
	$line = "      <TR>\n";
	$line .= "       <TD NOWRAP ALIGN=RIGHT>$curr GMT</TD>\n";
	$line .= "       <TD WIDTH=3 NOWRAP>&nbsp;</TD>\n";
	if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
	{
	    $line .= "       <TD NOWRAP ALIGN=RIGHT>" . $monthblocks[$z] . "</TD>\n";
	}
	else
	{
	    $line .= "       <TD NOWRAP ALIGN=RIGHT>" . &ratef($monthblocks[$z] * $SystemPrefs{"contestfactor"}[$Contest]) . "</TD>\n";
	}
	$line .= "       <TD WIDTH=3 NOWRAP>&nbsp;</TD>\n";
	$line .= "       <TD WIDTH=\"100%\" NOWRAP><TABLE WIDTH=\"100%\" BORDER=0 CELLSPACING=0 CELLPADDING=0><TR><TD CLASS=history WIDTH=\"$width%\">&nbsp;</TD><TD WIDTH=\"$antiwidth%\">&nbsp;</TD></TR></TABLE></TD>\n";
	$line .= "       <TD WIDTH=3 NOWRAP>&nbsp;</TD>\n";
	$line .= "       <TD NOWRAP ALIGN=RIGHT>$rate" . $SystemPrefs{"contestblockmeasure"}[$Contest] . "s/sec</TD>\n";
	$line .= "      </TR>\n";
	print OUTFILE "$line";
    }



    print OUTFILE "     </TABLE>\n";
    &write_footer($Contest);
    close OUTFILE;

    &Debugln("Exiting write_bymonth($Contest) ...");
}  # write_bymonth



#--- Method Prolog -----------------------------------------------------------
#  Name: write_byyear
#
#  Parameters: 
#      Contest - The contest to collect data for. 1=RC5, 2=DES, etc.
#
#  Return Value: None.
#
#  Description:
#    Write the main team stats by year
#
#--- End Method Prolog -------------------------------------------------------
sub write_byyear
{
    my $Contest = shift @_;
    &Debugln("Entering write_byyear($Contest) ...");
    my ($year, $mon, $mday, $hour, $min, $sec, $wday);
    my ($sql, $best, $numdays);
    my ($x, $y, $z);
    my $month;
    my $sth;
    my $blocks;
    my ($date_month, $date_year);
    my ($prev, $curr);
    my ($rate, $width, $antiwidth);
    my $line;

    open(OUTFILE, ">" . $UserPrefs{"htmldir"} . $SystemPrefs{"contestsuffix"}[$Contest] . "/" . $UserPrefs{"htmlyear"}) or die "Personal Proxy Statistics : Cannot open " . $UserPrefs{"htmldir"} . $SystemPrefs{"contestsuffix"}[$Contest] . "/" . $UserPrefs{"htmlyear"} . " ($!)\n";
    &write_header($Contest);
    print OUTFILE "     <TABLE BORDER=0 CELLPADDING=0 CELLSPACING=0 WIDTH=\"100%\">\n",
                  "      <TR ALIGN=CENTER>\n",
                  "       <TD CLASS=section NOWRAP>Year</TD>\n",
                  "       <TD CLASS=section WIDTH=3 NOWRAP>&nbsp;</TD>\n",
                  "       <TD CLASS=section NOWRAP>" . $SystemPrefs{"contestheaderblocktype"}[$Contest] . "s</TD>\n",
                  "       <TD CLASS=section WIDTH=3 NOWRAP>&nbsp;</TD>\n",
                  "       <TD CLASS=section WIDTH=\"100%\" NOWRAP>Graphical Rate in " . $SystemPrefs{"contestheaderblocktype"}[$Contest] . "s</TD>\n",
                  "       <TD CLASS=section WIDTH=3 NOWRAP>&nbsp;</TD>\n",
                  "       <TD CLASS=section NOWRAP>" . $SystemPrefs{"contestheaderblockmeasure"}[$Contest] . " Rate</TD>\n",
                  "      </TR>\n";

    $sql = "SELECT UNIX_TIMESTAMP(MAX(SUBMITTED_DATE)) FROM " . $SystemPrefs{"contestwutable"}[$Contest];
    my ($LastSubmitted) = &DLookup($sql);
    $sql = "SELECT UNIX_TIMESTAMP(MIN(SUBMITTED_DATE)) FROM " . $SystemPrefs{"contestwutable"}[$Contest];
    my ($FirstSubmitted) =&DLookup($sql);
    my $totaldays = int((($LastSubmitted - $FirstSubmitted) / (60*60*24)) + 1);

    # Calculate best year
    $best = 0;
    $z = 0;
    ($year, $mon, $mday, $hour, $min, $sec, $wday) = &nice_gmtime($SystemPrefs{"timebegin"});
    $prev = (($year+1) . "-1-1");
    my (@row, @dates_year, @yearblocks);
    
    $sql = "SELECT YEAR(SUBMITTED_DATE), SUM(WU_SIZE) FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " GROUP BY 1 ORDER BY 1 DESC";
    $sth = &SelectSQL($sql);
    while (@row = $sth->fetchrow_array()) 
    {
	$date_year = $row[0];
	$blocks = $row[1];
        if ($blocks > $best) 
        {
	    $best = $blocks;
	}
	$z++;
	$sql = "SELECT DATE_SUB(\"$prev\", INTERVAL 1 YEAR)";
	($prev) = &DLookup($sql);
        ($year, $month, $mday) = split(/-/, $prev);
	while ( int($year) != $date_year)
        {
    	    $dates_year[$z] = $year;
            $yearblocks[$z] =0;
            $z++;
	    $sql = "SELECT DATE_SUB(\"$prev\", INTERVAL 1 YEAR)";
	    ($prev) = &DLookup($sql); 
	    ($year, $month, $mday) = split(/-/, $prev);
        }
        $dates_year[$z] = $date_year;
	$prev = ($date_year . "-1-1");
	$yearblocks[$z] = $blocks;
    }

    # Calculate past rates
    if ($z > 1) 
    {
	if ($best == 0) 
	{
	    $width = 100;
	} 
	else 
	{
	    $width = int($yearblocks[1] * 100 / $best);
	}
	$antiwidth = 100 - $width;
#	($year, $mon, $mday, $hour, $min, $sec, $wday) = &nice_gmtime($SystemPrefs{"timebegin"} - 1 * (24*60*60));

	$sql="SELECT DAYOFYEAR(FROM_UNIXTIME(" . $SystemPrefs{"timebegin"} . "))";
	($numdays)=&DLookup($sql);

	$rate = &ratef(&rate_day($yearblocks[1], $SystemPrefs{"contestfactor"}[$Contest], $Contest) / $numdays); 
	$curr =  $dates_year[1];
	$line = "      <TR>\n";
	$line .= "       <TD NOWRAP ALIGN=RIGHT>$curr</TD>\n";
	$line .= "       <TD WIDTH=3 NOWRAP>&nbsp;</TD>\n";
	if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
	{
	    $line .= "       <TD NOWRAP ALIGN=RIGHT>" . $yearblocks[1] . "</TD>\n";
	}
	else
	{
	    $line .= "       <TD NOWRAP ALIGN=RIGHT>" . &ratef($yearblocks[1] * $SystemPrefs{"contestfactor"}[$Contest]) . "</TD>\n";
	}
	$line .= "       <TD WIDTH=3 NOWRAP>&nbsp;</TD>\n";
	$line .= "       <TD WIDTH=\"100%\" NOWRAP><TABLE WIDTH=\"100%\" BORDER=0 CELLSPACING=0 CELLPADDING=0><TR><TD CLASS=history WIDTH=\"$width%\">&nbsp;</TD><TD WIDTH=\"$antiwidth%\">&nbsp;</TD></TR></TABLE></TD>\n";
	$line .= "       <TD WIDTH=3 NOWRAP>&nbsp;</TD>\n";
	$line .= "       <TD NOWRAP ALIGN=RIGHT>$rate" . $SystemPrefs{"contestblockmeasure"}[$Contest] . "s/sec</TD>\n";
	$line .= "      </TR>\n";
	print OUTFILE "$line";
    }
    if ($z > 2)
    {
	for ($y = 2; $y < $z; $y++) 
	{
	    $curr = $dates_year[$y];
	
	    if ($best > 0) 
	    {
		$width = int($yearblocks[$y] * 100 / $best);
	    }
	    else 
	    {
		$width = 0;
	    }
	    $antiwidth = 100 - $width;
	    $sql="SELECT DAYOFYEAR(DATE_SUB(DATE_ADD(\"" . $dates_year[$y] . "-1-1\", INTERVAL 1 YEAR), INTERVAL 1 DAY))";
	    ($numdays)=&DLookup($sql);
	    $rate = &ratef(&rate_day($yearblocks[$y], $SystemPrefs{"contestfactor"}[$Contest], $Contest) / $numdays);
	    $line = "      <TR>\n";
	    $line .= "       <TD NOWRAP ALIGN=RIGHT>$curr</TD>\n";
	    $line .= "       <TD WIDTH=3 NOWRAP>&nbsp;</TD>\n";
	    if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
	    {
		$line .= "       <TD NOWRAP ALIGN=RIGHT>" . $yearblocks[$y] . "</TD>\n";
	    }
	    else
	    {
		$line .= "       <TD NOWRAP ALIGN=RIGHT>" . &ratef($yearblocks[$y] * $SystemPrefs{"contestfactor"}[$Contest]) . "</TD>\n";
	    }
	    $line .= "       <TD NOWRAP ALIGN=RIGHT>" . $yearblocks[$y] . "</TD>\n";
	    $line .= "       <TD WIDTH=3 NOWRAP>&nbsp;</TD>\n";
	    $line .= "       <TD WIDTH=\"100%\" NOWRAP><TABLE WIDTH=\"100%\" BORDER=0 CELLSPACING=0 CELLPADDING=0><TR><TD CLASS=history WIDTH=\"$width%\">&nbsp;</TD><TD WIDTH=\"$antiwidth%\">&nbsp;</TD></TR></TABLE></TD>\n";
	    $line .= "       <TD WIDTH=3 NOWRAP>&nbsp;</TD>\n";
	    $line .= "       <TD NOWRAP ALIGN=RIGHT>$rate" . $SystemPrefs{"contestblockmeasure"}[$Contest] . "s/sec</TD>\n";
	    $line .= "      </TR>\n";
	    print OUTFILE "$line";
	}
    }

    if ($z >= 1)
    {
	$sql="SELECT UNIX_TIMESTAMP(DATE_SUB(DATE_ADD(\"" . $dates_year[$z] . "-1-1\", INTERVAL 1 YEAR), INTERVAL 1 DAY))";
	my ($endofyear) = &DLookup($sql);
	if ($SystemPrefs{"timebegin"} < $endofyear)
	{
	    $endofyear = $SystemPrefs{"timebegin"};	
	}
	$sql="SELECT DAYOFYEAR(FROM_UNIXTIME(\"$endofyear\"))";
	($numdays) = &DLookup($sql);
	$sql="SELECT DAYOFYEAR(FROM_UNIXTIME(\"$FirstSubmitted\"))";
	my ($startday) = &DLookup($sql);
    	$numdays = $numdays - $startday +1;
	if ($best == 0) 
	{
	    $width = 100;
	} else {
	    $width = int($yearblocks[$z] * 100 / $best);
	}
	$antiwidth = 100 - $width;
	$rate = &ratef(&rate_day($yearblocks[$z], $SystemPrefs{"contestfactor"}[$Contest], $Contest) / $numdays); 
	$curr =  $dates_year[$z];
	$line = "      <TR>\n";
	$line .= "       <TD NOWRAP ALIGN=RIGHT>$curr</TD>\n";
	$line .= "       <TD WIDTH=3 NOWRAP>&nbsp;</TD>\n";
	if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
	{
	    $line .= "       <TD NOWRAP ALIGN=RIGHT>" . $yearblocks[$z] . "</TD>\n";
	}
	else
	{
	    $line .= "       <TD NOWRAP ALIGN=RIGHT>" . &ratef($yearblocks[$z] * $SystemPrefs{"contestfactor"}[$Contest]) . "</TD>\n";
	}
	$line .= "       <TD WIDTH=3 NOWRAP>&nbsp;</TD>\n";
	$line .= "       <TD WIDTH=\"100%\" NOWRAP><TABLE WIDTH=\"100%\" BORDER=0 CELLSPACING=0 CELLPADDING=0><TR><TD CLASS=history WIDTH=\"$width%\">&nbsp;</TD><TD WIDTH=\"$antiwidth%\">&nbsp;</TD></TR></TABLE></TD>\n";
	$line .= "       <TD WIDTH=3 NOWRAP>&nbsp;</TD>\n";
	$line .= "       <TD NOWRAP ALIGN=RIGHT>$rate" . $SystemPrefs{"contestblockmeasure"}[$Contest] . "s/sec</TD>\n";
	$line .= "      </TR>\n";
	print OUTFILE "$line";
    }



    print OUTFILE "     </TABLE>\n";
    &write_footer($Contest);
    close OUTFILE;

    &Debugln("Exiting write_byyear($Contest) ...");
}  # write_byyear



#--- Method Prolog -----------------------------------------------------------
#  Name: write_bydayofweek
#
#  Parameters: 
#      Contest - The contest to collect data for. 1=RC5, 2=DES, etc.
#
#  Return Value: None.
#
#  Description:
#    Write the main team stats by day of week
#
#--- End Method Prolog -------------------------------------------------------
sub write_bydayofweek
{
    my $Contest = shift @_;
    &Debugln("Entering write_bydayofweek($Contest) ...");
    my ($year, $mon, $mday, $hour, $min, $sec, $wday);
    my ($sql, $best, $numdays);
    my ($x, $y, $z);
    my $numblocks;
    my $sth;
    my $blocks;
    my $date;
    my ($prev, $curr);
    my ($rate, $width, $antiwidth);
    my $line;

    open(OUTFILE, ">" . $UserPrefs{"htmldir"} . $SystemPrefs{"contestsuffix"}[$Contest] . "/" . $UserPrefs{"htmldayofweek"}) or die "Personal Proxy Statistics : Cannot open " . $UserPrefs{"htmldir"} . $SystemPrefs{"contestsuffix"}[$Contest] . "/" . $UserPrefs{"htmldayofweek"} . " ($!)\n";
    &write_header($Contest);
    print OUTFILE "     <TABLE BORDER=0 CELLPADDING=0 CELLSPACING=0 WIDTH=\"100%\">\n",
                  "      <TR ALIGN=CENTER>\n",
                  "       <TD CLASS=section NOWRAP>Day of Week</TD>\n",
                  "       <TD CLASS=section WIDTH=3 NOWRAP>&nbsp;</TD>\n",
                  "       <TD CLASS=section NOWRAP>Days</TD>\n",
                  "       <TD CLASS=section WIDTH=3 NOWRAP>&nbsp;</TD>\n",
                  "       <TD CLASS=section NOWRAP>" . $SystemPrefs{"contestheaderblocktype"}[$Contest] . "s</TD>\n",
                  "       <TD CLASS=section WIDTH=3 NOWRAP>&nbsp;</TD>\n",
                  "       <TD CLASS=section WIDTH=\"100%\" NOWRAP>Graphical Rate in " . $SystemPrefs{"contestheaderblocktype"}[$Contest] . "s</TD>\n",
                  "       <TD CLASS=section WIDTH=3 NOWRAP>&nbsp;</TD>\n",
                  "       <TD CLASS=section NOWRAP>" . $SystemPrefs{"contestheaderblockmeasure"}[$Contest] . " Rate</TD>\n",
                  "      </TR>\n";

    $sql = "SELECT UNIX_TIMESTAMP(MAX(SUBMITTED_DATE)) FROM " . $SystemPrefs{"contestwutable"}[$Contest];
    my ($LastSubmitted) = &DLookup($sql);
    $sql = "SELECT UNIX_TIMESTAMP(MIN(SUBMITTED_DATE)) FROM " . $SystemPrefs{"contestwutable"}[$Contest];
    my ($FirstSubmitted) =&DLookup($sql);
    my $totaldays = int((($LastSubmitted - $FirstSubmitted) / (60*60*24)) + 1);

    # Calculate best day of week
    $best = 0;
    $z = 0;
    $prev = $SystemPrefs{"timebegin"};
    my (@row, @dayofweek, @dates);
    
    $sql = "SELECT UNIX_TIMESTAMP(SUBMITTED_DATE), SUM(WU_SIZE) FROM " . $SystemPrefs{"contestwutable"}[$Contest] . " GROUP BY 1 ORDER BY 1 DESC ";
    $sth = &SelectSQL($sql);
    while (@row = $sth->fetchrow_array()) 
    {
	($date) = $row[0];
	($blocks) = $row[1];
	($year, $mon, $mday, $hour, $min, $sec, $wday) = &nice_gmtime($date);
	$dayofweek[$wday] += $blocks;
	$dates[$wday]++;

	while ($prev > ($date + (60*60*24)))
	{
	    $prev = $prev - (60*60*24);
	    ($year, $mon, $mday, $hour, $min, $sec, $wday) = &nice_gmtime($prev);
	    $dates[$wday]++;
	}
	$prev = $date;
    }
    for ($y = 0; $y <= 6; $y++)
    {
	if ($best < $dayofweek[$y])
	{
	    $best = $dayofweek[$y];
	}
    }    

    # Calculate past rates
    for ($y = 0; $y <= 6; $y++)
    {
	if ($best == 0) 
	{
	    $width = 100;
	} 
	else 
	{
	    $width = int($dayofweek[$y] * 100 / $best);
	}
	$antiwidth = 100 - $width;
	($year, $mon, $mday, $hour, $min, $sec, $wday) = &nice_gmtime($SystemPrefs{"timebegin"} - 1 * (24*60*60));
	if ($dates[$y] > 0)
	{
	    $rate = &ratef(&rate_day($dayofweek[$y], $SystemPrefs{"contestfactor"}[$Contest], $Contest) / $dates[$y]); 
	}
	else
	{
	    $rate = &ratef(0);
	}
	$curr = $Days[$y];
	$line = "      <TR>\n";
	$line .= "       <TD NOWRAP ALIGN=RIGHT>$curr</TD>\n";
	$line .= "       <TD WIDTH=3 NOWRAP>&nbsp;</TD>\n";
	$line .= "       <TD NOWRAP ALIGN=RIGHT>" . $dates[$y] . "</TD>\n";
	$line .= "       <TD WIDTH=3 NOWRAP>&nbsp;</TD>\n";
	if ($SystemPrefs{"contestblockbase"}[$Contest] > 1)
	{
	    $line .= "       <TD NOWRAP ALIGN=RIGHT>" . $dayofweek[$y] . "</TD>\n";
	}
	else
	{
	    $line .= "       <TD NOWRAP ALIGN=RIGHT>" . &ratef($dayofweek[$y] * $SystemPrefs{"contestfactor"}[$Contest]) . "</TD>\n";
	}
	$line .= "       <TD WIDTH=3 NOWRAP>&nbsp;</TD>\n";
	$line .= "       <TD WIDTH=\"100%\" NOWRAP><TABLE WIDTH=\"100%\" BORDER=0 CELLSPACING=0 CELLPADDING=0><TR><TD CLASS=history WIDTH=\"$width%\">&nbsp;</TD><TD WIDTH=\"$antiwidth%\">&nbsp;</TD></TR></TABLE></TD>\n";
	$line .= "       <TD WIDTH=3 NOWRAP>&nbsp;</TD>\n";
	$line .= "       <TD NOWRAP ALIGN=RIGHT>$rate" . $SystemPrefs{"contestblockmeasure"}[$Contest] . "s/sec</TD>\n";
	$line .= "      </TR>\n";
	print OUTFILE "$line";
    }

    print OUTFILE "     </TABLE>\n";
    &write_footer($Contest);
    close OUTFILE;

    &Debugln("Exiting write_bydayofweek($Contest) ...");
}  # write_bydoyofweek





#--- Method Prolog -----------------------------------------------------------
#  Name: plot_gui_graph
#
#  Parameters: 
#      path - workpath
#      title - title within image
#      xmin - minimum on x-axis (date)
#      xmax - maximum on x-axis (date)
#      range - maximium on y-axis (blocks, can be * (autodetect))
#      averate - value for overall average line (from left to right)
#      Contest - The contest to collect data for. 1=RC5, 2=DES, etc.
#
#  Return Value: None.
#
#  Description:
#    Subroutine to create GNUplot config file and system call to execute it.
#
#--- End Method Prolog -------------------------------------------------------
sub plot_gui_graph
{
    my ($path, $title, $xmin, $xmax, $range, $averate, $Contest) = @_;
    my $line;
    my $measure;
    my $sql;
    
    open(GNUPLOTCONFIG, ">$path" . $UserPrefs{"gnuplotconfig"}) or die "Personal Proxy Statistics : Cannot open $path" . $UserPrefs{"gnuplotconfig"} . " ($!)\n";

    print GNUPLOTCONFIG "set title \"$title\\n\"\n";

    $measure = &ratef($SystemPrefs{"contestfactor"}[$Contest] -1);
    $measure = substr($measure, index($measure, "</STRONG>") -1, 1);
    
    if ("$xmin" == "")
    {
	$xmin = $xmax;
    }
    if ("$xmin" eq "$xmax") 
    {
	$sql = "SELECT DATE_FORMAT(DATE_ADD(\"$xmax 00:00:00\", INTERVAL 1 DAY), \"%Y/%m/%d\")";
	($xmax) = &DLookup($sql);
    }

    if ($UserPrefs{"chartlinestyle"} == 1) {
	print GNUPLOTCONFIG "set data style impulses\n";
    } elsif ($UserPrefs{"chartlinestyle"} == 2) {
	print GNUPLOTCONFIG "set data style lines\n";
    } elsif ($UserPrefs{"chartlinestyle"} == 3) {
	print GNUPLOTCONFIG "set data style linespoints\n";
    } else {
	print GNUPLOTCONFIG "set data style impulses\n";
    }

    if ($UserPrefs{"imagetype"} == 1) #Write GNUPLOTCONFIG for PNG-pictures
    {
       my $screenwidth = $UserPrefs{"chartwidth"} /  $SystemPrefs{"pngdefaultwidth"}; # Calculate imagesize multiplier (width)
       my $screenheight = $UserPrefs{"chartheight"} / $SystemPrefs{"pngdefaultheight"}; # Calculate imagesize multiplier (height)    
       print GNUPLOTCONFIG "set key top left\n",                        
       "set timefmt '%Y/%m/%d'\n",                        
       "set xdata time\n",                        
       "set xrange ['$xmin':'$xmax'] noreverse nowriteback\n",                        
       "set format x '%m/%d/%Y'\n",                        
       "set xtics rotate\n",                        
       "set ylabel 'Rate ($measure" . $SystemPrefs{"contestblockmeasure"}[$Contest] . "s/sec)'\n",                        
       "set yrange [0:$range] noreverse nowriteback\n",                        
       "set grid\n",#                        
       "set bmargin 6\n",  # bottom margin =6 for old gif gnuplot; =8 for png gnuplot (on SuSE)                        
       "set bmargin 8\n",			
       "set size $screenwidth,$screenheight\n",                        
       "set terminal png small color\n",                        
       "set output '$path" . $UserPrefs{"gnuplotoutput"} . "'\n";    

       $line = "plot '$path" . $SystemPrefs{"gnuplotdata"} . "' using 1:2 title 'daily', $averate title 'overall'";    
       if ($UserPrefs{"dayaverage"} > 1) 
       {	
          $line .= ", '$path" . $SystemPrefs{"gnuplotdata"} . "' using 1:3 title '" . $UserPrefs{"dayaverage"} . " day'";    
       }
    }
    else #Write GNUPLOTCONFIG for GIF-pictures
    {    
       print GNUPLOTCONFIG "set key top left\n",                        
       "set timefmt '%Y/%m/%d'\n",                        
       "set xdata time\n",                        
       "set xrange ['$xmin':'$xmax'] noreverse nowriteback\n",                        
       "set format x '%m/%d/%Y'\n",                        
       "set xtics rotate\n",                        
       "set ylabel 'Rate ($measure" . $SystemPrefs{"contestblockmeasure"}[$Contest] . "s/sec)'\n",                        
       "set yrange [0:$range] noreverse nowriteback\n",                        
       "set grid\n",                        
       "set bmargin 6\n",                        
       "set terminal gif transparent interlace size " . $UserPrefs{"chartwidth"} . "," . $UserPrefs{"chartheight"} . " x" . $UserPrefs{"backgroundcolor"} . " x" . $UserPrefs{"textcolor"} . " x" . $UserPrefs{"textcolor"} . " x" . $UserPrefs{"historycolor"} . " x" . $UserPrefs{"avehistorycolor"} . " x" . $UserPrefs{"ndayhistorycolor"} . "\n",
       "set output '$path" . $UserPrefs{"gnuplotoutput"} . "'\n";    

       $line = "plot '$path" . $SystemPrefs{"gnuplotdata"} . "' using 1:2 title 'daily', $averate title 'overall'";    
       if ($UserPrefs{"dayaverage"} > 1) 
       {
        $line .= ", '$path" . $SystemPrefs{"gnuplotdata"} . "' using 1:3 title '" . $UserPrefs{"dayaverage"} . " day'";
       }
    }

    print GNUPLOTCONFIG "$line\n";

    close GNUPLOTCONFIG;

    system($UserPrefs{"gnuplot"}, "$path" . $UserPrefs{"gnuplotconfig"}) == 0 or die "Personal Proxy Statistics : Cannot execute " . $UserPrefs{"gnuplot"} . " $path" . $UserPrefs{"gnuplotconfig"} . " ($?)\n";
}  # plot_gui_graph


###############################################################################################################
###############################################################################################################
#Unuse old stuff (until now...)

#-----------------------------------------------------------------------------
#
# Write zeros to the nth-day average array, initializing the array.
#
#-----------------------------------------------------------------------------
#sub clear_nday_ave {
#    my $x;
#
#    for ($x = 0; $x < $UserPrefs{"dayaverage"}; $x++) { $nday_ave[$x] = 0; }
#}  # clear_nday_ave


#-----------------------------------------------------------------------------
#
# Shift all values of the nth-day average down and insert the new rate.
#
#-----------------------------------------------------------------------------
#sub push_nday_ave {
#    my $x;
#    my $rate = shift @_;
#
#    if ($UserPrefs{"dayaverage"} > 1) {
#	for ($x = 1; $x < $UserPrefs{"dayaverage"}; $x++) { $nday_ave[$x - 1] = $nday_ave[$x]; }
#	$nday_ave[$Userprefs{"dayaverage"} - 1] = $rate;
#    }
#}  # push_nday_ave


#-----------------------------------------------------------------------------
#
# Calculate the nth-day average.
#
#-----------------------------------------------------------------------------
#sub calc_nday_ave {
#    my $x;
#    my $sum = 0;
#    
#    if ($UserPrefs{"dayaverage"} < 2) { return 0; }
#
#    for ($x = 0; $x < $UserPrefs{"dayaverage"}; $x++) { $sum += $nday_ave[$x]; }
#    return sprintf("%.2f", $sum / $UserPrefs{"dayaverage"});
#}  # calc_nday_ave

###############################################################################################################
###############################################################################################################


#--- Method Prolog -----------------------------------------------------------
#  Name: calculate_average
#
#  Parameters: 
#      blocks - Blocks/Workunits 
#      percision - percisiion after "."
#      Contest - The contest to collect data for. 1=RC5, 2=DES, etc.
#
#  Return Value: 
#      averageblocks - Formated average blocks for the last $UserPrefs{"dayaverage"} days ;-)
#
#  Description:
#    Return formated average blocks for the last $UserPrefs{"dayaverage"} days (default 21 days)
#
#--- End Method Prolog -------------------------------------------------------
sub calculate_average
{
    my $blocks = shift @_;
    my $percision = shift @_;
    my $Contest = shift @_;
    my ($x, $y);
    my $allblocks = 0;
    my $days = 0;
    my $averageblocks;
    
    for ($x = 0; $x <= $UserPrefs{"dayaverage"}; $x++)
    {
        if ($x < $UserPrefs{"dayaverage"})
        {
	    if (defined($Average[$x]))
	    {
		$days++;
		$allblocks +=$Average[$x];
	    }
    	    else
	    {
		$Average[$x] = $blocks;
		$days++;
		$allblocks = $allblocks + $blocks;
	    }
	}
	else
	{
	    $allblocks = $allblocks + $blocks;
	    $allblocks = $allblocks - $Average[0];
	    for ($y = 1; $y < $UserPrefs{"dayaverage"}; $y++)
	    {
		$Average[$y-1] = $Average[$y]; 
	    }
	    $Average[$x-1] = $blocks;
	}
    }
    if ($days > 0)
    {
	$averageblocks = ($allblocks / $days);
    }
    else
    {
	$averageblocks = 0;
    }

    return &rate_day($averageblocks,$SystemPrefs{"contestfactor"}[$Contest], $Contest);
}


#--- Method Prolog -----------------------------------------------------------
#  Name: clear_average
#
#  Parameters: None.
#
#  Return Value: None.
#
#  Description:
#    Clear "global" average array
#
#--- End Method Prolog -------------------------------------------------------
sub clear_average
{
    @Average = undef;    
}



#--- Method Prolog -----------------------------------------------------------
#  Name: percentf
#
#  Parameters: 
#      blocks - Blocks/Workunits 
#      percision - percision in numbers after the "." (sorry, how do I write this in "correct" english?) ;-)
#      Contest - The contest to collect data for. 1=RC5, 2=DES, etc.
#
#  Return Value: None.
#
#  Description:
#    Return a formatted percent given the number of blocks against the whole blocks for this contest
#
#--- End Method Prolog -------------------------------------------------------
sub percentf
{
    my $blocks = shift @_;
    my $percision = shift @_;
    my $Contest = shift @_;

    if (!defined($blocks)) { $blocks = 0; }
    if (!defined($percision)) { $percision = 0; }

    return sprintf("%." . $percision . "f", 100 * $blocks / &get_contest_blocks($Contest));    
}  # percentf


#--- Method Prolog -----------------------------------------------------------
#  Name: get_contest_blocks
#
#  Parameters: 
#      Contest - The contest to collect data for. 1=RC5, 2=DES, etc.
#
#  Return Value: None.
#
#  Description:
#    Return the total number of blocks in this contest.
#
#--- End Method Prolog -------------------------------------------------------
sub get_contest_blocks
{
    my $Contest = shift @_;
    return sprintf("%.0f", 2**($SystemPrefs{"contestsize"}[$Contest] - $SystemPrefs{"contestblockbase"}[$Contest] - $SystemPrefs{"contestblockweight"}[$Contest] + 1));
}  # get_contest_blocks


#--- Method Prolog -----------------------------------------------------------
#  Name: ls
#
#  Parameters: 
#      lastseentime - in seconds
#
#  Return Value: 
#      lastseen - string 
#
#  Description:
#    Returns formated last seen time. 
#
#--- End Method Prolog -------------------------------------------------------
sub ls
{
    my $lastseentime = shift @_;
    my $lastseen;
    
    # Print out the Last Seen column.
    if ($lastseentime >= 365*24*60*60) {
	$lastseen = int($lastseentime / (365*24*60*60)) . "+ years";
    } elsif ($lastseentime >= 2*24*60*60) {
	$lastseen = int($lastseentime / (24*60*60)) . " days";
    } elsif ($lastseentime >= 2*60*60) {
	$lastseen = int($lastseentime / (60*60)) . " hours";
    } elsif ($lastseentime >= 2*60) {
	$lastseen = int($lastseentime / 60) . " mins";
    } elsif ($lastseentime >= 0) {
	$lastseen = $lastseentime . " secs";	    
    } else {
	$lastseen = "0 sec";	    
    }

    return $lastseen;
}  # ls


#--- Method Prolog -----------------------------------------------------------
#  Name: ls_color
#
#  Parameters: 
#      lastseentime - in seconds
#
#  Return Value: 
#      lastseencolor - string with name of color (normal, critical, warning)
#
#  Description:
#    Returns the appropriate color class given a last seen time. 
#
#--- End Method Prolog -------------------------------------------------------
sub ls_color
{
    my $lastseentime = shift @_;
    my $lastseencolor = "normal";

    if ($UserPrefs{"colorlastseen"}) {
	if ($lastseentime >= $UserPrefs{"lastseencriticaltime"}) {
	    $lastseencolor = "critical";
	} elsif ($lastseentime >= $UserPrefs{"lastseenwarningtime"}) {
	    $lastseencolor = "warning";
	}
    }

    return $lastseencolor;
}  # ls_color

#--- Method Prolog -----------------------------------------------------------
#  Name: keysf
#
#  Parameters:
#      blocks - number of blocks/workunits 
#      Contest - The contest to collect data for. 1=RC5, 2=DES, etc.
#
#  Return Value: 
#      keys - string with number of keys
#
#  Description:
#    Return formatted number of keys checked given blocks done.
#
#--- End Method Prolog -------------------------------------------------------
sub keysf
{
    my $blocks = shift @_;
    my $Contest = shift @_;
    my $keys;

    if (!defined($blocks)) { $blocks = 0; }

    $keys = $blocks * (2**($SystemPrefs{"contestblockbase"}[$Contest] + $SystemPrefs{"contestblockweight"}[$Contest] - 1));

    if ($keys <= 0) {
	$keys = "None";
    } elsif ($keys < 10**6) {
	$keys = sprintf("%.2f", $keys / 10**3) . " Thousand";
    } elsif ($keys < 10**9) {
	$keys = sprintf("%.2f", $keys / 10**6) . " Million";
    } elsif ($keys < 10**12) {
	$keys = sprintf("%.2f", $keys / 10**9) . " Billion";
    } elsif ($keys < 10**15) {
	$keys = sprintf("%.2f", $keys / 10**12) . " Trillion";
    } elsif ($keys < 10**18) {
	$keys = sprintf("%.2f", $keys / 10**15) . " Quadrillion";
    } elsif ($keys < 10**21) {
	$keys = sprintf("%.2f", $keys / 10**18) . " Quintillion";
    } elsif ($keys < 10**24) {
	$keys = sprintf("%.2f", $keys / 10**21) . " Sextillion";
    } else {
	$keys = sprintf("%.2f", $keys / 10**24) . " Septillion";
    }

    return $keys;
}  # keysf





#--- Method Prolog -----------------------------------------------------------
#  Name: ResolveHosts
#
#  Parameters: None.
#
#  Return Value: None.
#
#  Description:
#    Resolve all host IP's into hostnames. Hostnames that have been resolved
#    are cached in the TBL_HOST table for a user defined length of time,
#    generally 60 days (the default).
#
#--- End Method Prolog -------------------------------------------------------
sub ResolveHosts
{
    &Debugln("Entering ResolveHosts() ...");

    my $ip;
    my ($byear, $bmonth, $bmday, $bhour, $bmin, $bsec) = @GMTTS;
    my $domain;
    my $sql;
    my ($date, $time, $year, $month, $mday, $hour, $min, $sec);

    foreach $ip (keys %Hosts) {
	my $resolve = 0;
	
	#--This IP needs to be resolved if it has never been updated, or if
	#--it has reached the maximum threshold for cache time.
	if (!defined($Hosts{$ip}{"updated"})) 
	{
	    $resolve = 1;
	} 
	elsif ($Hosts{$ip}{"updated"} eq "0000-00-00 00:00:00")
	{
	    $resolve = 0;
	}
	else
	{
    	    ($date, $time) = split(/ /, $Hosts{$ip}{"updated"});
    	    ($year, $month, $mday) = split(/-/, $date);
    	    ($hour, $min, $sec) = split(/:/, $time);
	    $sec = substr($sec, 0, 2);  #--Don't need the extra timezone at the end.
	    if ($month > 0)
	    {
		$month -= 1;  #--The month we need for timegm is 0-11, not 1-12 as displayed.
	    	my $threshold = timegm($sec, $min, $hour, $mday, $month, $year) + ($UserPrefs{"cachedays"} * 24*60*60);
	    	if ($threshold < $SystemPrefs{"timebegin"}) 
		{
		    $resolve = 1;
		}
	    }
	}

	#--Lookup and store hostname if required.
	if ($resolve) {
	    my $host = gethostbyaddr((pack "C4", split(/\./, $Hosts{$ip}{"ip"})), 2);
	    if (!defined($host)) 
	    { 
		$host = $ip; 
		$domain = undef;
	    }
	    else
	    {
		if ($host =~ /\.(\w+)$/) 
		{ 
		    $domain = $1; 
		}
		else
		{
		    $domain = undef;
		}
	    }
	    $Hosts{$ip}{"hostname"} = $host;
	    if (defined($domain))
	    {
    		$sql = "UPDATE TBL_HOST SET HOSTNAME='$host', UPDATED='$byear-$bmonth-$bmday $bhour:$bmin:$bsec', COUNTRY_ID='$domain' WHERE IP='$ip'";
	    }
	    else
	    {
    		$sql = "UPDATE TBL_HOST SET HOSTNAME='$host', UPDATED='$byear-$bmonth-$bmday $bhour:$bmin:$bsec' WHERE IP='$ip'";
	    }
	    &RunSQL($sql);
	    &Debug(".");
	}
    }

    &Debugln("Exiting ResolveHosts() ...");
}  #--ResolveHosts


#--- Method Prolog -----------------------------------------------------------
#  Name: LoadHosts
#
#  Parameters: None.
#
#  Return Value: None.
#
#  Description:
#    Read in all host information from the database into an Hosts 
#    hash. Using this hash should speed up script run time.
#
#--- End Method Prolog -------------------------------------------------------
sub LoadHosts
{
    my @row;
    
    my $count = 0;

    &Debugln("Entering LoadHosts() ...");

    #--Retrieve the last day logs were recorded in the database.
    my $sql = "SELECT HOST_ID, IP, NETMASK, HOSTNAME, DATE_FORMAT(UPDATED, \"%Y-%m-%d %H:%i:%s\") FROM TBL_HOST";
    my $sth = &SelectSQL($sql);
    while (@row = $sth->fetchrow_array()) {
	$Hosts{$row[1]}{"host_id"} = $row[0];	
	$Hosts{$row[1]}{"ip"} = $row[1];
	$Hosts{$row[1]}{"netmask"} = $row[2];	
	$Hosts{$row[1]}{"hostname"} = $row[3];	
	$Hosts{$row[1]}{"updated"} = $row[4];

	my ($ipa, $ipb, $ipc, $ipd) = split (/\./, $Hosts{$row[1]}{"ip"});
	$Hosts{$row[1]}{"ipa"} = sprintf "%08d", &dec2bin($ipa);
	$Hosts{$row[1]}{"ipb"} = sprintf "%08d", &dec2bin($ipb);
	$Hosts{$row[1]}{"ipc"} = sprintf "%08d", &dec2bin($ipc);
	$Hosts{$row[1]}{"ipd"} = sprintf "%08d", &dec2bin($ipd);
	my ($maska, $maskb, $maskc, $maskd) = split (/\./, $Hosts{$row[1]}{"netmask"});
	$Hosts{$row[1]}{"maska"} = sprintf "%08d", &dec2bin($maska);
	$Hosts{$row[1]}{"maskb"} = sprintf "%08d", &dec2bin($maskb);
	$Hosts{$row[1]}{"maskc"} = sprintf "%08d", &dec2bin($maskc);
	$Hosts{$row[1]}{"maskd"} = sprintf "%08d", &dec2bin($maskd);

	$count++;
    }
    
    
    &Debugln($count . " host records have been loaded.");
}  #--LoadHosts


#--- Method Prolog -----------------------------------------------------------
#  Name: dec2bin
#
#  Parameters:
#    (shift) - decimal value
#
#  Return Value: 
#    str - value as 32 bit binaery
#
#  Description:
#    Converts value from decimal to binaery
#
#--- End Method Prolog -------------------------------------------------------
sub dec2bin
{
    my $str = unpack("B32", pack("N", shift));
#    $str =~ s/^0+(?=\d)//;
    return $str;
}  #--dec2bin

#--- Method Prolog -----------------------------------------------------------
#  Name: InsertHost
#
#  Parameters:
#    host - The new host address to insert into the database.
#
#  Return Value: None.
#
#  Description:
#    Insert the given host into the database.
#
#--- End Method Prolog -------------------------------------------------------
sub InsertHost
{
    my $host = shift @_;
    my $sql;
    my $host_id;

    $sql = "SELECT MAX(HOST_ID) AS MAX_HOST_ID FROM TBL_HOST";
    ($host_id) = &DLookup($sql);
    if (defined($host_id)) {
	$host_id++;
    } else {
	$host_id = 1;
    }

    #--Insert the new email into the database.
    $sql = "INSERT INTO TBL_HOST (HOST_ID, IP, NETMASK) VALUES ($host_id, '$host', \"255.255.255.255\")";
    &RunSQL($sql);

    #--Insert the new email address into the dynamic email hash.
    $Hosts{$host}{"host_id"} = $host_id;
    $Hosts{$host}{"ip"} = $host;
}  #--InsertHost


#--- Method Prolog -----------------------------------------------------------
#  Name: LoadEmails
#
#  Parameters: None.
#
#  Return Value: None.
#
#  Description:
#    Read in all participant information from the database into an Emails 
#    hash. Using this hash should speed up script run time.
#
#--- End Method Prolog -------------------------------------------------------
sub LoadEmails
{
    my @row;
    my $count = 0;

    &Debugln("Entering LoadEmails() ...");

    #--Retrieve the last day logs were recorded in the database.
    my $sql = "SELECT PARTICIPANT_ID, EMAIL, DNET_ID, FNAME, LNAME, NICKNAME, DOB, HOMEPAGE, COUNTRY_ID, MEMBER, RETIRE_TO_ID FROM TBL_PARTICIPANT";
    my $sth = &SelectSQL($sql);
    while (@row = $sth->fetchrow_array()) {
	$Emails{$row[1]}{"participant_id"} = $row[0];	
	$Emails{$row[1]}{"email"} = lc($row[1]);	
	$Emails{$row[1]}{"dnet_id"} = $row[2];	
	$Emails{$row[1]}{"fname"} = $row[3];	
	$Emails{$row[1]}{"lname"} = $row[4];	
	$Emails{$row[1]}{"nickname"} = $row[5];	
	$Emails{$row[1]}{"dob"} = $row[6];	
	$Emails{$row[1]}{"homepage"} = $row[7];	
	$Emails{$row[1]}{"country_id"} = $row[8];	
	$Emails{$row[1]}{"member"} = $row[9];	
	$Emails{$row[1]}{"retire_to_id"} = $row[10];	
	$count++;
    }

    &Debugln($count . " email records have been loaded.");
}  #--LoadEmails


#--- Method Prolog -----------------------------------------------------------
#  Name: InsertEmail
#
#  Parameters:
#    email - The new email address to insert into the database.
#
#  Return Value: None.
#
#  Description:
#    Inser the given email address into the database.
#
#--- End Method Prolog -------------------------------------------------------
sub InsertEmail
{
    my $email = shift @_;
    my $sql;
    my $participant_id;
    my $country_id;

    $email = lc($email);

    $sql = "SELECT MAX(PARTICIPANT_ID) AS MAX_PARTICIPANT_ID FROM TBL_PARTICIPANT";
    ($participant_id) = &DLookup($sql);
    if (defined($participant_id)) {
	$participant_id++;
    } else {
	$participant_id = 1;
    }

    #--Grab up to 4 characters of the top level email domain.
    if ($email =~ /\.(\w+)$/) { $country_id = $1; }    
    $country_id = substr($country_id, 0, 4);

    #--Insert the new email into the database.
    $sql = "INSERT INTO TBL_PARTICIPANT (PARTICIPANT_ID, EMAIL, COUNTRY_ID, MEMBER, RETIRE_TO_ID) VALUES ($participant_id, '$email', '$country_id', 1, 0)";
    &RunSQL($sql);

    #--Insert the new email address into the dynamic email hash.
    $Emails{$email}{"participant_id"} = $participant_id;
    $Emails{$email}{"email"} = $email;
    $Emails{$email}{"country_id"} = $country_id;
    $Emails{$email}{"member"} = "t";
}  #--InsertEmail


#--- Method Prolog -----------------------------------------------------------
#  Name: SetTimestamp
#
#  Parameters: None.
#
#  Return Value: None.
#
#  Description:
#    Save the time the script was started. Two global arrays are stored, one
#    based on the user defined timezone, and the other GMT timezone.
#
#--- End Method Prolog -------------------------------------------------------
sub SetTimestamp
{ 
    my ($year, $month, $mday, $hour, $min, $sec, $wday, $tz);

    #--Timestamp begins after we have collected all the data.
    $SystemPrefs{"timebegin"} = time;

    #--Calculate user supplied time zone timestamp.
    ($year, $month, $mday, $hour, $min, $sec, $wday, $tz) = &UserTime($SystemPrefs{"timebegin"});
    $SystemPrefs{"timestamp"} = "$mday $Months[$month-1] $year $hour:$min " . $UserPrefs{"timezone"};
    @UserTS = ($year, $month, $mday, $hour, $min, $sec);

    #--Calculate GMT timestamp.
    ($year, $month, $mday, $hour, $min, $sec, $wday, $tz) = &GMTTime($SystemPrefs{"timebegin"});
    @GMTTS = ($year, $month, $mday, $hour, $min, $sec);

    #--Calculate standard variables for today.
    $SystemPrefs{"secstoday"} = $hour * 3600 + $min * 60 + $sec;
    if ($SystemPrefs{"secstoday"} == 0) { $SystemPrefs{"secstoday"} = 1; }
}  #--SetTimestamp


#--- Method Prolog -----------------------------------------------------------
#  Name: SelectSQL
#
#  Parameters:
#    sql - The preformatted aggregate SQL statement to run.
#
#  Return Value:
#    sth - The resulting statement handle. Access the data in the recordset
#          with a command like: while (@row = $sth->fetchrow_array()) { ... }
#
#  Description:
#    Executes the specified SQL statement and return the results. This
#    function is for SELECT statements only. All other commands (such as
#    INSERT, DELETE, UPDATE)  should use RunSQL instead.
#
#--- End Method Prolog -------------------------------------------------------
sub SelectSQL
{
    my $sql = shift @_;
    if ($SystemPrefs{"sqldebugenable"} > 0)
    {
	&Debugln("$sql");
    }
    my $sth = $SystemPrefs{"dbh"}->prepare($sql);
    $sth->execute() or die $DBI::errstr;

    return $sth;
}  #--SelectSQL


#--- Method Prolog -----------------------------------------------------------
#  Name: DLookup
#
#  Parameters: None.
#
#  Return Value: None.
#
#  Description:
#    This domain lookup provides a quick way to run a simple SQL statement 
#    that returns a single row of data.
#
#--- End Method Prolog -------------------------------------------------------
sub DLookup
{
    my $sql = shift @_;
    my $sth = &SelectSQL($sql);
    my @row = $sth->fetchrow_array();
    $sth->finish();
    return @row;
}  #--DLookup


#--- Method Prolog -----------------------------------------------------------
#  Name: RunSQL
#
#  Parameters:
#    sql - The preformatted aggregate SQL statement to run.
#
#  Return Value: None.
#
#  Description:
#    Executes the specified SQL data manipulation statement. The command
#    is executed and there should be no return value. SQL statements such as
#    INSERT, DELETE, and UPDATE should use this function, while SELECT
#    statements should call SelectSQL instead.
#
#--- End Method Prolog -------------------------------------------------------
sub RunSQL
{
    my $sql = shift @_;
    my $sth;
    if ($SystemPrefs{"sqldebugenable"} > 0)
    {
	&Debugln("$sql");
    }
    $sth = $SystemPrefs{"dbh"}->prepare($sql);
    $sth->execute() or die $DBI::errstr;
    $sth->finish();   
}  #--RunSQL


#--- Method Prolog -----------------------------------------------------------
#  Name: ConnectDB
#
#  Parameters:
#    db - The name of the database to establish a connection to.
#
#  Return Value: None.
#
#  Description:
#    Connect to the database and create the database handle. Die otherwise.
#
#--- End Method Prolog -------------------------------------------------------
sub ConnectDB
{
    my $db = shift @_;

    &Debugln("Connecting to the database '$db' ...");
    
    my $ip = "";
    my $port = "";
    
    if ($UserPrefs{"dbip"} ne "socket")
    {
	$ip = $UserPrefs{"dbip"};
	$port = $UserPrefs{"dbport"};
    }
    
    $SystemPrefs{"dbh"} = DBI->connect(
        "dbi:mysql:dbname=" . $db . ":" . $ip . ":" . $port , 
        "ppstats",
        "12345",
	{ AutoCommit => 1, RaiseError => 0 }
    );
    die $DBI::errstr unless defined($SystemPrefs{"dbh"});

    #--Set the default time zone for all datetime data types to GMT.
#    &RunSQL("SET TIMEZONE TO 'GMT'");
#    &RunSQL("SET DATESTYLE TO 'ISO'");

    &Debugln("Connected to database.");    
}  #--ConnectDB


#--- Method Prolog -----------------------------------------------------------
#  Name: DisconnectDB
#
#  Parameters: None.
#
#  Return Value: None.
#
#  Description:
#    Disconnect from the database.
#
#--- End Method Prolog -------------------------------------------------------
sub DisconnectDB
{
    $SystemPrefs{"dbh"}->disconnect();
    &Debugln("Disconnected from the database.");       	
}  #--DisconnectDB


#--- Method Prolog -----------------------------------------------------------
#  Name: SetTimeZoneDB
#
#  Parameters:
#    zone - The time zone abbreviation. See %TimeZones for complete listing.
#
#  Return Value: None.
#
#  Description:
#    Sets the default PostgreSQL database time zone to the specified value.
#    All date/time calculations will be reported in this format.
#
#--- End Method Prolog -------------------------------------------------------
sub SetTimeZoneDB
{
    my $zone = shift @_;
    my $sql;

    if (!defined($TimeZones{$zone})) {
	die $SystemPrefs{"appname"} . ": Time zone '$zone' not recognized.\n";
    }

    $sql = "SET TIMEZONE TO '$zone'";
    &RunSQL($sql);
}  #--SetTimeZoneDB


#--- Method Prolog -----------------------------------------------------------
#  Name: SetDateStyleDB
#
#  Parameters:
#    style - The date style type.
#
#  Return Value: None.
#
#  Description:
#    Set the given datetime style. When datetime fields are returned, they
#    will be in this format. ISO format is 'YYYY-MM-DD HH:MM:SS+TZ'
#
#--- End Method Prolog -------------------------------------------------------
sub SetDateStyleDB
{
    my $style = shift @_;
    my $sql;

    $sql = "SET DATESTYLE TO '$style'";
    &RunSQL($sql);
}  #--SetDateStyleDB


#--- Method Prolog -----------------------------------------------------------
#  Name: VacuumDB
#
#  Parameters: None.
#
#  Return Value: None.
#
#  Description:
#    Execute a vacuume analyze statement on the database. Since PostgreSQL
#    doesn't automatically record statistics, this statement should be run
#    after the database changes in a significant way. Vacuuming the database
#    calculates statistics to make queries run faster, and cleans out any
#    data that has been marked as deleted.
#
#--- End Method Prolog -------------------------------------------------------
sub VacuumDB
{
    &Debugln("Executing 'vacuum analyze' on the database ...");
#    &RunSQL("VACUUM ANALYZE");

    &RunSQL("OPTIMIZE TABLE `TBL_WU_RC564`");
    &RunSQL("OPTIMIZE TABLE `TBL_WU_CSC`");
    &RunSQL("OPTIMIZE TABLE `TBL_WU_OGR`");
    &RunSQL("OPTIMIZE TABLE `TBL_WU_DES`");

    &RunSQL("OPTIMIZE TABLE `TBL_COUNTRY`");
    &RunSQL("OPTIMIZE TABLE `TBL_OS`");
    &RunSQL("OPTIMIZE TABLE `TBL_CPU`");
    &RunSQL("OPTIMIZE TABLE `TBL_HOST`");
    &RunSQL("OPTIMIZE TABLE `TBL_PARTICIPANT`");

    &Debugln("Finished 'vacuum analyze'.");
}  #--SetTimeZoneDB


#--- Method Prolog -----------------------------------------------------------
#  Name: GMTTime
#
#  Parameters: None.
#
#  Return Value:
#    Array of (year,mon,mday,hour,min,sec,wday,tz).
#
#  Description:
#    Get preformatted GMT array (year,month,dayofmonth,hour,min,sec,weekday,tz).
#
#--- End Method Prolog -------------------------------------------------------
sub GMTTime
{
    my @gmt = gmtime shift @_;
    my $i;

    $gmt[4]++;
    $gmt[5] += 1900;
    for ($i = 0; $i < 5; $i++) {
        while (length($gmt[$i]) < 2) {
            $gmt[$i] = "0" . $gmt[$i];
        }
    }

    return ($gmt[5], $gmt[4], $gmt[3], $gmt[2], $gmt[1], $gmt[0], $gmt[6], "+00");
}  #--GMTTime


#--- Method Prolog -----------------------------------------------------------
#  Name: UserTime
#
#  Parameters: None.
#
#  Return Value:
#    Array of (year,mon,mday,hour,min,sec,wday,tz).
#
#  Description:
#    Get preformatted user supplied timezone array 
#    (year,month,dayofmonth,hour,min,sec,weekday,tz).
#
#--- End Method Prolog -------------------------------------------------------
sub UserTime
{
    my @gmt = gmtime ((shift @_) + ($TimeZones{$UserPrefs{"timezone"}}*60*60));
    my $i;

    $gmt[4]++;
    $gmt[5] += 1900;
    for ($i = 0; $i < 5; $i++) {
        while (length($gmt[$i]) < 2) {
            $gmt[$i] = "0" . $gmt[$i];
        }
    }

    return ($gmt[5], $gmt[4], $gmt[3], $gmt[2], $gmt[1], $gmt[0], $gmt[6], $TimeZones{$UserPrefs{"timezone"}});
}  #--UserTime


#--- Method Prolog -----------------------------------------------------------
#  Name: ParseCommandLine
#
#  Parameters: None.
#
#  Return Value: None.
#
#  Description:
#    ParseCommandLine takes a list of possible options and a boolean 
#    indicating whether the option has a value following, and sets up an 
#    associative array %opt of the values of the options given on the command 
#    line. It removes all the arguments it uses from @ARGV and returns them 
#    in @optArgs.
#
#--- End Method Prolog -------------------------------------------------------
sub ParseCommandLine
{
    my @optval = @_;
    my ($opt, @opts, %valFollows, @newargs, $arg);

    while (@optval) {
	$opt = shift(@optval);
	push(@opts, $opt);
	$valFollows{$opt} = shift(@optval);
    }

    @optArgs = ();
    %opt = ();

    arg: while ($arg = shift(@ARGV)) {
	foreach $opt (@opts) {
	    if ($arg eq $opt) {
		push(@optArgs, $arg);
		if ($valFollows{$opt}) {
		    if (@ARGV == 0) {
			&DisplayUsage();
		    }
		    $opt{$opt} = shift(@ARGV);
		    push(@optArgs, $opt{$opt});
		} else {
		    $opt{$opt} = 1;
		}
		next arg;
	    }
	}
	push(@newargs, $arg);
    }

    @ARGV = @newargs;
}  #--ParseCommandLine


#--- Method Prolog -----------------------------------------------------------
#  Name: Debugln
#
#  Parameters:
#    msg - String message to display to standard error.
#
#  Return Value: None.
#
#  Description:
#    If debug flag is turned on, print out debug, message and line break
#    to standard error. 
#
#--- End Method Prolog -------------------------------------------------------
sub Debugln
{
    my $msg = shift @_;

    if ($SystemPrefs{"debugenable"} == 1) {
	if ($msg ne "") { print STDERR "debug: $msg\n"; }
	else { print STDERR "\n"; }
    }
}  #--Debugln


#--- Method Prolog -----------------------------------------------------------
#  Name: Debug
#
#  Parameters:
#    msg - String message to display to standard error.
#
#  Return Value: None.
#
#  Description:
#    If debug flag is turned on, print out message to standard error.
#
#--- End Method Prolog -------------------------------------------------------
sub Debug
{
    my $msg = shift @_;

    if ($SystemPrefs{"debugenable"} == 1 && $msg ne "") { print STDERR "$msg"; }
}  #--Debug


#--- Method Prolog -----------------------------------------------------------
#  Name: EnableDebugMessages
#
#  Parameters: None.
#
#  Return Value: None.
#
#  Description:
#    Turn on system debuging.
#
#--- End Method Prolog -------------------------------------------------------
sub EnableDebugMessages
{
    print "\n" . $SystemPrefs{"appname"} . " v" . $SystemPrefs{"appver"} . "\n",
	  "Debug is enabled. Runtime information will be sent to STDERR.\n\n";
    $SystemPrefs{"debugenable"} = 1;
}  #--EnableDebugMessages


#--- Method Prolog -----------------------------------------------------------
#  Name: EnableSQLDebugMessages
#
#  Parameters: None.
#
#  Return Value: None.
#
#  Description:
#    Turn on SQL debuging.
#
#--- End Method Prolog -------------------------------------------------------
sub EnableSQLDebugMessages
{
    print "\n" . $SystemPrefs{"appname"} . " v" . $SystemPrefs{"appver"} . "\n",
	  "SQL-Debug is enabled. SQL-Calls will be sent to STDERR.\n\n";
    $SystemPrefs{"sqldebugenable"} = 1;
    $SystemPrefs{"debugenable"} = 1;
}  #--EnableSQLDebugMessages


#--- Method Prolog -----------------------------------------------------------
#  Name: DisplayUsage
#
#  Parameters: None.
#
#  Return Value: None.
#
#  Description:
#    Display program name and options.
#
#--- End Method Prolog -------------------------------------------------------
sub DisplayUsage
{
    my $Contest;
    
    print "\n" . $SystemPrefs{"appname"} . " v" . $SystemPrefs{"appver"} . "\n",
	  "Usage: " . $SystemPrefs{"app"} . " [OPTIONS]...\n",
          "Perl routines for processing Distributed.Net personal proxy log files.\n\n";
    for ($Contest = 1; $Contest <= $SystemPrefs{"projects"}; $Contest++)
    {
	print  "  -" . $SystemPrefs{"contestsuffix"}[$Contest] . "      Disables processing and html creation of data from Project " . $SystemPrefs{"contestsuffix"}[$Contest] . "\n";
	print  "  +" . $SystemPrefs{"contestsuffix"}[$Contest] . "      Enables processing and html creation of data from Project " . $SystemPrefs{"contestsuffix"}[$Contest] . "\n";
    }
    print "  -debug     Display debug information while script executes.\n",
	  "  -debugsql  Displays ALL SQL-Queries while script executes.\n",
          "  -version   Display version and copyright information and exit.\n",
          "  -help      Print this help and exit.\n\n";
    exit;
}  #--DisplayUsage


